#!/bin/sh

# get the command line args
iter=$1
flag=$2

pbsdir=/data/sep/myfusan/BP2007_covan/PBS/
bindir=/data/sep/myfusan/BP2007_covan/Bin/
pardir=/data/sep/myfusan/BP2007_covan/Par/
datdir=/data/sep/myfusan/BP2007_covan/Shots/
inpdir=/data/sep/myfusan/BP2007_covan/Input/
imgdir=/data/sep/myfusan/BP2007_covan/Img/
moddir=/data/sep/myfusan/BP2007_covan/Mod/
grddir=/data/sep/myfusan/BP2007_covan/Grd/
scrdir=/data/sep/myfusan/BP2007_covan/Scripts/
objdir=/data/sep/myfusan/BP2007_covan/Fobj/
wrkdir=/data/sep/myfusan/BP2007_covan/

# ===============================================================
# define the useful files
svelname=${grddir}/svel_${flag}_${iter}.H
sepsname=${grddir}/seps_${flag}_${iter}.H
sdelname=${grddir}/sdel_${flag}_${iter}.H

bvelname=${moddir}/vp.${flag}_${iter}.H
bepsname=${moddir}/eps.${flag}_${iter}.H
bdelname=${moddir}/del.${flag}_${iter}.H

stplname=${wrkdir}/stplnth_${flag}

iter_in=1
flag_in=lnsh
# try Three different step lengthes
for alfa in 0.1 0.5 2
do
  tvelname=${moddir}/vp.${flag}_${flag_in}_${iter_in}.H
  tepsname=${moddir}/eps.${flag}_${flag_in}_${iter_in}.H
  tdelname=${moddir}/del.${flag}_${flag_in}_${iter_in}.H
  Math file1=${bvelname} file2=${svelname} exp="file1+file2*${alfa}" > ${tvelname}
  Math file1=${bepsname} file2=${sepsname} exp="file1+file2*${alfa}" > ${tepsname}
  Math file1=${bdelname} file2=${sdelname} exp="file1+file2*${alfa}" > ${tdelname}
  ${scrdir}/migration1.sh ${iter_in} ${flag_in} ${flag}
  ${scrdir}/residual1.sh ${iter_in} ${flag_in} ${flag}
  ((iter_in=iter_in+1))
done

# do a quardratic fit to search the step length  
f1=`Disfil number=n format='%13.8g' < ${objdir}/fobj_${flag}_${flag_in}_1.H`
f2=`Disfil number=n format='%13.8g' < ${objdir}/fobj_${flag}_${flag_in}_2.H`
f3=`Disfil number=n format='%13.8g' < ${objdir}/fobj_${flag}_${flag_in}_3.H`

a1=0.1
a2=0.5
a3=2

up=`echo "(${a2}*${a2}-${a3}*${a3})*${f1} + (${a3}*${a3}-${a1}*${a1})*${f2} + (${a1}*${a1}-${a2}*${a2})*${f3}" | bc -l`
down=`echo "2*((${a2}-${a3})*${f1} + (${a3}-${a1})*${f2} + (${a1}-${a2})*${f3})" | bc -l`
alfa_min=`echo "${up} ${down}" | awk '{printf "%f", $1 / $2}'`

if [ $( echo "${alfa_min} > 0" | bc ) -eq 1 ];
then 
  # compute the objective function from the minimum
  tvelname=${moddir}/vp.${flag}_${flag_in}_${iter_in}.H
  tepsname=${moddir}/eps.${flag}_${flag_in}_${iter_in}.H
  tdelname=${moddir}/del.${flag}_${flag_in}_${iter_in}.H
  Math file1=${bvelname} file2=${svelname} exp="file1+file2*${alfa_min}" > ${tvelname}
  Math file1=${bepsname} file2=${sepsname} exp="file1+file2*${alfa_min}" > ${tepsname}
  Math file1=${bdelname} file2=${sdelname} exp="file1+file2*${alfa_min}" > ${tdelname}
  ${bindir}/BoundAniso.x eps=${tepsname} del=${tdelname}
  ${scrdir}/migration1.sh ${iter_in} ${flag_in} ${flag}
  ${scrdir}/residual1.sh ${iter_in} ${flag_in} ${flag}
  f4=`Disfil number=n format='%13.8g' < ${objdir}/fobj_${flag}_${flag_in}_4.H`
else
  f4=100000000
fi

f0=`Disfil number=n format='%13.8g' < ${objdir}/fobj_${flag}_${iter}.H`

((iter_n=iter+1))
nvelname=${moddir}/vp.${flag}_${iter_n}.H
nepsname=${moddir}/eps.${flag}_${iter_n}.H
ndelname=${moddir}/del.${flag}_${iter_n}.H
# check if the objective function decrease
if [ $( echo "${f4} < ${f0}" | bc ) -eq 1 ];
then
  iter_in=4
  echo ${alfa_min} >> ${stplname}
else
  if [ $( echo "${f1} < ${f2}" | bc ) -eq 1 ] && [ $( echo "${f1} < ${f3}" | bc ) -eq 1 ]
  then
    iter_in=1
    echo ${a1} >> ${stplname}
  fi
  if [ $( echo "${f2} < ${f1}" | bc ) -eq 1 ] && [ $( echo "${f2} < ${f3}" | bc ) -eq 1 ]
  then
    iter_in=2
    echo ${a2} >> ${stplname}
  fi
  if [ $( echo "${f3} < ${f1}" | bc ) -eq 1 ] && [ $( echo "${f3} < ${f2}" | bc ) -eq 1 ]
  then
    iter_in=3
    echo ${a3} >> ${stplname}
  fi
fi

${scrdir}/update1.sh ${flag_in} ${iter_in} ${flag} ${iter_n}

pyRemove.py ${moddir}/*${flag}*${flag_in}*.H
pyRemove.py ${imgdir}/*${flag}*${flag_in}*.H
pyRemove.py ${objdir}/*${flag}*${flag_in}*.H
