program GetClay 

  use sep

  implicit none

  real, allocatable                     :: vel(:,:,:), poro(:,:,:), peff(:,:,:)
  real, allocatable                     :: pclay(:,:,:)
  integer                               :: n1out, n2out
  real                                  :: d1, o1, d2, o2, d3, o3
  integer                               :: n1, n2, n3, i1, i2, i3
  character(20)                         :: label1, label2, label3
  real									:: vsalt
  integer								:: method 
  real									:: clay_content


  call sep_init()

  ! Get data dimensions from command line.
  call from_history("n1",n1)	           
  call from_history("n2",n2)
  call from_history("n3",n3)
  call from_history("o1",o1)
  call from_history("o2",o2)
  call from_history("o3",o3)
  call from_history("d1",d1)
  call from_history("d2",d2)
  call from_history("d3",d3)

  ! get the salt velocity from command line
  call from_param("vsalt",vsalt,4490.)

  call from_param("method",method,1)

  allocate(vel(n1, n2, n3),poro(n1,n2,n3),peff(n1,n2,n3))
  allocate(pclay(n1,n2,n3))

  call sep_read(vel)
  call sreed("poro",poro,4*n1*n2*n3)
  call sreed("peff",peff,4*n1*n2*n3)

  do i3=1,n3
		  do i2=1,n2
				  do i1=1,n1
						pclay(i1,i2,i3) = clay_content(vel(i1,i2,i3),poro(i1,i2,i3),peff(i1,i2,i3),method)
				  end do
		  end do
  end do

  call to_history("n1",n1)             
  call to_history("n2",n2)
  call to_history("n3",n3)
  call to_history("o1",o1)
  call to_history("o2",o2)
  call to_history("o3",o3)
  call to_history("d1",d1)
  call to_history("d2",d2)
  call to_history("d3",d3)
  call to_history("esize", 4)

  call sep_write(pclay)
  deallocate(vel,poro,peff,pclay)

end program 

real function clay_content(vel,poro,peff,method) 
  real 				:: vel, poro, peff, pclay
  real 				:: tmp
  integer		 	:: method


  vel = vel / 1000.; 			! convert from m/s to km/s
  peff = peff / 1000.;			! convert from bar to kbar

  if (method == 1) then
    tmp = vp - 0.446*(peff - exp(-16.7*peff)) - 5.77 + 6.94*poro;
    tmp = tmp / 1.73;
    clay_content = tmp * tmp
  else if (method == 2) then
	tmp = 5.8 - vp - 8.6*poro;
	clay_content = tmp / 2.4;
  else if (method == 3) then
    tmp = 5.81 - vp - 9.42*poro
	clay_content = tmp / 2.21
  end if	

  return
end function



