program GetTemp 

  use sep

  implicit none

  real, allocatable                     :: vel(:,:,:)
  integer                               :: n1out, n2out
  real                                  :: d1, o1, d2, o2, d3, o3
  integer                               :: n1, n2, n3, i1, i2, i3
  character(20)                         :: label1, label2, label3
  real					:: tgrad, depth, tsurf 
  integer                      :: ompchunk, nth
  integer  ,          external :: omp_get_num_procs
  integer  ,          external :: omp_get_num_threads
  integer  ,          external :: omp_set_num_threads
  integer  ,          external :: omp_get_thread_num

  call sep_init()

  ! Get data dimensions from command line.
  call from_history("n1",n1)
  call from_history("n2",n2)
  call from_history("n3",n3)   ! depth in km
  call from_history("o1",o1)
  call from_history("o2",o2)
  call from_history("o3",o3)
  call from_history("d1",d1)
  call from_history("d2",d2)
  call from_history("d3",d3)

  ! get the surface temp and thermol gradient from command line 
  call from_param("tgrad", tgrad, 25.)
  call from_param("tsurf", tsurf, 14.)

  allocate(vel(n1, n2, n3))

!   setup OpenMP stuff
!   ------------------
    ompchunk = 1
    nth      = 1

    !$OMP PARALLEL
    if (omp_get_thread_num() == 0) then
       nth = omp_get_num_threads()
       write(0,*)"nthread =", nth, omp_get_num_procs()
    end if
    !$OMP END PARALLEL

  !$OMP PARALLEL DO SCHEDULE(DYNAMIC,OMPCHUNK) DEFAULT(SHARED) FIRSTPRIVATE(i1) 
  do i1=1,n1
	depth = (i1-1)*d1 + o1
	depth = depth/0.3048;
	vel(i1,:,:) = tsurf + depth*tgrad 
  end do
  !$OMP END PARALLEL DO

  call to_history("n1",n1)             
  call to_history("n2",n2)
  call to_history("n3",n3)
  call to_history("o1",o1)
  call to_history("o2",o2)
  call to_history("o3",o3)
  call to_history("d1",d1)
  call to_history("d2",d2)
  call to_history("d3",d3)
  call to_history("esize", 4)

  call sep_write(vel)
  deallocate(vel)

end program 
