program OdfAvg 

  use sep

  implicit none

  real, allocatable                     :: poro(:,:,:),vshale(:,:,:),pillite(:,:,:)
  real, allocatable                     :: Aeps(:,:,:),Adel(:,:,:),Avel(:,:,:)
  integer                               :: n1out, n2out
  real                                  :: d1, o1, d2, o2, d3, o3
  integer                               :: n1, n2, n3, phi_n, i, iphi
  integer								:: i1, i2, i3, iw
  character(20)                         :: label1, label2, label3
  real, allocatable						:: W200(:), W400(:)
  real									:: W200_max, W400_max, W000
  real									:: phi_c, phi_d, phi
  real									:: W200_exp, W400_exp, pi=3.1415
  real,dimension(3,6)					:: Cij=0.
  real,dimension(3)						:: vp=0., vs=0., eps=0., del=0., gam=0., rho=0.
  real,dimension(2)						:: a1=0., a2=0., a3=0., la=0., mu=0.
  real,dimension(3)						:: aC11=0., aC33=0., aC12=0., aC13=0., aC44=0., aC66=0.
  real									:: C11, C12, C13, C33, C44, pcl, pil, pqu, aa, bb, cc
  real									:: M, Nu, den
  logical								:: allign

  integer                      :: ompchunk, nth
  integer  ,          external :: omp_get_num_procs
  integer  ,          external :: omp_get_num_threads
  integer  ,          external :: omp_set_num_threads
  integer  ,          external :: omp_get_thread_num

  call sep_init()

  ! Get data dimensions from command line.
  call from_history("n1",n1)	           
  call from_history("n2",n2)
  call from_history("n3",n3)
  call from_history("o1",o1)
  call from_history("o2",o2)
  call from_history("o3",o3)
  call from_history("d1",d1)
  call from_history("d2",d2)
  call from_history("d3",d3)

  ! get the allignment switch
  call from_param("allign", allign, .true.)

  ! get the critical porosity from the command line 
  call from_param("poro_c",phi_c,0.55)
  call from_param("poro_d",phi_d,0.01)
  phi_n = phi_c / phi_d + 1;

  allocate(poro(n1,n2,n3),vshale(n1,n2,n3),pillite(n1,n2,n3))
  call sep_read(poro)
  where(poro > phi_c) poro=phi_c
  call sreed("vshale" , vshale, 4*n1*n2*n3)
  call sreed("pillite",pillite, 4*n1*n2*n3)
  where(vshale>1) vshale=1

  ! allocate space for output
  allocate(Aeps(n1,n2,n3),Adel(n1,n2,n3),Avel(n1,n2,n3))

  ! Initialize the mineral property
  ! clay, illite, and quartz
  rho = (/2.4,   2.4,  2.65/);       ! g/cc 
  vp  = (/3.075, 4.94, 6.0 /);       ! km/s
  vs  = (/1.5,   2.6,  4.0 /);       ! km/s
  eps = (/0.255, 1.04, 0.   /);      
  del = (/-0.05,  0.0,  0.   /);
  gam = (/0.48, 1.68,  0.   /);

!  rho = (/2.4,   2.4,  2.65/);       ! g/cc 
!  vp  = (/3.075, 4.94, 6.0 /);       ! km/s
!  vs  = (/1.5,   2.6,  4.0 /);       ! km/s
!  eps = (/0., 0, 0   /);      
!  del = (/0., 0, 0   /);
!  gam = (/0., 0, 0   /);

  ! compute the five elastic constants for each mineral
  ! weak anisotropy assumption implied for C13
  ! C11, C33, C13, C44, C66, C12
  do i = 1,3
	M  = rho(i) * vp(i)**2;
	Nu = rho(i) * vs(i)**2;
    Cij(i,1) = M*(1+2*eps(i));
	Cij(i,2) = M;
	Cij(i,3) = M*del(i) + M - 2*Nu;
	Cij(i,4) = Nu;
	Cij(i,5) = Nu * (1+2*gam(i));
	Cij(i,6) = Cij(i,1) - 2*Cij(i,5);
  end do
  aC11(3) = Cij(3,1) 
  aC33(3) = Cij(3,2) 
  aC13(3) = Cij(3,3) 
  aC44(3) = Cij(3,4) 
  aC66(3) = Cij(3,5) 
  aC12(3) = Cij(3,6) 

  ! compute the parameters for averaging
  ! only for clay and illite
  ! C11, C33, C13, C44, C66, C12
  a1=0;a2=0;a3=0;la=0;mu=0;
  do i = 1,2
	a1(i) = Cij(i,1) + Cij(i,2) - 2*Cij(i,3) - 4*Cij(i,4)	  
	a2(i) = Cij(i,1) - 3*Cij(i,6) + 2*Cij(i,3) - 2*Cij(i,4)	  
	a3(i) = 4*Cij(i,1) - 3*Cij(i,2) - Cij(i,3) - 2*Cij(i,4)	  
	la(i) = 1./15 * (Cij(i,1)+Cij(i,2)+5*Cij(i,6)+8*Cij(i,3)-4*Cij(i,4))
	mu(i) = 1./30 * (7*Cij(i,1)+2*Cij(i,2)-5*Cij(i,6)-4*Cij(i,3)+12*Cij(i,4))
  end do	

  ! compute the orientation distribution function
  call from_param("W200_exp",W200_exp,1.)
  call from_param("W400_exp",W400_exp,1.)
  allocate(W200(phi_n),W400(phi_n))
  W000	   = sqrt(0.5) / (4*pi*pi);
  W200_max = sqrt(2.5) / (4*pi*pi);
  W400_max = sqrt(4.5) / (4*pi*pi);
  do i=1,phi_n
	phi = (i-1)*phi_d	  
	W200(i) = W200_max*(1-phi/phi_c)**W200_exp;	  
	W400(i) = W400_max*(1-phi/phi_c)**W400_exp;	  
  end do

  ! compute the average stiffness tensor for clay and illite 
  ! according to the porosity cube

  do i3=1,n3
	do i2=1,n2
	  do i1=1,n1
		phi = poro(i1,i2,i3)
		iphi = phi / phi_d + 1
		! get the average stiffness tensor
		do i = 1,2
		  if(allign) then
		     aC11(i) = la(i) + 2*mu(i) + 4 *sqrt(2.)/105*pi*pi* (2*sqrt(5.)*a3(i)*W200(iphi) +  3*a1(i)*W400(iphi))  	
		     aC33(i) = la(i) + 2*mu(i) - 16*sqrt(2.)/105*pi*pi* (  sqrt(5.)*a3(i)*W200(iphi) -  2*a1(i)*W400(iphi))  	
		     aC12(i) = la(i) - 4*sqrt(2.)/315*pi*pi* (2*sqrt(5.)*( 7*a2(i)-a3(i))*W200(iphi) -  3*a1(i)*W400(iphi))  	
		     aC13(i) = la(i) + 4*sqrt(2.)/315*pi*pi* (  sqrt(5.)*( 7*a2(i)-a3(i))*W200(iphi) - 12*a1(i)*W400(iphi))  	
		     aC44(i) = mu(i) - 2*sqrt(2.)/315*pi*pi* (  sqrt(5.)*( 7*a2(i)+a3(i))*W200(iphi) + 24*a1(i)*W400(iphi))  	
		     aC66(i) = (aC11(i) - aC12(i))/2. 
		  else 
			  aC11(i) = Cij(i,1) 
		 	  aC33(i) = Cij(i,2) 
			  aC13(i) = Cij(i,3) 
			  aC44(i) = Cij(i,4) 
			  aC66(i) = Cij(i,5) 
			  aC12(i) = Cij(i,6) 
		  end if
		end do  

		! Backus average over clay and illite 
		pil = vshale(i1,i2,i3)*Pillite(i1,i2,i3);
		pcl = vshale(i1,i2,i3)*(1-Pillite(i1,i2,i3));
		pqu = 1 - vshale(i1,i2,i3);
		den = (pqu * rho(3) + pcl * rho(1) + pil * rho(2))*(1-phi) + phi
		aa  = pcl * aC13(1)/aC33(1) + pil * aC13(2)/aC33(2) + pqu * aC13(3)/aC33(3)
		bb  = pcl * aC13(1)*aC13(1)/aC33(1) + pil * aC13(2)*aC13(2)/aC33(2) + pqu * aC13(3)*aC13(3)/aC33(3)
		cc  = pcl * aC11(1) + pil * aC11(2) + pqu * aC11(3)
		C33 = 1./ (pcl / aC33(1) + pil / aC33(2) + pqu / aC33(3));
		C13 = aa * C33;
		C11 = aa * aa * C33 - bb + cc 
		C44 = 1./ (pcl / aC44(1) + pil / aC44(2) + pqu / aC44(3));
		C12 = C11 - cc + pcl * aC12(1) + pil * aC12(2) + pqu * aC12(3);
		Aeps(i1,i2,i3) = (C11-C33)/C33/2
		Adel(i1,i2,i3) = 0.1+(((C13+C44)**2-(C33-C44)**2)/(2*C33*(C33-C44)))
		Avel(i1,i2,i3) = sqrt(C33/den) 
      end do
    end do
  end do

  call to_history("n1",n1)             
  call to_history("n2",n2)
  call to_history("n3",n3)
  call to_history("o1",o1)
  call to_history("o2",o2)
  call to_history("o3",o3)
  call to_history("d1",d1)
  call to_history("d2",d2)
  call to_history("d3",d3)
  call to_history("esize", 4)

  call to_history("n1",n1,"Adel")             
  call to_history("n2",n2,"Adel")
  call to_history("n3",n3,"Adel")
  call to_history("o1",o1,"Adel")
  call to_history("o2",o2,"Adel")
  call to_history("o3",o3,"Adel")
  call to_history("d1",d1,"Adel")
  call to_history("d2",d2,"Adel")
  call to_history("d3",d3,"Adel")
  call to_history("esize", 4,"Adel")
  call to_history("label2", "location(km)","Adel")
  call to_history("label1", "depth(km)","Adel")

  call to_history("n1",n1,"Avel")             
  call to_history("n2",n2,"Avel")
  call to_history("n3",n3,"Avel")
  call to_history("o1",o1,"Avel")
  call to_history("o2",o2,"Avel")
  call to_history("o3",o3,"Avel")
  call to_history("d1",d1,"Avel")
  call to_history("d2",d2,"Avel")
  call to_history("d3",d3,"Avel")
  call to_history("esize", 4,"Avel")
  call to_history("label2", "in-line (m)","Avel")
  call to_history("label1", "depth (kft)","Avel")

  call sep_write(Aeps)
  call srite("Adel",Adel,4*n1*n2*n3)
  call srite("Avel",Avel,4*n1*n2*n3)

  deallocate(poro,vshale,pillite)
  deallocate(Aeps,Adel,Avel)

end program 


