#define NULL            0
/* allocate a 1-d array */
void *alloc1 (int n1, int size)
{
	void *p;

	if ((p=(void *)malloc(n1*size))==NULL)
		return NULL;
	return p;
}

/* free a 1-d array */
void free1 (void *p)
{
	free(p);
}

/* allocate a 2-d array */
void **alloc2 (int n1, int n2, int size)
{
	int i2;
	void **p;

	if ((p=(void**)malloc(n2*sizeof(void*)))==NULL) 
		return NULL;
	if ((p[0]=(void*)malloc(n2*n1*size))==NULL) {
		free(p);
		return NULL;
	}
	for (i2=0; i2<n2; i2++)
		p[i2] = (char*)p[0]+size*n1*i2;
	return p;
}

/* free a 2-d array */
void free2 (void **p)
{
	free(p[0]);
	free(p);
}

/* allocate a 3-d array */
void ***alloc3 (int n1, int n2, int n3, int size)
{
	int i3,i2;
	void ***p;

	if ((p=(void***)malloc(n3*sizeof(void**)))==NULL)
		return NULL;
	if ((p[0]=(void**)malloc(n3*n2*sizeof(void*)))==NULL) {
		free(p);
		return NULL;
	}
	if ((p[0][0]=(void*)malloc(n3*n2*n1*size))==NULL) {
		free(p[0]);
		free(p);
		return NULL;
	}

	for (i3=0; i3<n3; i3++) {
		p[i3] = p[0]+n2*i3;
		for (i2=0; i2<n2; i2++)
			p[i3][i2] = (char*)p[0][0]+size*n1*(i2+n2*i3);
	}
	return p;
}

/* free a 3-d array */
void free3 (void ***p)
{
	free(p[0][0]);
	free(p[0]);
	free(p);
}

/* allocate a 4-d array */
void ****alloc4 (int n1, int n2, int n3, int n4, int size)
{
	int i4,i3,i2;
	void ****p;

	if ((p=(void****)malloc(n4*sizeof(void***)))==NULL)
		return NULL;
	if ((p[0]=(void***)malloc(n4*n3*sizeof(void**)))==NULL) {
		free(p);
		return NULL;
	}
	if ((p[0][0]=(void**)malloc(n4*n3*n2*sizeof(void*)))==NULL) {
		free(p[0]);
		free(p);
		return NULL;
	}
	if ((p[0][0][0]=(void*)malloc(n4*n3*n2*n1*size))==NULL) {
		free(p[0][0]);
		free(p[0]);
		free(p);
		return NULL;
	}
	for (i4=0; i4<n4; i4++) {
		p[i4] = p[0]+i4*n3;
		for (i3=0; i3<n3; i3++) {
			p[i4][i3] = p[0][0]+n2*(i3+n3*i4);
			for (i2=0; i2<n2; i2++)
				p[i4][i3][i2] = (char*)p[0][0][0]+
						size*n1*(i2+n2*(i3+n3*i4));
		}
	}
	return p;
}

/* free a 4-d array */
void free4 (void ****p)
{
	free(p[0][0][0]);
	free(p[0][0]);
	free(p[0]);
	free(p);
}

/* The following two functions were added by Zhaobo Meng, Jan. 1997*/
/* allocate a 5-d array */
void *****alloc5 (int n1, int n2, int n3, int n4, int n5, int size)
{
	int i5,i4,i3,i2;
	void *****p;

	if ((p=(void*****)malloc(n5*sizeof(void****)))==NULL)
		return NULL;
	if ((p[0]=(void****)malloc(n5*n4*sizeof(void***)))==NULL) {
		free(p);
		return NULL;
	}
	if ((p[0][0]=(void***)malloc(n5*n4*n3*sizeof(void**)))==NULL) {
		free(p[0]);
		free(p);
		return NULL;
	}
	if ((p[0][0][0]=(void**)malloc(n5*n4*n3*n2*sizeof(void*)))==NULL) {
		free(p[0][0]);
		free(p[0]);
		free(p);
		return NULL;
	}
	if ((p[0][0][0][0]=(void*)malloc(n5*n4*n3*n2*n1*size))==NULL) {
		free(p[0][0][0]);
		free(p[0][0]);
		free(p[0]);
		free(p);
		return NULL;
	}
	for (i5=0; i5<n5; i5++) {
		p[i5] = p[0]+n4*i5;
		for (i4=0; i4<n4; i4++) {
			p[i5][i4] = p[0][0]+n3*(i4+n4*i5);
			for (i3=0; i3<n3; i3++) {
				p[i5][i4][i3] = p[0][0][0]+n2*(i3+n3*(i4+n4*i5));
				for (i2=0; i2<n2; i2++)
					p[i5][i4][i3][i2] = (char*)p[0][0][0][0]+
						size*n1*(i2+n2*(i3+n3*(i4+n4*i5)));
			}
		}
	}
	return p;
}

/* free a 5-d array */
void free5 (void *****p)
{
	free(p[0][0][0][0]);
	free(p[0][0][0]);
	free(p[0][0]);
	free(p[0]);
	free(p);
}

/* The following two functions were added by Zhaobo Meng, Jan. 1997*/
/* allocate a 6-d array */
void ******alloc6 (int n1, int n2, int n3, int n4, int n5, int n6,
                  int size)
{
	int i6,i5,i4,i3,i2;
	void ******p;

	if ((p=(void******)malloc(n6*sizeof(void*****)))==NULL)
		return NULL;

	if ((p[0]=(void*****)malloc(n6*n5*sizeof(void****)))==NULL) {
                free(p);
		return NULL;
        }

	if ((p[0][0]=(void****)malloc(n6*n5*n4*sizeof(void***)))==NULL) {
		free(p[0]);
                free(p);
		return NULL;
	}
	if ((p[0][0][0]=(void***)malloc(n6*n5*n4*n3*sizeof(void**)))==NULL) {
		free(p[0][0]);
                free(p[0]);
		free(p);
		return NULL;
	}
	if ((p[0][0][0][0]=(void**)malloc(n6*n5*n4*n3*n2*sizeof(void*)))==NULL) {
	        free(p[0][0][0]);
		free(p[0][0]);
		free(p[0]);
		free(p);
		return NULL;
	}
	if ((p[0][0][0][0][0]=(void*)malloc(n6*n5*n4*n3*n2*n1*size))==NULL) {
	        free(p[0][0][0][0]);
		free(p[0][0][0]);
		free(p[0][0]);
		free(p[0]);
		free(p);
		return NULL;
	}

        for (i6=0; i6<n6; i6++) {
                p[i6] = p[0]+n5*i6;
	        for (i5=0; i5<n5; i5++) {
	                p[i6][i5] = p[0][0]+n4*(i5+n5*i6);
			for (i4=0; i4<n4; i4++) {
			        p[i6][i5][i4] = p[0][0][0]+n3*(i4+n4*(i5+n5*i6));
				for (i3=0; i3<n3; i3++) {
				        p[i6][i5][i4][i3] = p[0][0][0][0]
					      +n2*(i3+n3*(i4+n4*(i5+n5*i6)));
					for (i2=0; i2<n2; i2++)
					        p[i6][i5][i4][i3][i2] = 
						      (char*)p[0][0][0][0][0]+
				       size*n1*(i2+n2*(i3+n3*(i4+n4*(i5+n5*i6))));
			        }
		        }
	        }
        }
	return p;
}

/* free a 6-d array */
void free6 (void ******p)
{
        free(p[0][0][0][0][0]);
	free(p[0][0][0][0]);
	free(p[0][0][0]);
	free(p[0][0]);
	free(p[0]);
	free(p);
}


/* allocate a 1-d array of ints */
int *allocate_i1(int n1)
{
	return (int*)alloc1(n1,sizeof(int));
}
/* free a 1-d array of ints */
void free_i1(int *p)
{
	free1(p);
}

/* allocate a 2-d array of ints */
int **allocate_i2(int n2, int n1)
{
	return (int**)alloc2(n1,n2,sizeof(int));
}

/* free a 2-d array of ints */
void free_i2(int **p)
{
	free2((void**)p);
}

/* allocate a 3-d array of ints */
int ***allocate_i3(int n3, int n2, int n1)
{
	return (int***)alloc3(n1,n2,n3,sizeof(int));
}

/* free a 3-d array of ints */
void free_i3(int ***p)
{
	free3((void***)p);
}

/* allocate a 1-d array of floats */
float *allocate_f1(int n1)
{
	return (float*)alloc1(n1,sizeof(float));
}



/* free a 1-d array of floats */
void free_f1(float *p)
{
	free1(p);
}



/* allocate a 2-d array of floats */
float **allocate_f2(int n2, int n1)
{
	return (float**)alloc2(n1,n2,sizeof(float));
}

/* free a 2-d array of floats */
void free_f2(float **p)
{
	free2((void**)p);
}

/* allocate a 3-d array of floats */
float ***allocate_f3(int n3, int n2, int n1)
{
	return (float***)alloc3(n1,n2,n3,sizeof(float));
}

/* free a 3-d array of floats */
void free_f3(float ***p)
{
	free3((void***)p);
}

/* allocate a 4-d array of floats, added by Zhaobo Meng, 1997 */
float ****allocate_f4(int n4, int n3, int n2, int n1)
{
        return (float****)alloc4(n1,n2,n3,n4,sizeof(float));
}

/* free a 4-d array of floats, added by Zhaobo Meng, 1997 */
void free_f4(float ****p)
{
        free4((void****)p);
}

/* allocate a 5-d array of floats, added by Zhaobo Meng, 1997 */
float *****allocate_f5(int n5, int n4, int n3, int n2, int n1)
{
        return (float*****)alloc5(n1,n2,n3,n4,n5,sizeof(float));
}

/* free a 5-d array of floats, added by Zhaobo Meng, 1997 */
void free_f5(float *****p)
{
        free5((void*****)p);
}

/* allocate a 6-d array of floats, added by Zhaobo Meng, 1997 */
float ******allocate_f6(int n6, int n5, int n4, int n3, int n2, int n1)
{
        return (float******)alloc6(n1,n2,n3,n4,n5,n6,sizeof(float));
}

/* free a 6-d array of floats, added by Zhaobo Meng, 1997 */
void free_f6(float ******p)
{
        free6((void******)p);
}

__complex__ float *allocate_c1(int n1)
{
	return (__complex__ float*)alloc1(n1,sizeof(__complex__ float));
}
void free_c1(__complex__ float *p)
{
	free1(p);
}

__complex__ float **allocate_c2(int n2, int n1)
{
	return (__complex__ float**)alloc2(n1,n2,sizeof(__complex__ float));
}

void free_c2(__complex__ float **p)
{
	free2((void**)p);
}
__complex__ float ***allocate_c3(int n3, int n2, int n1)
{
	return (__complex__ float***)alloc3(n1,n2,n3,sizeof(__complex__ float));
}

void free_c3(__complex__ float ***p)
{
	free3((void***)p);
}

__complex__ float ****allocate_c4(int n4, int n3, int n2, int n1)
{
        return (__complex__ float****)alloc4(n1,n2,n3,n4,sizeof(__complex__ float));
}

void free_c4(__complex__ float ****p)
{
        free4((void****)p);
}

__complex__ float *****allocate_c5(int n5, int n4, int n3, int n2, int n1)
{
        return (__complex__ float*****)alloc5(n1,n2,n3,n4,n5,sizeof(__complex__ float));
}

void free_c5(__complex__ float *****p)
{
        free5((void*****)p);
}

__complex__ float ******allocate_c6(int n6, int n5, int n4, int n3, int n2, int n1)
{
        return (__complex__ float******)alloc6(n1,n2,n3,n4,n5,n6,sizeof(__complex__ float));
}

void free_c6(__complex__ float ******p)
{
        free6((void******)p);
}
void **alloc1_pointer (int n2, int size)
{
	void **p;

	if ((p=(void **)malloc(n2*size))==NULL)
		return NULL;
	return p;
}



float **allocate_f1_pointer(int n2)
{
	return (float**)alloc1_pointer(n2,sizeof(float *));
}

__complex__ float **allocate_c1_pointer(int n2)
{
        return (__complex__ float**)alloc1_pointer(n2,sizeof(__complex__ float *));
}

float ****allocate_f1_pointer3(int n4)
{
   float ****p;
   if ((p= (float ****) malloc(n4*sizeof(float ***) ) )==NULL)
     return NULL;
   return p;
}
