program main 

! This program implements the nonlinear conjugate gradient method
! coefficients for beta is defined using Polak-Ribiere
! refer the wiki page for details

  use sep

  implicit none

  integer                                :: n1, n2, n3
  real                                   :: o1, o2, o3, d1, d2, d3
  integer                                :: i, j, k, z_seabed
  real,allocatable                       :: cvel(:,:,:), ceps(:,:,:), cdel(:,:,:)      ! current (-gradient)
  real,allocatable                       :: pgvel(:,:,:), pgeps(:,:,:), pgdel(:,:,:)    ! previous (-gradient) 
  real,allocatable                       :: psvel(:,:,:), pseps(:,:,:), psdel(:,:,:)    ! previous search direction 
  real,allocatable                       :: svel(:,:,:), seps(:,:,:), sdel(:,:,:)      ! new search direction 
  double precision                       :: beta_v, beta_e, beta_d
  real                                   :: min_e, min_d 
  real                                   :: max_e, max_d 

  call sep_init(SOURCE)
  call getch_add_string("head=/dev/null")
  call getch_add_string("noheader=y")
  call set_no_putch()
  call sep_begin_prog()


  call from_aux("cvel", "n1", n1)
  call from_aux("cvel", "n2", n2)
  call from_aux("cvel", "n3", n3)
  call from_aux("cvel", "o1", o1)
  call from_aux("cvel", "o2", o2)
  call from_aux("cvel", "o3", o3)
  call from_aux("cvel", "d1", d1)
  call from_aux("cvel", "d2", d2)
  call from_aux("cvel", "d3", d3)

  allocate(cvel(n1,n2,n3),ceps(n1,n2,n3),cdel(n1,n2,n3))
  allocate(pgvel(n1,n2,n3),pgeps(n1,n2,n3),pgdel(n1,n2,n3))
  allocate(psvel(n1,n2,n3),pseps(n1,n2,n3),psdel(n1,n2,n3))
  allocate(svel(n1,n2,n3),seps(n1,n2,n3),sdel(n1,n2,n3))

  call sreed("cvel",cvel,4*n1*n2*n3)
  call sreed("pgvel",pgvel,4*n1*n2*n3)
  call sreed("psvel",psvel,4*n1*n2*n3)

  call sreed("ceps",ceps,4*n1*n2*n3)
  call sreed("pgeps",pgeps,4*n1*n2*n3)
  call sreed("pseps",pseps,4*n1*n2*n3)

  call sreed("cdel",cdel,4*n1*n2*n3)
  call sreed("pgdel",pgdel,4*n1*n2*n3)
  call sreed("psdel",psdel,4*n1*n2*n3)


  beta_v = sum(dprod(cvel,(cvel-pgvel))) / (sum(dprod(pgvel,pgvel)) + epsilon(beta_v))
  beta_e = sum(dprod(ceps,(ceps-pgeps))) / (sum(dprod(pgeps,pgeps)) + epsilon(beta_e))
  beta_d = sum(dprod(cdel,(cdel-pgdel))) / (sum(dprod(pgdel,pgdel)) + epsilon(beta_d))

  beta_v = max(beta_v,0.)
  beta_e = max(beta_e,0.)
  beta_d = max(beta_d,0.)

  svel = cvel + beta_v*psvel
  seps = ceps + beta_e*pseps
  sdel = cdel + beta_e*psdel

  call srite("svel", svel, 4*n1*n2*n3)
  call srite("seps", seps, 4*n1*n2*n3)
  call srite("sdel", sdel, 4*n1*n2*n3)

  call to_history("n1", n1, "svel")
  call to_history("n2", n2, "svel")
  call to_history("n3", n3, "svel")
  call to_history("d1", d1, "svel")
  call to_history("d2", d2, "svel")
  call to_history("d3", d3, "svel")
  call to_history("o1", o1, "svel")
  call to_history("o2", o2, "svel")
  call to_history("o3", o3, "svel")
  call to_history("esize", 4, "svel")

  call to_history("n1", n1, "seps")
  call to_history("n2", n2, "seps")
  call to_history("n3", n3, "seps")
  call to_history("d1", d1, "seps")
  call to_history("d2", d2, "seps")
  call to_history("d3", d3, "seps")
  call to_history("o1", o1, "seps")
  call to_history("o2", o2, "seps")
  call to_history("o3", o3, "seps")
  call to_history("esize", 4, "seps")

  call to_history("n1", n1, "sdel")
  call to_history("n2", n2, "sdel")
  call to_history("n3", n3, "sdel")
  call to_history("d1", d1, "sdel")
  call to_history("d2", d2, "sdel")
  call to_history("d3", d3, "sdel")
  call to_history("o1", o1, "sdel")
  call to_history("o2", o2, "sdel")
  call to_history("o3", o3, "sdel")
  call to_history("esize", 4, "sdel")

  deallocate(cvel,ceps,cdel,pgvel,pgeps,pgdel,psvel,pseps,psdel,svel,seps,sdel)

  call sep_end_prog()

  call exit()
end program  
