!!.. compute coefficient for each depth according 
!!.. to eta and delta 
module ani_getcef_mod
  use bwi_struct_mod
  implicit none
  integer, private 				:: ne, nd, nstep, nstep_dr, nx, ny 
  real   , private 				:: oe, od, de, dd
  real	 , allocatable, private :: coef(:,:,:) 
  real	 , allocatable, private :: dr_coef(:,:,:) 
contains

!=====================================================================
!
  integer function ani_getcef_init(cgrid, pgrid, coef_in, dr_coef_in) result(stat)
    type(cef_grid) :: cgrid
    type(pro_grid) :: pgrid
	real, intent(in)   :: coef_in(:,:,:)
	real, optional, intent(in) :: dr_coef_in(:,:,:)

	nstep = cgrid%step%n
	nstep_dr = nstep * 2
	nd	  = cgrid%cfd%n
	ne	  = cgrid%cfe%n

	od	  = cgrid%cfd%o
	oe	  = cgrid%cfe%o

	dd	  = cgrid%cfd%d
	de	  = cgrid%cfe%d

	nx    = pgrid%ax%n
	ny    = pgrid%ay%n

	allocate(coef(nstep,nd,ne))
	coef = coef_in

	if (present(dr_coef_in)) then
		allocate(dr_coef(nstep_dr,nd,ne))
		dr_coef = dr_coef_in
	end if
    stat  = 0
  end function
!======================================================================
  integer function ani_getcef_clos() result(stat)
    deallocate(coef)
	if(allocated(dr_coef)) deallocate(dr_coef)
    stat  = 0
  end function
!======================================================================
!.. compute the coefficients for the current depth step
!----------------------------------------------------------------------
  integer function ani_getcef_oper(eta_slice,delta_slice,alpha,beta,dade,dbde,dadd,dbdd) result(stat)
    real, dimension(:,:) 				:: eta_slice
    real, dimension(:,:) 				:: delta_slice
    real, dimension(:,:)				:: alpha , beta
    real, dimension(:,:), optional		:: dade, dbde, dadd, dbdd

    integer              :: ie, id, ix, iy, ic

    if (present(dade).and.present(dbde)) then
      do iy = 1, ny
      do ix = 1, nx
         ie = (eta_slice(ix,iy  ) - oe) / de + 1 
         id = (delta_slice(ix,iy) - od) / dd + 1 
         if (ie>=1 .and. ie<=ne .and. id>=1 .and. id<=nd) then
            dbdd(ix,iy) = dr_coef(1,id,ie)
            dadd(ix,iy) = dr_coef(2,id,ie)
            dbde(ix,iy) = dr_coef(3,id,ie)
            dade(ix,iy) = dr_coef(4,id,ie)
         end if
       end do
       end do
     end if  

     do iy = 1, ny
     do ix = 1, nx
        ie = (eta_slice(ix,iy  ) - oe) / de + 1 
        id = (delta_slice(ix,iy) - od) / dd + 1 
        if (ie>=1 .and. ie<=ne .and. id>=1 .and. id<=nd) then
          beta (ix,iy) = coef(1,id,ie)
          alpha(ix,iy) = coef(2,id,ie)
        end if
     end do
     end do
    stat = 0 

  end function

!======================================================================
!.. compute the coefficients for a specific eta and delta value 
!----------------------------------------------------------------------
  integer function ani_getcef_oper2(eta,delta,alpha,beta) result(stat)
    real :: eta
    real :: delta
    real :: alpha, beta

    integer              :: ie, id, ix, iy, ic

	ie = (eta   - oe) / de + 1 
	id = (delta - od) / dd + 1 

	if (ie>=1 .and. ie<=ne .and. id>=1 .and. id<=nd) then
		beta  = coef(1,id,ie)
		alpha = coef(2,id,ie)
	else
		write(0,*) "bad eta and delta value"
		write(0,*) "eta=", eta, "delta", delta
		stop
	end if
    stat = 0 
  end function

end module
