module base_trismt_mod
  implicit none

contains

  integer function base_tris1d_oper(field,rect1,nsmt) result(stat)
    real         :: field(:)
    integer      :: rect1,nsmt
    integer      :: n1,i1,ismt
    real,pointer :: temp(:)
    n1=size(field)
    allocate(temp(n1))
    do ismt=1, nsmt
       if (rect1>1 .and. n1>1) then
          call triangle(rect1,1,n1,field,temp)
          field = temp 
       end if
    end do
    deallocate(temp)
    stat = 0
  end function

  integer function base_tris2d_oper(field,rect1,rect2,nsmt) result(stat)
    real         :: field(:,:)
    integer      :: rect1,rect2,nsmt
    integer      :: n1,n2,i1,i2,ismt 
    real,pointer :: temp(:)
    n1=size(field,1)
    n2=size(field,2)
    allocate(temp(max(n1,n2)))  
    do ismt=1, nsmt
       if (rect1>1 .and. n1>1) then
          do i2=1, n2
             call triangle(rect1,1,n1,field(:,i2),temp)
             field(1:n1,i2)=temp(1:n1) 
          end do
       end if
       if (rect2>1 .and. n2>1) then
          do i1=1, n1
             call triangle(rect2,1,n2,field(i1,:),temp)
             field(i1,1:n2)=temp(1:n2)
          end do
       end if
    end do 
    deallocate(temp)
    stat = 0
  end function

  integer function base_tris3d_oper(field,rect1,rect2,rect3,nsmt) result(stat)
    real         :: field(:,:,:)
    integer      :: rect1,rect2,rect3,nsmt
    integer      :: n1,n2,n3,i1,i2,i3,ismt
    real,pointer :: temp(:)
    n1=size(field,1)
    n2=size(field,2)
    n3=size(field,3)
    allocate(temp(max(n1,max(n2,n3))))
    do ismt=1, nsmt 
       if (rect1>1 .and. n1>1) then
          do i3=1,n3
          do i2=1,n2
             call triangle(rect1,1,n1,field(:,i2,i3),temp)
             field(1:n1,i2,i3)=temp(1:n1)
          end do
         end do
       end if
       if (rect2>1 .and. n2>1) then
          do i3=1,n3
          do i1=1,n1
             call triangle(rect2,1,n2,field(i1,:,i3),temp)
             field(i1,1:n2,i3)=temp(1:n2)
          end do
          end do
       end if
       if (rect3>1 .and. n3>1) then
          do i2=1,n2
          do i1=1,n1
             call triangle(rect3,1,n3,field(i1,i2,:),temp)
             field(i1,i2,1:n3)=temp(1:n3)
          end do
          end do
       end if
    end do 
    deallocate(temp)
    stat = 0
  end function

  subroutine triangle(nr, m1, n12, uu, vv)
  ! input:        nr      rectangle width (points) (Triangle base twice as wide.)
  ! input:        uu(m1,i2),i2=1,n12      is a vector of data.
  ! output:       vv(m1,i2),i2=1,n12      may be on top of uu
    integer nr,m1,n12, i,np,nq
    real uu( m1, n12),  vv( m1, n12)
    real pp(n12+nr-1), qq(n12+nr+nr-2), tt(n12)
    do i=1,n12
      qq(i) = uu(1,i)
    end do
    if ( n12 .eq. 1 ) then
    !  call copy( n12, qq, tt)
       tt = qq
    else
       call boxconv( nr, n12, qq, pp)
       np = nr+n12-1
       call boxconv( nr, np , pp, qq)
       nq = nr+np-1
       do i= 1, n12
          tt(i) = qq(i+nr-1)
       end do
       do i= 1, nr-1
          ! fold back near end
          tt(i) = tt(i) + qq(nr-i)
       end do
       do i= 1, nr-1
          ! fold back far end
          tt(n12-i+1) = tt(n12-i+1) + qq(n12+(nr-1)+i)
       end do
    end if
    do i=1,n12
       vv(1,i) = tt(i)
    end do
    return
  end subroutine

  subroutine boxconv(nb, nx, xx, yy)
  ! inputs:       nx,  xx(i), i=1,nx      the data
  !               nb                      the box length
  ! output:       yy(i),i=1,nx+nb-1       smoothed data
    integer nx, ny, nb, i
    real xx(nx), yy(1)
    real bb(nx+nb)
    if ( nb < 1 .or. nb > nx) then
       call erexit('boxconv')  ! "||" means .OR.
    end if
    ny = nx+nb-1
    do i= 1, ny
       bb(i) = 0.
    end do
    bb(1) = xx(1)
    do i= 2, nx
       bb(i) = bb(i-1) + xx(i)         ! make B(Z) = X(Z)/(1-Z)
    end do
    do i= nx+1, ny
       bb(i) = bb(i-1)
    end do
    do i= 1, nb
       yy(i) = bb(i)
    end do
    do i= nb+1, ny
       yy(i) = bb(i) - bb(i-nb)        ! make Y(Z) = B(Z)*(1-Z**nb)
    end do
    do i= 1, ny
       yy(i) = yy(i) / nb
    end do
    return
  end subroutine


end module

