module bwi_dipflt_mod
  use base_struct_mod
  implicit none

  real, dimension(:,:,:), pointer, private :: v2
  type(axis)                     , private :: ax, ay, az

contains

  integer function bwi_dipflt_init(v2_in, ax_in, ay_in, az_in) result(stat)
    real, dimension(:,:,:), target :: v2_in
    type(axis)                     :: ax_in, ay_in, az_in
    v2 => v2_in
    ax = ax_in
    ay = ay_in
    az = az_in
    stat = 0
  end function

  integer function bwi_dipflt_oper(adj, add, f, g) result(stat)
    logical, intent(in) :: adj, add
    real, dimension(:)  :: f, g
    call bwi_dipflt(adj, add, f, g)
    stat = 0
  end function

  subroutine bwi_dipflt(adj, add, f, g)
    logical                            :: adj, add
    real   , dimension(ax%n,ay%n,az%n) :: f, g

    integer                            :: ix, iz
    real                               :: fx, fz, gx, gz, v1i, v2i  

    if (.not.add) then
       if (adj) then
          f = 0.
       else
          g = 0.
       end if
    end if 

    if (ax%n>1 .and. ay%n==1 .and. az%n>1) then
       if (adj) then
          do iz=2, az%n
          do ix=2, ax%n
             v2i = v2(ix,1,iz)
             v1i = sqrt(1.-v2i*v2i)
             gx  = v1i*g(ix,1,iz)
             gz  = v2i*g(ix,1,iz) 
             f(ix  ,1,iz  ) = f(ix  ,1,iz  ) + 0.5/ax%d*gx + 0.5/az%d*gz
             f(ix-1,1,iz  ) = f(ix-1,1,iz  ) - 0.5/ax%d*gx + 0.5/az%d*gz
             f(ix  ,1,iz-1) = f(ix  ,1,iz-1) + 0.5/ax%d*gx - 0.5/az%d*gz
             f(ix-1,1,iz-1) = f(ix-1,1,iz-1) - 0.5/ax%d*gx - 0.5/az%d*gz
          end do
          end do 

       else
          do iz=2, az%n
          do ix=2, ax%n
             v2i = v2(ix,1,iz)
             v1i = sqrt(1.-v2i*v2i)
             fx  = 0.5/ax%d*(f(ix,1,iz)-f(ix-1,1,iz)) + 0.5/ax%d*(f(ix,1,iz-1)-f(ix-1,1,iz-1))
             fz  = 0.5/az%d*(f(ix,1,iz)-f(ix,1,iz-1)) + 0.5/az%d*(f(ix-1,1,iz)-f(ix-1,1,iz-1))
             g(ix,1,iz) = g(ix,1,iz) + v1i*fx + v2i*fz
          end do
          end do
       end if
    else
    end if
  end subroutine

end module
