module bwi_diphlx_mod
  use  base_struct_mod
  use  base_hlxtyp_mod
  use  base_wilson_mod
  implicit none

  type(helix)     , private :: dipflt
  real            , private :: dipflt0
  type(axis)      , private :: adip
  real            , private :: dipeps
  real            , private :: d1, d2
  integer         , private :: n1, na, niter
  character(len=5), private :: scheme

contains

  integer function bwi_diphlx_init(adip_in, dipeps_in, d1_in, d2_in, n1_in, na_in, niter_in, scheme_in) result(stat)
    type(axis)       :: adip_in
    real             :: dipeps_in
    real             :: d1_in, d2_in
    integer          :: n1_in, na_in, niter_in
    character(len=5) :: scheme_in

    adip   = adip_in
    dipeps = dipeps_in
    d1     = d1_in
    d2     = d2_in
    n1     = n1_in
    na     = na_in
    niter  = niter_in
    scheme = scheme_in



!   2 by 3 stencil
    call allocatehelix(dipflt, 7)
    dipflt%lag = (/1,n1-1,n1,n1+1,2*n1-1,2*n1,2*n1+1/) 

!   initialize factorization
    stat = base_wilson_init(maxval(dipflt%lag)+1)
    stat = 0
  end function

  integer function bwi_diphlx_clos() result(stat)
    stat = base_wilson_clos() 
    call deallocatehelix(dipflt)
    stat = 0
  end function

  integer function bwi_diphlx_oper(dipfac, dipfac0) result(stat)
    type(helix), dimension(:) :: dipfac
    real       , dimension(:) :: dipfac0
 
    integer                   :: idip, ia
    real                      :: p, dip, PI
    real                      :: v1, v2, f11, f12, f13, f21, f22, f23

    PI = 3.14159265359

!   the first 1 to n-1 are dip filters
!   ----------------------------------
    do idip=1, adip%n
       dip = adip%o+(idip-1)*adip%d

       v1 = cos(dip*PI/180.)/d1
       v2 = sin(dip*PI/180.)/d2
       p  = v2/v1

       if (idip==adip%n) then
          dipflt0 = 2/d1/d1 + 2/d2/d2 + dipeps
          dipflt%flt(1) = 1/d1/d1
          dipflt%flt(2) = 0.
          dipflt%flt(3) = 1/d2/d2
          dipflt%flt(4) = 0.
          dipflt%flt(5) = 0.
          dipflt%flt(6) = 0.
          dipflt%flt(7) = 0. 

       else
          select case (scheme)
            case ("LW2x3")
!                2 by 3 Lax-Wendroff, type II, x and x+1
                f11 = -0.5*p*(1+p); f12 = p*p-1; f13 = 0.5*p*(1-p)
                f21 = 0.          ; f22 = 1.   ; f23 = 0.
!                f11 = -0.5*v2*(v1+v2); f12 = v2*v2-v1*v1; f13 = 0.5*v2*(v1-v2)
!                f21 = 0.             ; f22 = v1*v1      ; f23 = 0.

            case ("UW2x3")
!                2 by 3 upwind, type II, x and x+1
!                ---------------------------------
                 if (dip>=0.) then
                    f11 = -p; f12 = p-1 ; f13 = 0.
                    f21 = 0.; f22 = 1.  ; f23 = 0. 
                 else
                    f11 = 0.; f12 = -1-p; f13 = p
                    f21 = 0.; f22 = 1.  ; f23 = 0.
                 end if

!                 if (dip>=0.) then
!                    f11 = -v2; f12 = v2-v1 ; f13 = 0.
!                    f21 = 0. ; f22 = v1    ; f23 = 0.
!                 else
!                    f11 = 0. ; f12 = -v1-v2; f13 = v2
!                    f21 = 0. ; f22 = v1    ; f23 = 0.
!                 end if

            case default
                 write(0,*) "ERROR: scheme = LW2x3/[UW2x3]"
                 STOP
          end select

!         2 by 3 auto-correlation
!         -----------------------
          dipflt0 = f11*f11+f12*f12+f13*f13+f21*f21+f22*f22+f23*f23 + dipeps
          dipflt%flt(1) = f11*f21+f12*f22+f13*f23
          dipflt%flt(2) = f12*f21+f13*f22
          dipflt%flt(3) = f11*f12+f12*f13+f22*f21+f22*f23
          dipflt%flt(4) = f11*f22+f12*f23
          dipflt%flt(5) = f13*f21
          dipflt%flt(6) = f11*f13+f21*f23
          dipflt%flt(7) = f11*f23

       end if

       dipfac0(idip)    = 0.
       dipfac(idip)%flt = 0.
       do ia=1, na
          dipfac(idip)%lag(ia)      = ia
          dipfac(idip)%lag(na+ia)   = n1+ia-na
          dipfac(idip)%lag(2*na+ia) = n1+ia
       end do

       stat = base_wilson_oper(niter, dipflt0, dipflt, dipfac0(idip), dipfac(idip))

    end do
    stat = 0
  end function

end module
