module bwi_grdprc_mod
  use base_struct_mod
  use base_spline_mod
  use base_hlxtyp_mod
  use base_nshlxc_mod
  use bwi_fileio_mod
!  use bwi_dipcal_mod
  use bwi_poldiv_mod

  implicit none

  real, allocatable                     :: ptch(:)
  type(nfilter)                         :: ns_filt
  type( filter)                         :: lapfilt
  integer                               :: nfilt, nang

contains

! scale the gradient by depth or sqrt(depth)
! -------------------------------------------
  integer function bwi_gscale_oper(grdv_, grde_, grdd_, vgrid) result(stat)
    character(len=*)                    :: grdv_, grde_, grdd_
    type(vel_grid)                      :: vgrid

    integer                             :: izs
    real                                :: fctr, zs, zmax
    real, dimension(:,:,:), allocatable :: grdv, grde, grdd
    double precision                    :: norm

    allocate(grdv(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(grde(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(grdd(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
!   read the gradient
!   -----------------
    call bwi_fio_vmod_updt(grdv, vgrid, grdv_, "reed")
    call bwi_fio_vmod_updt(grde, vgrid, grde_, "reed")
    call bwi_fio_vmod_updt(grdd, vgrid, grdd_, "reed")
!   apply the scaling 
!   -----------------
    zmax = vgrid%az%n*vgrid%az%d
    do izs=1, vgrid%az%n
       zs   = izs*vgrid%az%d
       fctr = sqrt(zs)/sqrt(zmax)
       grdv(:,:,izs) = grdv(:,:,izs)*fctr
       grde(:,:,izs) = grde(:,:,izs)*fctr
       grdd(:,:,izs) = grdd(:,:,izs)*fctr
    end do

!   normalize the gradient (NEW !!!!!!!!!!!)
!   ----------------------------------------
    norm = sqrt(sum(dble(grdv)*dble(grdv)))
    grdv = grdv/(norm+epsilon(norm))
    norm = sqrt(sum(dble(grde)*dble(grde)))
    grde = grde/(norm+epsilon(norm))
    norm = sqrt(sum(dble(grdd)*dble(grdd)))
    grdd = grdd/(norm+epsilon(norm))

!   write out the scaled gradient
!   -----------------------------
    call bwi_fio_vmod_updt(grdv, vgrid, grdv_, "rite")
    call bwi_fio_vmod_updt(grde, vgrid, grde_, "rite")
    call bwi_fio_vmod_updt(grdd, vgrid, grdd_, "rite")
    deallocate(grdv,grde,grdd)
    stat = 0
  end function

! scale the gradient according to the standard deviation matrix 
! -------------------------------------------------------------
  integer function bwi_gxparm_oper(grdv_, grde_, grdd_, stdv_, vgrid) result(stat)
    character(len=*)                      :: grdv_
    character(len=*)                      :: grde_
    character(len=*)                      :: grdd_
    character(len=*)                      :: stdv_
    type(vel_grid)                        :: vgrid

    integer                               :: izs, ixs, iys
    real                                  :: vv, ee, dd 
    real, dimension(:,:,:),   allocatable :: grdv
    real, dimension(:,:,:),   allocatable :: grde
    real, dimension(:,:,:),   allocatable :: grdd
    real, dimension(:,:,:,:), allocatable :: stdv
    double precision                      :: norm

    allocate(grdv(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(grde(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(grdd(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(stdv(vgrid%ax%n,vgrid%ay%n,vgrid%az%n,6))
!   read the gradient
!   -----------------
    call bwi_fio_vmod_updt(grdv, vgrid, grdv_, "reed")
    call bwi_fio_vmod_updt(grde, vgrid, grde_, "reed")
    call bwi_fio_vmod_updt(grdd, vgrid, grdd_, "reed")
!	read the standard deviation matrix
!	----------------------------------
    call bwi_fio_stdv(stdv, vgrid, stdv_, "reed")
!   apply x-parameter scaling 
!   1 - Cvv, 2 - Cee, 3 - Cdd
!   4 - Cve, 5 - Cvd, 6 - Ced
!   -------------------------
    do izs=1, vgrid%az%n
       do iys=1, vgrid%ay%n
          do ixs=1, vgrid%ax%n
             vv = stdv(ixs,iys,izs,1)*grdv(ixs,iys,izs) + stdv(ixs,iys,izs,4)*grde(ixs,iys,izs) + stdv(ixs,iys,izs,5)*grdd(ixs,iys,izs)
             ee = stdv(ixs,iys,izs,2)*grde(ixs,iys,izs) + stdv(ixs,iys,izs,4)*grdv(ixs,iys,izs) + stdv(ixs,iys,izs,6)*grdd(ixs,iys,izs)
             dd = stdv(ixs,iys,izs,3)*grdd(ixs,iys,izs) + stdv(ixs,iys,izs,5)*grdv(ixs,iys,izs) + stdv(ixs,iys,izs,6)*grde(ixs,iys,izs)
             grdv(ixs,iys,izs) = vv 
             grde(ixs,iys,izs) = ee 
             grdd(ixs,iys,izs) = dd 
          end do
       end do
    end do

!   write out the preconditioned grd 
!   --------------------------------
    call bwi_fio_vmod_updt(grdv, vgrid, grdv_, "rite")
    call bwi_fio_vmod_updt(grde, vgrid, grde_, "rite")
    call bwi_fio_vmod_updt(grdd, vgrid, grdd_, "rite")
    deallocate(grdv,grde,grdd,stdv)
    stat = 0
  end function

  integer function bwi_grdmsk_oper(grdo_, gmsk_, vgrid) result(stat)
    character(len=*) :: grdo_, gmsk_
    type(vel_grid)   :: vgrid

    real, dimension(:,:,:), allocatable :: grdo, gmsk

    allocate(grdo(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(gmsk(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    call bwi_fio_vmod_updt(grdo, vgrid, grdo_, "reed")
    call bwi_fio_vmod_updt(gmsk, vgrid, gmsk_, "reed")
    grdo = grdo*gmsk
    call bwi_fio_vmod_updt(grdo, vgrid, grdo_, "rite")
    deallocate(grdo)
    deallocate(gmsk)  
    stat = 0
  end function

  integer function bwi_dipflt_init(dipx_, dipy_, vgrid) result(stat)

    allocate(ptch(vgrid%ax%n*vgrid%ay%n*vgrid%az%n))
  


  stat = 0
  end function
  integer function bwi_dipflt_oper(grdo_, dipx_, dipy_, vgrid) result(stat)
    character(len=*)                    :: grdo_
    character(len=*)                    :: dipx_
    character(len=*)                    :: dipy_
    type(vel_grid)                      :: vgrid
 
    real, dimension(:,:,:), allocatable :: grdo
    real, dimension(:,:,:), allocatable :: dipx 
    real, dimension(:,:,:), allocatable :: dipy
    real, dimension(:,:)  , allocatable :: tmpx
    real, dimension(:,:)  , allocatable :: tmpy
    integer                             :: ix, iy

    allocate(grdo(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(dipx(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(dipy(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(tmpx(vgrid%ax%n,vgrid%az%n)) 
    allocate(tmpy(vgrid%ay%n,vgrid%az%n)) 

    call bwi_fio_vmod_updt(grdo, vgrid, grdo_, "reed")

!   Ax^{-1}Ay^{-1}Ay^{-T}Ax^{-T}
    if (vgrid%ax%n>1) then
       call bwi_fio_vmod_updt(dipx, vgrid, dipx_, "reed")
        
       stat = bwi_poldiv_init(dfcx, dfcx0, vgrid%ax%n, vgrid%az%n)
       do iy=1, vgrid%ay%n
          stat = bwi_poldiv_oper(.true. ,.false.,tmpx,grdo(:,iy,:),selx(:,iy,:))
          grdo(:,iy,:) = tmpx
       end do
       stat = bwi_poldiv_clos()
    end if

    if (vgrid%ay%n>1) then
       stat = bwi_poldiv_init(dfcy, dfcy0, vgrid%ay%n, vgrid%az%n)
       do ix=1, vgrid%ax%n
          stat = bwi_poldiv_oper(.true. ,.false.,tmpy,grdo(ix,:,:),sely(ix,:,:))
          stat = bwi_poldiv_oper(.false.,.false.,tmpy,grdo(ix,:,:),sely(ix,:,:))
       end do
       stat = bwi_poldiv_clos()
    end if

    if (vgrid%ax%n>1) then
       stat = bwi_poldiv_init(dfcx, dfcx0, vgrid%ax%n, vgrid%az%n)
       do iy=1, vgrid%ay%n
          tmpx = grdo(:,iy,:) 
          stat = bwi_poldiv_oper(.false.,.false.,tmpx,grdo(:,iy,:),selx(:,iy,:))
       end do
       stat = bwi_poldiv_clos()
    end if 

    call bwi_fio_vmod_updt(grdo, vgrid, grdo_, "rite")
    deallocate(grdo)
    deallocate(tmpx)
    deallocate(tmpy)
    stat = 0
  end function

  integer function bwi_triflt_oper(grdo_, vgrid, rectx, recty, rectz, nsmt) result(stat)
    character(len=*)                    :: grdo_
    type(vel_grid)                      :: vgrid
    integer                             :: rectx, recty, rectz, nsmt

    real, dimension(:,:,:), allocatable :: grdo

    allocate(grdo(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    call bwi_fio_vmod_updt(grdo, vgrid, grdo_, "reed")
    stat = base_tris3d_oper(grdo, rectx, recty, rectz, nsmt)
    call bwi_fio_vmod_updt(grdo, vgrid, grdo_, "rite")
    deallocate(grdo)
    stat = 0
  end function

  integer function bwi_spline_oper(grdo_, vgrid, axk, ayk, azk) result(stat)
    character(len=*)                    :: grdo_
    type(vel_grid)                      :: vgrid
    type(axis)                          :: axk, ayk, azk

    real, dimension(:,:,:), allocatable :: grdo
    real, dimension(:,:,:), allocatable :: vspl

    allocate(grdo(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(vspl(axk%n,ayk%n,azk%n))
    stat = base_spln3d_init(vgrid%ax, vgrid%ay, vgrid%az, axk, ayk, azk)
    call bwi_fio_vmod_updt(grdo, vgrid, grdo_, "reed")
    stat = base_spln3d_oper(.true. , .false., vspl, grdo)
    stat = base_spln3d_oper(.false., .false., vspl, grdo)
    call bwi_fio_vmod_updt(grdo, vgrid, grdo_, "rite")
    stat = base_spln3d_clos()
    deallocate(grdo)
    deallocate(vspl)
    stat = 0
  end function

  integer function bwi_hesnor_oper(hess_, igrid, hgrid) result(stat)
    character(len=*)                      :: hess_
    type(img_grid)                        :: igrid
    type(hes_grid)                        :: hgrid

    integer                               :: izs
    real                                  :: hess_max

    real, dimension(:,:,:,:), allocatable :: hess

    if (igrid%ax%n /=hgrid%ax_modl%n ) call seperr("ERROR: igrid%ax%n/=hgrid%ax_modl%n")
    if (igrid%ay%n /=hgrid%ay_modl%n ) call seperr("ERROR: igrid%ay%n/=hgrid%ay_modl%n")
    if (igrid%az%n /=hgrid%az_modl%n ) call seperr("ERROR: igrid%az%n/=hgrid%az_modl%n")
    if (igrid%ahx%n/=hgrid%ahx_modl%n) call seperr("ERROR: igrid%ahx%n/=hgrid%ahx_modl%n")
    if (igrid%ahy%n/=hgrid%ahy_modl%n) call seperr("ERROR: igrid%ahy%n/=hgrid%ahy_modl%n") 
    allocate(hess(hgrid%ahx_modl%n,hgrid%ahy_modl%n,hgrid%ax_modl%n,hgrid%ay_modl%n))
    hess_max = 0
    do izs=1, igrid%az%n
       call bwi_fio_imag_updt(hess, igrid, izs, hess_, "reed")
       if (hess_max<=maxval(hess)) hess_max = maxval(hess)
    end do
    do izs=1, igrid%az%n
       call bwi_fio_imag_updt(hess, igrid, izs, hess_, "reed")
       hess = hess/hess_max
       call bwi_fio_imag_updt(hess, igrid, izs, hess_, "rite")
    end do
    deallocate(hess) 
    stat = 0
  end function

end module
