module bwi_histos_mod_iso
  use  sep
  use  base_ioutil_mod
  use  bwi_struct_mod
  implicit none

contains

  subroutine bwi_histos_imgfwd(dgrid, pgrid, vgrid, igrid, csou_, crec_, bvel_, bimg_)
    type(dat_grid)   :: dgrid
    type(pro_grid)   :: pgrid
    type(vel_grid)   :: vgrid
    type(img_grid)   :: igrid
    character(len=*) :: csou_, crec_, bvel_, bimg_

    type(axis)       :: agxcs, agycs, asxcs, asycs

!   ----------------
!   DEFINE DATA GRID
!   ----------------

    call from_param("ngx", dgrid%agx%n, 1 ); call from_param("dgx", dgrid%agx%d, 1.); call from_param("ogx", dgrid%agx%o, 0.)
    call from_param("ngy", dgrid%agy%n, 1 ); call from_param("dgy", dgrid%agy%d, 1.); call from_param("ogy", dgrid%agy%o, 0.)
    call from_param("nsx", dgrid%asx%n, 1 ); call from_param("dsx", dgrid%asx%d, 1.); call from_param("osx", dgrid%asx%o, 0.)
    call from_param("nsy", dgrid%asy%n, 1 ); call from_param("dsy", dgrid%asy%d, 1.); call from_param("osy", dgrid%asy%o, 0.)

    dgrid%ntrg = dgrid%agx%n*dgrid%agy%n
    dgrid%nshg = dgrid%asx%n*dgrid%asy%n

    call hinaxis(csou_, dgrid%aw, 3)

    if (dgrid%comsou) then
       call hinaxis(csou_, agxcs, 1)
       call hinaxis(csou_, agycs, 2)
       call hinaxis(csou_, asxcs, 4)
       call hinaxis(csou_, asycs, 5)
       dgrid%ntrs = agxcs%n * agycs%n           !!.. size of composite source
       dgrid%nshs = asxcs%n * asycs%n           !!.. number of composite sources
       write(0,*)"dgrid%ntrs=",dgrid%ntrs, "?", "   dgrid%ntrg=", dgrid%ntrg
       write(0,*)"dgrid%nshs=",dgrid%nshs, "?", "   dgrid%nshg=", dgrid%nshg
       if (dgrid%ntrs /= dgrid%ntrg) call seperr("ERROR: dgrid%ntrs /= dgrid%ntrg")
       if (dgrid%nshs /= dgrid%nshg) call seperr("ERROR: dgrid%nshs /= dgrid%nshg")
    else
       dgrid%ntrs = 1
       dgrid%nshs = 1
    end if

    if (dgrid%offset) then
       dgrid%gxmin_save = dgrid%agx%o
       dgrid%gymin_save = dgrid%agy%o
    end if

    call houaxis(crec_, dgrid%agx, 1)
    call houaxis(crec_, dgrid%agy, 2)
    call houaxis(crec_, dgrid%aw , 3)
    call houaxis(crec_, dgrid%asx, 4)
    call houaxis(crec_, dgrid%asy, 5)

    if (dgrid%offset) then
       call auxputch("label1", "s", "offx", crec_)
       call auxputch("label2", "s", "offy", crec_)
    else
       call auxputch("label1", "s", "gx"  , crec_)
       call auxputch("label2", "s", "gy"  , crec_)
    end if

    call auxputch("label3", "s", "freq", crec_)
    call auxputch("label4", "s", "sx"  , crec_)
    call auxputch("label5", "s", "sy"  , crec_)
    call auxputch("esize" , "i", 8     , crec_)

!   -----------------------------------
!   DEFINE IMAGING AND PROPAGATION GRID
!   -----------------------------------

!   padding for the wavefield propagation
!   -------------------------------------
    call from_param("nxpad_beg", pgrid%nxpad_beg, 0)
    call from_param("nxpad_end", pgrid%nxpad_end, 0)
    call from_param("nypad_beg", pgrid%nypad_beg, 0)
    call from_param("nypad_end", pgrid%nypad_end, 0)
    call from_param("nzpad_beg", pgrid%nzpad_beg, 0)
    call from_param("nzpad_end", pgrid%nzpad_end, 0)

!   velocity parameter initialization
!   ---------------------------------
    call hinaxis(bvel_, vgrid%ax, 1)
    call hinaxis(bvel_, vgrid%ay, 2)
    call hinaxis(bvel_, vgrid%az, 3)

!   read image axeses
!   -----------------
    call hinaxis(bimg_, igrid%ahx, 1)
    call hinaxis(bimg_, igrid%ahy, 2)
    call hinaxis(bimg_, igrid%ax , 3)
    call hinaxis(bimg_, igrid%ay , 4)
    call hinaxis(bimg_, igrid%az , 5)

    igrid%image_jx = nint(igrid%ax%d/vgrid%ax%d); igrid%image_dx = igrid%image_jx*vgrid%ax%d
    igrid%image_jy = nint(igrid%ay%d/vgrid%ay%d); igrid%image_dy = igrid%image_jy*vgrid%ay%d
    igrid%image_jz = nint(igrid%az%d/vgrid%az%d); igrid%image_dz = igrid%image_jz*vgrid%az%d

    if (igrid%stkimg) then
       igrid%asx%n=1; igrid%asx%o=0.; igrid%asx%d=1.
       igrid%asy%n=1; igrid%asy%o=0.; igrid%asy%d=1.
    else
       call hinaxis(bimg_, igrid%asx, 6)
       call hinaxis(bimg_, igrid%asy, 7)
    end if
    return
  end subroutine

  subroutine bwi_histos_imgadj(dgrid, pgrid, vgrid, igrid, csou_, crec_, bvel_, bimg_, hsou_)
    type(dat_grid)   :: dgrid
    type(pro_grid)   :: pgrid
    type(vel_grid)   :: vgrid
    type(img_grid)   :: igrid
    character(len=*) :: csou_, crec_, bvel_, bimg_, hsou_ 

    type(axis)       :: agxcs, agycs, asxcs, asycs
    real             :: image_xmin_default, image_xmax_default, image_dx_default
    real             :: image_ymin_default, image_ymax_default, image_dy_default
    real             :: image_zmin_default, image_zmax_default, image_dz_default
    integer          :: nshg

!   ----------------
!   DEFINE DATA GRID
!   ----------------

!   read the receiver coordinates, which are assumed to be regular
!   --------------------------------------------------------------
    call hinaxis(crec_, dgrid%agx, 1)
    call hinaxis(crec_, dgrid%agy, 2)
    call hinaxis(crec_, dgrid%aw , 3)
!    call hinaxis(crec_, dgrid%asx, 4)
!    call hinaxis(crec_, dgrid%asy, 5)

!   total receiver traces for one shot
!   ----------------------------------
    dgrid%ntrg = dgrid%agx%n * dgrid%agy%n

!   total shots
!   -----------
!    dgrid%nshg = dgrid%asx%n * dgrid%asy%n

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
if (dgrid%hdrsou) then
   call auxin(hsou_)
   call from_aux(hsou_, "n2", dgrid%nshg)
   call from_aux(crec_, "n4", nshg)
   if (dgrid%nshg /= nshg) call seperr("ERROR: hsou-nshg /= crec-nshg")
else
   call hinaxis(crec_, dgrid%asx, 4)
   call hinaxis(crec_, dgrid%asy, 5)
   dgrid%nshg = dgrid%asx%n * dgrid%asy%n
end if
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!   if composite sources are used 
!   -----------------------------
    if (dgrid%comsou) then
       call hinaxis(csou_, agxcs, 1)
       call hinaxis(csou_, agycs, 2)
       call hinaxis(csou_, asxcs, 4)
       call hinaxis(csou_, asycs, 5)
       dgrid%ntrs = agxcs%n * agycs%n           !!.. size of composite source
       dgrid%nshs = asxcs%n * asycs%n           !!.. number of composite sources
       if (dgrid%ntrs /= dgrid%ntrg) call seperr("ERROR: dgrid%ntrs /= dgrid%ntrg")
       if (dgrid%nshs /= dgrid%nshg) call seperr("ERROR: dgrid%nshs /= dgrid%nshg")
    else
       dgrid%ntrs = 1
       dgrid%nshs = 1
    end if

!   if marine acquisition geometry
!   ------------------------------
    if (dgrid%offset) then
       dgrid%gxmin_save = dgrid%agx%o
       dgrid%gymin_save = dgrid%agy%o
    end if

!   ---------------------------------
!   DEFINE IMAGE AND PROPAGATION GRID
!   ---------------------------------

!   padding for the wavefield propagation
!   -------------------------------------
    call from_param("nxpad_beg", pgrid%nxpad_beg, 0)
    call from_param("nxpad_end", pgrid%nxpad_end, 0)
    call from_param("nypad_beg", pgrid%nypad_beg, 0)
    call from_param("nypad_end", pgrid%nypad_end, 0)
    call from_param("nzpad_beg", pgrid%nzpad_beg, 0)
    call from_param("nzpad_end", pgrid%nzpad_end, 0)

!   velocity parameter initialization
!   ---------------------------------
    call hinaxis(bvel_, vgrid%ax, 1)
    call hinaxis(bvel_, vgrid%ay, 2)
    call hinaxis(bvel_, vgrid%az, 3)

!   the default image size is set to be the same as the velocity model
!   ------------------------------------------------------------------
    image_xmin_default=vgrid%ax%o; image_dx_default=vgrid%ax%d; image_xmax_default=vgrid%ax%o+(vgrid%ax%n-1)*vgrid%ax%d
    image_ymin_default=vgrid%ay%o; image_dy_default=vgrid%ay%d; image_ymax_default=vgrid%ay%o+(vgrid%ay%n-1)*vgrid%ay%d
    image_zmin_default=vgrid%az%o; image_dz_default=vgrid%az%d; image_zmax_default=vgrid%az%o+(vgrid%az%n-1)*vgrid%az%d

!   read the imaging parameters
!   ---------------------------
    call from_param("image_xmin", igrid%image_xmin, image_xmin_default)
    call from_param("image_xmax", igrid%image_xmax, image_xmax_default)
    call from_param("image_ymin", igrid%image_ymin, image_ymin_default)
    call from_param("image_ymax", igrid%image_ymax, image_ymax_default)
    call from_param("image_zmin", igrid%image_zmin, image_zmin_default)
    call from_param("image_zmax", igrid%image_zmax, image_zmax_default)
    call from_param("image_dx"  , igrid%image_dx  , image_dx_default  )
    call from_param("image_dy"  , igrid%image_dy  , image_dy_default  )
    call from_param("image_dz"  , igrid%image_dz  , image_dz_default  )

!   safe guarding
!   -------------
    igrid%image_xmin = vgrid%ax%o + int(1.5+(igrid%image_xmin-vgrid%ax%o)/vgrid%ax%d-1)*vgrid%ax%d
    igrid%image_ymin = vgrid%ay%o + int(1.5+(igrid%image_ymin-vgrid%ay%o)/vgrid%ay%d-1)*vgrid%ay%d
    igrid%image_zmin = vgrid%az%o + int(1.5+(igrid%image_zmin-vgrid%az%o)/vgrid%az%d-1)*vgrid%az%d

    igrid%image_xmax = vgrid%ax%o + int(1.5+(igrid%image_xmax-vgrid%ax%o)/vgrid%ax%d-1)*vgrid%ax%d
    igrid%image_ymax = vgrid%ay%o + int(1.5+(igrid%image_ymax-vgrid%ay%o)/vgrid%ay%d-1)*vgrid%ay%d
    igrid%image_zmax = vgrid%az%o + int(1.5+(igrid%image_zmax-vgrid%az%o)/vgrid%az%d-1)*vgrid%az%d

    igrid%image_jx = nint(igrid%image_dx/vgrid%ax%d); igrid%image_dx = igrid%image_jx*vgrid%ax%d
    igrid%image_jy = nint(igrid%image_dy/vgrid%ay%d); igrid%image_dy = igrid%image_jy*vgrid%ay%d
    igrid%image_jz = nint(igrid%image_dz/vgrid%az%d); igrid%image_dz = igrid%image_jz*vgrid%az%d

    if(igrid%image_xmin < image_xmin_default) call seperr("image_xmin is out of the input velocity model")
    if(igrid%image_xmax > image_xmax_default) call seperr("image_xmax is out of the input velocity model")
    if(igrid%image_ymin < image_ymin_default) call seperr("image_ymin is out of the input velocity model")
    if(igrid%image_ymax > image_ymax_default) call seperr("image_ymax is out of the input velocity model")
    if(igrid%image_zmin < image_zmin_default) call seperr("image_zmin is out of the input velocity model")
    if(igrid%image_zmax > image_zmax_default) call seperr("image_zmax is out of the input velocity model")

!   define the image grid
!   ---------------------
    igrid%ax%o=igrid%image_xmin; igrid%ax%d=igrid%image_dx; igrid%ax%n=1.5+(igrid%image_xmax-igrid%image_xmin)/igrid%image_dx
    igrid%ay%o=igrid%image_ymin; igrid%ay%d=igrid%image_dy; igrid%ay%n=1.5+(igrid%image_ymax-igrid%image_ymin)/igrid%image_dy
    igrid%az%o=igrid%image_zmin; igrid%az%d=igrid%image_dz; igrid%az%n=1.5+(igrid%image_zmax-igrid%image_zmin)/igrid%image_dz

    call from_param("nhx", igrid%ahx%n, 1); call from_param("dhx", igrid%ahx%d, vgrid%ax%d); call from_param("ohx", igrid%ahx%o, 0.)
    call from_param("nhy", igrid%ahy%n, 1); call from_param("dhy", igrid%ahy%d, vgrid%ay%d); call from_param("ohy", igrid%ahy%o, 0.) 
    call from_param("nhz", igrid%ahz%n, 1); call from_param("dhz", igrid%ahz%d, vgrid%az%d); call from_param("ohz", igrid%ahz%o, 0.)

!   write out image history
!   -----------------------
    call houaxis(bimg_, igrid%ahx, 1); call auxputch("label1", "s", "hx", bimg_)
    call houaxis(bimg_, igrid%ahy, 2); call auxputch("label2", "s", "hy", bimg_)
    call houaxis(bimg_, igrid%ax , 3); call auxputch("label3", "s", "mx", bimg_)
    call houaxis(bimg_, igrid%ay , 4); call auxputch("label4", "s", "my", bimg_)
    call houaxis(bimg_, igrid%az , 5); call auxputch("label5", "s", "mz", bimg_)

!   output image without stacking over shot axis
!   -------------------------------------------- 
    if (igrid%stkimg) then
       igrid%asx%n=1; igrid%asx%o=0.; igrid%asx%d=1.
       igrid%asy%n=1; igrid%asy%o=0.; igrid%asy%d=1.
    else
       call hinaxis(crec_, igrid%asx, 4)
       call hinaxis(crec_, igrid%asy, 5)
       call houaxis(bimg_, igrid%asx, 6)
       call houaxis(bimg_, igrid%asy, 7)
       call auxputch("label6", "s", "sx", bimg_)
       call auxputch("label7", "s", "sy", bimg_)
    end if

!   output complex or real image
!   ----------------------------
    if (igrid%cpximg) then
       call auxputch("esize", "i", 8, bimg_)
    else
       call auxputch("esize", "i", 4, bimg_)
    end if
    return
  end subroutine

  subroutine bwi_histos_imginv()
  end subroutine

  subroutine bwi_histos_tomfwd(dgrid, pgrid, vgrid, igrid, csou_, crec_, bvel_, dvel_, dimg_)
    type(dat_grid)   :: dgrid
    type(pro_grid)   :: pgrid
    type(vel_grid)   :: vgrid
    type(img_grid)   :: igrid
    character(len=*) :: csou_, crec_, bvel_, dvel_, dimg_

    type(axis)       :: agxcs, agycs, asxcs, asycs, ax, ay, az
    real             :: image_xmin_default, image_xmax_default, image_dx_default
    real             :: image_ymin_default, image_ymax_default, image_dy_default
    real             :: image_zmin_default, image_zmax_default, image_dz_default

!   read the receiver coordinates, which are assumed to be regular
!   --------------------------------------------------------------
    call hinaxis(crec_, dgrid%agx, 1)
    call hinaxis(crec_, dgrid%agy, 2)
    call hinaxis(crec_, dgrid%aw , 3)
    call hinaxis(crec_, dgrid%asx, 4)
    call hinaxis(crec_, dgrid%asy, 5)

!   total receiver traces for one shot
!   ----------------------------------
    dgrid%ntrg = dgrid%agx%n * dgrid%agy%n

!   total number of shots
!   ---------------------
    dgrid%nshg = dgrid%asx%n * dgrid%asy%n

!   if composite sources are used 
!   -----------------------------
    if (dgrid%comsou) then
       call hinaxis(csou_, agxcs, 1)
       call hinaxis(csou_, agycs, 2)
       call hinaxis(csou_, asxcs, 4)
       call hinaxis(csou_, asycs, 5)
       dgrid%ntrs = agxcs%n * agycs%n           !!.. size of composite source
       dgrid%nshs = asxcs%n * asycs%n           !!.. number of composite sources
       if (dgrid%ntrs /= dgrid%ntrg) call seperr("ERROR: dgrid%ntrs /= dgrid%ntrg")
       if (dgrid%nshs /= dgrid%nshg) call seperr("ERROR: dgrid%nshs /= dgrid%nshg")
    else
       dgrid%ntrs = 1
       dgrid%nshs = 1
    end if

!   if marine acquisition geometry
!   ------------------------------
    if (dgrid%offset) then
       dgrid%gxmin_save = dgrid%agx%o
       dgrid%gymin_save = dgrid%agy%o
    end if

!   DEFINE IMAGE AND PROPAGATION GRID
!   ---------------------------------

!   padding for the wavefield propagation
!   -------------------------------------
    call from_param("nxpad_beg", pgrid%nxpad_beg, 0)
    call from_param("nxpad_end", pgrid%nxpad_end, 0)
    call from_param("nypad_beg", pgrid%nypad_beg, 0)
    call from_param("nypad_end", pgrid%nypad_end, 0)
    call from_param("nzpad_beg", pgrid%nzpad_beg, 0)
    call from_param("nzpad_end", pgrid%nzpad_end, 0)

!   velocity parameter initialization
!   ---------------------------------
    call hinaxis(bvel_, vgrid%ax, 1)
    call hinaxis(bvel_, vgrid%ay, 2)
    call hinaxis(bvel_, vgrid%az, 3)

    call hinaxis(dvel_, ax, 1)
    call hinaxis(dvel_, ay, 2)
    call hinaxis(dvel_, az, 3)

    if (vgrid%ax%n/=ax%n .or. vgrid%ax%d/=ax%d .or. vgrid%ax%o/=ax%o) call seperr("ERROR: x's in bvel_ and dvel_ don't match")
    if (vgrid%ay%n/=ay%n .or. vgrid%ay%d/=ay%d .or. vgrid%ay%o/=ay%o) call seperr("ERROR: y's in bvel_ and dvel_ don't match")
    if (vgrid%az%n/=az%n .or. vgrid%az%d/=az%d .or. vgrid%az%o/=az%o) call seperr("ERROR: z's in bvel_ and dvel_ don't match")

!   the default image size is set to be the same as the velocity model
!   ------------------------------------------------------------------
    image_xmin_default=vgrid%ax%o; image_dx_default=vgrid%ax%d; image_xmax_default=vgrid%ax%o+(vgrid%ax%n-1)*vgrid%ax%d
    image_ymin_default=vgrid%ay%o; image_dy_default=vgrid%ay%d; image_ymax_default=vgrid%ay%o+(vgrid%ay%n-1)*vgrid%ay%d
    image_zmin_default=vgrid%az%o; image_dz_default=vgrid%az%d; image_zmax_default=vgrid%az%o+(vgrid%az%n-1)*vgrid%az%d

!   read the imaging parameters
!   ---------------------------
    call from_param("image_xmin", igrid%image_xmin, image_xmin_default)
    call from_param("image_xmax", igrid%image_xmax, image_xmax_default)
    call from_param("image_ymin", igrid%image_ymin, image_ymin_default)
    call from_param("image_ymax", igrid%image_ymax, image_ymax_default)
    call from_param("image_zmin", igrid%image_zmin, image_zmin_default)
    call from_param("image_zmax", igrid%image_zmax, image_zmax_default)
    call from_param("image_dx"  , igrid%image_dx  , image_dx_default  )
    call from_param("image_dy"  , igrid%image_dy  , image_dy_default  )
    call from_param("image_dz"  , igrid%image_dz  , image_dz_default  )

!   safe guarding
!   -------------
    igrid%image_xmin = vgrid%ax%o + int(1.5+(igrid%image_xmin-vgrid%ax%o)/vgrid%ax%d-1)*vgrid%ax%d
    igrid%image_ymin = vgrid%ay%o + int(1.5+(igrid%image_ymin-vgrid%ay%o)/vgrid%ay%d-1)*vgrid%ay%d
    igrid%image_zmin = vgrid%az%o + int(1.5+(igrid%image_zmin-vgrid%az%o)/vgrid%az%d-1)*vgrid%az%d

    igrid%image_xmax = vgrid%ax%o + int(1.5+(igrid%image_xmax-vgrid%ax%o)/vgrid%ax%d-1)*vgrid%ax%d
    igrid%image_ymax = vgrid%ay%o + int(1.5+(igrid%image_ymax-vgrid%ay%o)/vgrid%ay%d-1)*vgrid%ay%d
    igrid%image_zmax = vgrid%az%o + int(1.5+(igrid%image_zmax-vgrid%az%o)/vgrid%az%d-1)*vgrid%az%d

    igrid%image_jx = nint(igrid%image_dx/vgrid%ax%d); igrid%image_dx = igrid%image_jx*vgrid%ax%d
    igrid%image_jy = nint(igrid%image_dy/vgrid%ay%d); igrid%image_dy = igrid%image_jy*vgrid%ay%d
    igrid%image_jz = nint(igrid%image_dz/vgrid%az%d); igrid%image_dz = igrid%image_jz*vgrid%az%d

    if(igrid%image_xmin < image_xmin_default) call seperr("image_xmin is out of the input velocity model")
    if(igrid%image_xmax > image_xmax_default) call seperr("image_xmax is out of the input velocity model")
    if(igrid%image_ymin < image_ymin_default) call seperr("image_ymin is out of the input velocity model")
    if(igrid%image_ymax > image_ymax_default) call seperr("image_ymax is out of the input velocity model")
    if(igrid%image_zmin < image_zmin_default) call seperr("image_zmin is out of the input velocity model")
    if(igrid%image_zmax > image_zmax_default) call seperr("image_zmax is out of the input velocity model")

!   define the image grid
!   ---------------------
    igrid%ax%o=igrid%image_xmin; igrid%ax%d=igrid%image_dx; igrid%ax%n=1.5+(igrid%image_xmax-igrid%image_xmin)/igrid%image_dx
    igrid%ay%o=igrid%image_ymin; igrid%ay%d=igrid%image_dy; igrid%ay%n=1.5+(igrid%image_ymax-igrid%image_ymin)/igrid%image_dy
    igrid%az%o=igrid%image_zmin; igrid%az%d=igrid%image_dz; igrid%az%n=1.5+(igrid%image_zmax-igrid%image_zmin)/igrid%image_dz

    call from_param("nhx", igrid%ahx%n, 1); call from_param("dhx", igrid%ahx%d, vgrid%ax%d); call from_param("ohx", igrid%ahx%o, 0.)
    call from_param("nhy", igrid%ahy%n, 1); call from_param("dhy", igrid%ahy%d, vgrid%ay%d); call from_param("ohy", igrid%ahy%o, 0.)
    call from_param("nhz", igrid%ahz%n, 1); call from_param("dhz", igrid%ahz%d, vgrid%az%d); call from_param("ohz", igrid%ahz%o, 0.)

!   write out image history
!   -----------------------
    call houaxis(dimg_, igrid%ahx, 1); call auxputch("label1", "s", "hx", dimg_)
    call houaxis(dimg_, igrid%ahy, 2); call auxputch("label2", "s", "hy", dimg_)
    call houaxis(dimg_, igrid%ax , 3); call auxputch("label3", "s", "mx", dimg_)
    call houaxis(dimg_, igrid%ay , 4); call auxputch("label4", "s", "my", dimg_)
    call houaxis(dimg_, igrid%az , 5); call auxputch("label5", "s", "mz", dimg_)

!   output image without stacking over shot axis
!   -------------------------------------------- 
    if (igrid%stkimg) then
       igrid%asx%n=1; igrid%asx%o=0.; igrid%asx%d=1.
       igrid%asy%n=1; igrid%asy%o=0.; igrid%asy%d=1.
    else
       call hinaxis(crec_, igrid%asx, 4)
       call hinaxis(crec_, igrid%asy, 5)
       call houaxis(dimg_, igrid%asx, 6)
       call houaxis(dimg_, igrid%asy, 7)
       call auxputch("label6", "s", "sx", dimg_)
       call auxputch("label7", "s", "sy", dimg_)
    end if
    return
  end subroutine

  subroutine bwi_histos_tomadj(dgrid, pgrid, vgrid, igrid, csou_, crec_, bvel_, dvel_, dimg_)
    type(dat_grid)   :: dgrid
    type(pro_grid)   :: pgrid
    type(vel_grid)   :: vgrid
    type(img_grid)   :: igrid
    character(len=*) :: csou_, crec_, bvel_, dvel_, dimg_

    type(axis)       :: agxcs, agycs, asxcs, asycs

!   read the receiver coordinates, which are assumed to be regular
!   --------------------------------------------------------------
    call hinaxis(crec_, dgrid%agx, 1)
    call hinaxis(crec_, dgrid%agy, 2)
    call hinaxis(crec_, dgrid%aw , 3)
    call hinaxis(crec_, dgrid%asx, 4)
    call hinaxis(crec_, dgrid%asy, 5)

!   total receiver traces for one shot
!   ----------------------------------
    dgrid%ntrg = dgrid%agx%n * dgrid%agy%n

!   total number of shots
!   ---------------------
    dgrid%nshg = dgrid%asx%n * dgrid%asy%n

!   if composite sources are used 
!   -----------------------------
    if (dgrid%comsou) then
       call hinaxis(csou_, agxcs, 1)
       call hinaxis(csou_, agycs, 2)
       call hinaxis(csou_, asxcs, 4)
       call hinaxis(csou_, asycs, 5)
       dgrid%ntrs = agxcs%n * agycs%n           !!.. size of composite source
       dgrid%nshs = asxcs%n * asycs%n           !!.. number of composite sources
       if (dgrid%ntrs /= dgrid%ntrg) call seperr("ERROR: dgrid%ntrs /= dgrid%ntrg")
       if (dgrid%nshs /= dgrid%nshg) call seperr("ERROR: dgrid%nshs /= dgrid%nshg")
    else
       dgrid%ntrs = 1
       dgrid%nshs = 1
    end if

!   if marine acquisition geometry
!   ------------------------------
    if (dgrid%offset) then
       dgrid%gxmin_save = dgrid%agx%o
       dgrid%gymin_save = dgrid%agy%o
    end if

!   DEFINE IMAGE AND PROPAGATION GRID
!   ---------------------------------

!   padding for the wavefield propagation
!   -------------------------------------
    call from_param("nxpad_beg", pgrid%nxpad_beg, 0)
    call from_param("nxpad_end", pgrid%nxpad_end, 0)
    call from_param("nypad_beg", pgrid%nypad_beg, 0)
    call from_param("nypad_end", pgrid%nypad_end, 0)
    call from_param("nzpad_beg", pgrid%nzpad_beg, 0)
    call from_param("nzpad_end", pgrid%nzpad_end, 0)

!   velocity parameter initialization
!   ---------------------------------
    call hinaxis(bvel_, vgrid%ax, 1)
    call hinaxis(bvel_, vgrid%ay, 2)
    call hinaxis(bvel_, vgrid%az, 3)

!   read image axeses
!   -----------------
    call hinaxis(dimg_, igrid%ahx, 1)
    call hinaxis(dimg_, igrid%ahy, 2)
    call hinaxis(dimg_, igrid%ax , 3)
    call hinaxis(dimg_, igrid%ay , 4)
    call hinaxis(dimg_, igrid%az , 5)

    igrid%image_jx = nint(igrid%ax%d/vgrid%ax%d); igrid%image_dx = igrid%image_jx*vgrid%ax%d
    igrid%image_jy = nint(igrid%ay%d/vgrid%ay%d); igrid%image_dy = igrid%image_jy*vgrid%ay%d
    igrid%image_jz = nint(igrid%az%d/vgrid%az%d); igrid%image_dz = igrid%image_jz*vgrid%az%d

    if (igrid%stkimg) then
       igrid%asx%n=1; igrid%asx%o=0.; igrid%asx%d=1.
       igrid%asy%n=1; igrid%asy%o=0.; igrid%asy%d=1.
    else
       call hinaxis(dimg_, igrid%asx, 6)
       call hinaxis(dimg_, igrid%asy, 7)
    end if

    call houaxis(dvel_, vgrid%ax, 1); call auxputch("label1", "s", "x", dvel_)
    call houaxis(dvel_, vgrid%ay, 2); call auxputch("label2", "s", "y", dvel_)
    call houaxis(dvel_, vgrid%az, 3); call auxputch("label3", "s", "z", dvel_)

    return 
  end subroutine

  subroutine bwi_histos_tominv(dgrid, pgrid, vgrid, igrid, hgrid, csou_, crec_, bvel_, bimg_, hess_, grdo_, grdp_, srch_)
    type(dat_grid)   :: dgrid
    type(pro_grid)   :: pgrid
    type(vel_grid)   :: vgrid
    type(img_grid)   :: igrid
    type(hes_grid)   :: hgrid
    character(len=*) :: csou_, crec_, bvel_, bimg_, hess_, grdo_, grdp_, srch_

    type(axis)       :: agxcs, agycs, asxcs, asycs
    real             :: image_xmin_default, image_xmax_default, image_dx_default
    real             :: image_ymin_default, image_ymax_default, image_dy_default
    real             :: image_zmin_default, image_zmax_default, image_dz_default

!   read the receiver coordinates, which are assumed to be regular
!   --------------------------------------------------------------
    call hinaxis(crec_, dgrid%agx, 1)
    call hinaxis(crec_, dgrid%agy, 2)
    call hinaxis(crec_, dgrid%aw , 3)
    call hinaxis(crec_, dgrid%asx, 4)
    call hinaxis(crec_, dgrid%asy, 5)

!   total receiver traces for one shot
!   ----------------------------------
    dgrid%ntrg = dgrid%agx%n * dgrid%agy%n

!   total number of shots
!   ---------------------
    dgrid%nshg = dgrid%asx%n * dgrid%asy%n

!   if composite sources are used 
!   -----------------------------
    if (dgrid%comsou) then
       call hinaxis(csou_, agxcs, 1)
       call hinaxis(csou_, agycs, 2)
       call hinaxis(csou_, asxcs, 4)
       call hinaxis(csou_, asycs, 5)
       dgrid%ntrs = agxcs%n * agycs%n           !!.. size of composite source
       dgrid%nshs = asxcs%n * asycs%n           !!.. number of composite sources
       if (dgrid%ntrs /= dgrid%ntrg) call seperr("ERROR: dgrid%ntrs /= dgrid%ntrg")
       if (dgrid%nshs /= dgrid%nshg) call seperr("ERROR: dgrid%nshs /= dgrid%nshg")
    else
       dgrid%ntrs = 1
       dgrid%nshs = 1
    end if

!   if marine acquisition geometry
!   ------------------------------
    if (dgrid%offset) then
       dgrid%gxmin_save = dgrid%agx%o
       dgrid%gymin_save = dgrid%agy%o
    end if

!   ---------------------------------
!   DEFINE IMAGE AND PROPAGATION GRID
!   ---------------------------------

!   padding for the wavefield propagation
!   -------------------------------------
    call from_param("nxpad_beg", pgrid%nxpad_beg, 0)
    call from_param("nxpad_end", pgrid%nxpad_end, 0)
    call from_param("nypad_beg", pgrid%nypad_beg, 0)
    call from_param("nypad_end", pgrid%nypad_end, 0)
    call from_param("nzpad_beg", pgrid%nzpad_beg, 0)
    call from_param("nzpad_end", pgrid%nzpad_end, 0)

!   velocity parameter initialization
!   ---------------------------------
    call hinaxis(bvel_, vgrid%ax, 1)
    call hinaxis(bvel_, vgrid%ay, 2)
    call hinaxis(bvel_, vgrid%az, 3)

!   the default image size is set to be the same as the velocity model
!   ------------------------------------------------------------------
    image_xmin_default=vgrid%ax%o; image_dx_default=vgrid%ax%d; image_xmax_default=vgrid%ax%o+(vgrid%ax%n-1)*vgrid%ax%d
    image_ymin_default=vgrid%ay%o; image_dy_default=vgrid%ay%d; image_ymax_default=vgrid%ay%o+(vgrid%ay%n-1)*vgrid%ay%d
    image_zmin_default=vgrid%az%o; image_dz_default=vgrid%az%d; image_zmax_default=vgrid%az%o+(vgrid%az%n-1)*vgrid%az%d

!   read the imaging parameters
!   ---------------------------
    call from_param("image_xmin", igrid%image_xmin, image_xmin_default)
    call from_param("image_xmax", igrid%image_xmax, image_xmax_default)
    call from_param("image_ymin", igrid%image_ymin, image_ymin_default)
    call from_param("image_ymax", igrid%image_ymax, image_ymax_default)
    call from_param("image_zmin", igrid%image_zmin, image_zmin_default)
    call from_param("image_zmax", igrid%image_zmax, image_zmax_default)
    call from_param("image_dx"  , igrid%image_dx  , image_dx_default  )
    call from_param("image_dy"  , igrid%image_dy  , image_dy_default  )
    call from_param("image_dz"  , igrid%image_dz  , image_dz_default  )

!   safe guarding
!   -------------
    igrid%image_xmin = vgrid%ax%o + int(1.5+(igrid%image_xmin-vgrid%ax%o)/vgrid%ax%d-1)*vgrid%ax%d
    igrid%image_ymin = vgrid%ay%o + int(1.5+(igrid%image_ymin-vgrid%ay%o)/vgrid%ay%d-1)*vgrid%ay%d
    igrid%image_zmin = vgrid%az%o + int(1.5+(igrid%image_zmin-vgrid%az%o)/vgrid%az%d-1)*vgrid%az%d

    igrid%image_xmax = vgrid%ax%o + int(1.5+(igrid%image_xmax-vgrid%ax%o)/vgrid%ax%d-1)*vgrid%ax%d
    igrid%image_ymax = vgrid%ay%o + int(1.5+(igrid%image_ymax-vgrid%ay%o)/vgrid%ay%d-1)*vgrid%ay%d
    igrid%image_zmax = vgrid%az%o + int(1.5+(igrid%image_zmax-vgrid%az%o)/vgrid%az%d-1)*vgrid%az%d

    igrid%image_jx = nint(igrid%image_dx/vgrid%ax%d); igrid%image_dx = igrid%image_jx*vgrid%ax%d
    igrid%image_jy = nint(igrid%image_dy/vgrid%ay%d); igrid%image_dy = igrid%image_jy*vgrid%ay%d
    igrid%image_jz = nint(igrid%image_dz/vgrid%az%d); igrid%image_dz = igrid%image_jz*vgrid%az%d

    if(igrid%image_xmin < image_xmin_default) call seperr("image_xmin is out of the input velocity model")
    if(igrid%image_xmax > image_xmax_default) call seperr("image_xmax is out of the input velocity model")
    if(igrid%image_ymin < image_ymin_default) call seperr("image_ymin is out of the input velocity model")
    if(igrid%image_ymax > image_ymax_default) call seperr("image_ymax is out of the input velocity model")
    if(igrid%image_zmin < image_zmin_default) call seperr("image_zmin is out of the input velocity model")
    if(igrid%image_zmax > image_zmax_default) call seperr("image_zmax is out of the input velocity model")

!   define the image grid
!   ---------------------
    igrid%ax%o=igrid%image_xmin; igrid%ax%d=igrid%image_dx; igrid%ax%n=1.5+(igrid%image_xmax-igrid%image_xmin)/igrid%image_dx
    igrid%ay%o=igrid%image_ymin; igrid%ay%d=igrid%image_dy; igrid%ay%n=1.5+(igrid%image_ymax-igrid%image_ymin)/igrid%image_dy
    igrid%az%o=igrid%image_zmin; igrid%az%d=igrid%image_dz; igrid%az%n=1.5+(igrid%image_zmax-igrid%image_zmin)/igrid%image_dz

    call from_param("nhx", igrid%ahx%n, 1); call from_param("dhx", igrid%ahx%d, vgrid%ax%d); call from_param("ohx", igrid%ahx%o, 0.)
    call from_param("nhy", igrid%ahy%n, 1); call from_param("dhy", igrid%ahy%d, vgrid%ay%d); call from_param("ohy", igrid%ahy%o, 0.)
    call from_param("nhz", igrid%ahz%n, 1); call from_param("dhz", igrid%ahz%d, vgrid%az%d); call from_param("ohz", igrid%ahz%o, 0.)

!   write out background image history
!   ----------------------------------
    call houaxis(bimg_, igrid%ahx, 1); call auxputch("label1", "s", "hx", bimg_)
    call houaxis(bimg_, igrid%ahy, 2); call auxputch("label2", "s", "hy", bimg_)
    call houaxis(bimg_, igrid%ax , 3); call auxputch("label3", "s", "mx", bimg_)
    call houaxis(bimg_, igrid%ay , 4); call auxputch("label4", "s", "my", bimg_)
    call houaxis(bimg_, igrid%az , 5); call auxputch("label5", "s", "mz", bimg_)

!   output image without stacking over shot axis
!   -------------------------------------------- 
    if (igrid%stkimg) then
       igrid%asx%n=1; igrid%asx%o=0.; igrid%asx%d=1.
       igrid%asy%n=1; igrid%asy%o=0.; igrid%asy%d=1.
    else
       igrid%asx = dgrid%asx
       igrid%asy = dgrid%asy 
       call houaxis(bimg_, igrid%asx, 6)
       call houaxis(bimg_, igrid%asy, 7)
       call auxputch("label6", "s", "sx", bimg_)
       call auxputch("label7", "s", "sy", bimg_)
    end if

!   hessian
!   ------- 
    if (hgrid%wnthes) then
       hgrid%ahx_modl = igrid%ahx
       hgrid%ahy_modl = igrid%ahy
       hgrid%ax_modl  = igrid%ax
       hgrid%ay_modl  = igrid%ay
       hgrid%az_modl  = igrid%az
       hgrid%asx      = igrid%asx
       hgrid%asy      = igrid%asy
       call houaxis(hess_, hgrid%ahx_modl, 1); call auxputch("label1", "s", "hx_modl", hess_)
       call houaxis(hess_, hgrid%ahy_modl, 2); call auxputch("label2", "s", "hy_modl", hess_)
       call houaxis(hess_, hgrid%ax_modl , 3); call auxputch("label3", "s", "mx_modl", hess_)
       call houaxis(hess_, hgrid%ay_modl , 4); call auxputch("label4", "s", "my_modl", hess_)
       call houaxis(hess_, hgrid%az_modl , 5); call auxputch("label5", "s", "mz_modl", hess_)
       if (.not.igrid%stkimg) then
          call houaxis(hess_, hgrid%asx, 6); call auxputch("label6", "s", "sx", hess_)
          call houaxis(hess_, hgrid%asy, 7); call auxputch("label7", "s", "sy", hess_)
       end if
    end if 

!   aux files
!   ---------
    call houaxis(grdo_, vgrid%ax, 1); call houaxis(grdo_, vgrid%ay, 2); call houaxis(grdo_, vgrid%az, 3)
    call houaxis(grdp_, vgrid%ax, 1); call houaxis(grdp_, vgrid%ay, 2); call houaxis(grdp_, vgrid%az, 3)
    call houaxis(srch_, vgrid%ax, 1); call houaxis(srch_, vgrid%ay, 2); call houaxis(srch_, vgrid%az, 3)

    return
  end subroutine
 
  subroutine bwi_histos_hesian(dgrid, pgrid, vgrid, igrid, hgrid, sfnc_, gsou_, grec_, bvel_, hess_)
    type(dat_grid)   :: dgrid
    type(pro_grid)   :: pgrid
    type(vel_grid)   :: vgrid
    type(img_grid)   :: igrid
    type(hes_grid)   :: hgrid
    character(len=*) :: sfnc_, gsou_, grec_, bvel_, hess_

    type(axis)       :: agxcs, agycs, asxcs, asycs
    real             :: image_xmin_default, image_xmax_default, image_dx_default
    real             :: image_ymin_default, image_ymax_default, image_dy_default
    real             :: image_zmin_default, image_zmax_default, image_dz_default

!   ----------------
!   DEFINE DATA GRID
!   ----------------

!   read the receiver coordinates, which are assumed to be regular
!   --------------------------------------------------------------

    if (hgrid%grecin) then
       call hinaxis(grec_, dgrid%agx, 1)
       call hinaxis(grec_, dgrid%agy, 2)
       call hinaxis(grec_, dgrid%aw , 3)
       call hinaxis(grec_, dgrid%asx, 4)
       call hinaxis(grec_, dgrid%asy, 5)
    else
       call from_param("ngx", dgrid%agx%n, 1); call from_param("dgx", dgrid%agx%d, 1.); call from_param("ogx", dgrid%agx%o, 0.)
       call from_param("ngy", dgrid%agy%n, 1); call from_param("dgy", dgrid%agy%d, 1.); call from_param("ogy", dgrid%agy%o, 0.)
       call from_param("nsx", dgrid%asx%n, 1); call from_param("dsx", dgrid%asx%d, 1.); call from_param("osx", dgrid%asx%o, 0.)
       call from_param("nsy", dgrid%asy%n, 1); call from_param("dsy", dgrid%asy%d, 1.); call from_param("osy", dgrid%asy%o, 0.)
       call hinaxis(gsou_, dgrid%aw, 3)
    end if
!   total receiver traces for one shot
!   ----------------------------------
    dgrid%ntrg = dgrid%agx%n * dgrid%agy%n
!   total shots
!   -----------
    dgrid%nshg = dgrid%asx%n * dgrid%asy%n

!   if composite sources are used 
!   -----------------------------
    if (dgrid%comsou) then
       call hinaxis(gsou_, agxcs, 1)
       call hinaxis(gsou_, agycs, 2)
       call hinaxis(gsou_, asxcs, 4)
       call hinaxis(gsou_, asycs, 5)
       dgrid%ntrs = agxcs%n * agycs%n           !!.. size of composite source
       dgrid%nshs = asxcs%n * asycs%n           !!.. number of composite sources
       if (dgrid%ntrs /= dgrid%ntrg) call seperr("ERROR: dgrid%ntrs /= dgrid%ntrg")
       if (dgrid%nshs /= dgrid%nshg) call seperr("ERROR: dgrid%nshs /= dgrid%nshg")
    else
       dgrid%ntrs = 1
       dgrid%nshs = 1
    end if

!   if marine acquisition geometry
!   ------------------------------
    if (dgrid%offset) then
       dgrid%gxmin_save = dgrid%agx%o
       dgrid%gymin_save = dgrid%agy%o
    end if

!   ---------------------------------
!   DEFINE IMAGE AND PROPAGATION GRID
!   ---------------------------------

!   padding for the wavefield propagation
!   -------------------------------------
    call from_param("nxpad_beg", pgrid%nxpad_beg, 0)
    call from_param("nxpad_end", pgrid%nxpad_end, 0)
    call from_param("nypad_beg", pgrid%nypad_beg, 0)
    call from_param("nypad_end", pgrid%nypad_end, 0)
    call from_param("nzpad_beg", pgrid%nzpad_beg, 0)
    call from_param("nzpad_end", pgrid%nzpad_end, 0)

!   velocity parameter initialization
!   ---------------------------------
    call hinaxis(bvel_, vgrid%ax, 1)
    call hinaxis(bvel_, vgrid%ay, 2)
    call hinaxis(bvel_, vgrid%az, 3)

!   the default image size is set to be the same as the velocity model
!   ------------------------------------------------------------------
    image_xmin_default=vgrid%ax%o; image_dx_default=vgrid%ax%d; image_xmax_default=vgrid%ax%o+(vgrid%ax%n-1)*vgrid%ax%d
    image_ymin_default=vgrid%ay%o; image_dy_default=vgrid%ay%d; image_ymax_default=vgrid%ay%o+(vgrid%ay%n-1)*vgrid%ay%d
    image_zmin_default=vgrid%az%o; image_dz_default=vgrid%az%d; image_zmax_default=vgrid%az%o+(vgrid%az%n-1)*vgrid%az%d

!   read the imaging parameters
!   ---------------------------
    call from_param("image_xmin", igrid%image_xmin, image_xmin_default)
    call from_param("image_xmax", igrid%image_xmax, image_xmax_default)
    call from_param("image_ymin", igrid%image_ymin, image_ymin_default)
    call from_param("image_ymax", igrid%image_ymax, image_ymax_default)
    call from_param("image_zmin", igrid%image_zmin, image_zmin_default)
    call from_param("image_zmax", igrid%image_zmax, image_zmax_default)
    call from_param("image_dx"  , igrid%image_dx  , image_dx_default  )
    call from_param("image_dy"  , igrid%image_dy  , image_dy_default  )
    call from_param("image_dz"  , igrid%image_dz  , image_dz_default  )

!   safe guarding
!   -------------
    igrid%image_xmin = vgrid%ax%o + int(1.5+(igrid%image_xmin-vgrid%ax%o)/vgrid%ax%d-1)*vgrid%ax%d
    igrid%image_ymin = vgrid%ay%o + int(1.5+(igrid%image_ymin-vgrid%ay%o)/vgrid%ay%d-1)*vgrid%ay%d
    igrid%image_zmin = vgrid%az%o + int(1.5+(igrid%image_zmin-vgrid%az%o)/vgrid%az%d-1)*vgrid%az%d

    igrid%image_xmax = vgrid%ax%o + int(1.5+(igrid%image_xmax-vgrid%ax%o)/vgrid%ax%d-1)*vgrid%ax%d
    igrid%image_ymax = vgrid%ay%o + int(1.5+(igrid%image_ymax-vgrid%ay%o)/vgrid%ay%d-1)*vgrid%ay%d
    igrid%image_zmax = vgrid%az%o + int(1.5+(igrid%image_zmax-vgrid%az%o)/vgrid%az%d-1)*vgrid%az%d

    igrid%image_jx = nint(igrid%image_dx/vgrid%ax%d); igrid%image_dx = igrid%image_jx*vgrid%ax%d
    igrid%image_jy = nint(igrid%image_dy/vgrid%ay%d); igrid%image_dy = igrid%image_jy*vgrid%ay%d
    igrid%image_jz = nint(igrid%image_dz/vgrid%az%d); igrid%image_dz = igrid%image_jz*vgrid%az%d

    if(igrid%image_xmin < image_xmin_default) call seperr("image_xmin is out of the input velocity model")
    if(igrid%image_xmax > image_xmax_default) call seperr("image_xmax is out of the input velocity model")
    if(igrid%image_ymin < image_ymin_default) call seperr("image_ymin is out of the input velocity model")
    if(igrid%image_ymax > image_ymax_default) call seperr("image_ymax is out of the input velocity model")
    if(igrid%image_zmin < image_zmin_default) call seperr("image_zmin is out of the input velocity model")
    if(igrid%image_zmax > image_zmax_default) call seperr("image_zmax is out of the input velocity model")

!   define the image grid
!   ---------------------
    igrid%ax%o=igrid%image_xmin; igrid%ax%d=igrid%image_dx; igrid%ax%n=1.5+(igrid%image_xmax-igrid%image_xmin)/igrid%image_dx
    igrid%ay%o=igrid%image_ymin; igrid%ay%d=igrid%image_dy; igrid%ay%n=1.5+(igrid%image_ymax-igrid%image_ymin)/igrid%image_dy
    igrid%az%o=igrid%image_zmin; igrid%az%d=igrid%image_dz; igrid%az%n=1.5+(igrid%image_zmax-igrid%image_zmin)/igrid%image_dz

    call from_param("nhx", igrid%ahx%n, 1); call from_param("dhx", igrid%ahx%d, vgrid%ax%d); call from_param("ohx", igrid%ahx%o, 0.)
    call from_param("nhy", igrid%ahy%n, 1); call from_param("dhy", igrid%ahy%d, vgrid%ay%d); call from_param("ohy", igrid%ahy%o, 0.)
    call from_param("nhz", igrid%ahz%n, 1); call from_param("dhz", igrid%ahz%d, vgrid%az%d); call from_param("ohz", igrid%ahz%o, 0.)

    hgrid%ax_modl  = igrid%ax
    hgrid%ay_modl  = igrid%ay
    hgrid%az_modl  = igrid%az
    hgrid%ahx_modl = igrid%ahx
    hgrid%ahy_modl = igrid%ahy

!   output image without stacking over shot axis
!   -------------------------------------------- 
    if (igrid%stkimg) then
       igrid%asx%n=1; igrid%asx%o=0.; igrid%asx%d=1.
       igrid%asy%n=1; igrid%asy%o=0.; igrid%asy%d=1.
    else
       igrid%asx = dgrid%asx
       igrid%asy = dgrid%asy
    end if

    hgrid%asx = igrid%asx
    hgrid%asy = igrid%asy

!   write out image history
!   -----------------------
    call houaxis(hess_, hgrid%ahx_modl, 1); call auxputch("label1", "s", "hx", hess_)
    call houaxis(hess_, hgrid%ahy_modl, 2); call auxputch("label2", "s", "hy", hess_)
    call houaxis(hess_, hgrid%ax_modl , 3); call auxputch("label3", "s", "mx", hess_)
    call houaxis(hess_, hgrid%ay_modl , 4); call auxputch("label4", "s", "my", hess_)
    call houaxis(hess_, hgrid%az_modl , 5); call auxputch("label5", "s", "mz", hess_)
    if (.not.igrid%stkimg) then
       call houaxis(hess_, hgrid%asx, 6); call auxputch("label6", "s", "sx", hess_)
       call houaxis(hess_, hgrid%asy, 7); call auxputch("label7", "s", "sy", hess_)
    end if
    return
  end subroutine
 
  subroutine bwi_histos_wfld(dgrid, pgrid, filetag)
    type(dat_grid)   :: dgrid
    type(pro_grid)   :: pgrid
    character(len=*) :: filetag
    type(axis)       :: az, ash

    az%o = pgrid%az%o
    az%d = pgrid%az%d
    az%n = pgrid%nzstep
    ash%o = 0
    ash%d = 1.
    ash%n = dgrid%nshg
    call houaxis(filetag, pgrid%ax, 1)
    call houaxis(filetag, pgrid%ay, 2)
    call houaxis(filetag, dgrid%aw, 3)
    call houaxis(filetag, az      , 4)
    call houaxis(filetag, ash     , 5)
    call auxputch("esize", "i", 8, filetag)
    return
  end subroutine
end module
