program main
  use base_ioutil_mod
  use bwi_diphlx_mod
  use bwi_grdprc_mod
  implicit none

  type(axis)         :: axk, ayk, azk, adip
  type(vel_grid)     :: vgrid
  type(img_grid)     :: igrid
  character(len=5)   :: scheme
  character(len=256) :: gdov_, gdoe_, gdod_ 
  character(len=256) :: vmsk_, emsk_, dmsk_
  character(len=256) :: bimg_, dipx_, dipy_, stdv_

  logical            :: smtgrd, wntdip, gscale, gxparm
  character(len=6)   :: smttyp
  integer            :: rectx_grd_v, recty_grd_v, rectz_grd_v, nsmt_grd_v, stat
  integer            :: rectx_grd_e, recty_grd_e, rectz_grd_e, nsmt_grd_e
  integer            :: rectx_grd_d, recty_grd_d, rectz_grd_d, nsmt_grd_d
  integer            :: rectx_dip, recty_dip, rectz_dip, nsmt_dip
  integer            :: ndip, dipitr, na, idip, repeat, iflt
  real               :: xk_min, yk_min, zk_min
  real               :: xk_max, yk_max, zk_max 
  real               :: dip_min, dip_max, dipeps 

  integer    , dimension(:,:,:), allocatable :: selx, sely
  type(helix), dimension(:)    , allocatable :: dfcx, dfcy
  real       , dimension(:)    , allocatable :: dfcx0, dfcy0
  real       , dimension(:,:,:), allocatable :: dipx, dipy           

  call sep_init(SOURCE)
  call getch_add_string("head=/dev/null")
  call getch_add_string("noheader=y")
  call set_no_putch()
  call sep_begin_prog()

  gdov_ = "gdov"
  gdoe_ = "gdoe"
  gdod_ = "gdod"
  vmsk_ = "vmsk"
  emsk_ = "emsk"
  dmsk_ = "dmsk"
  bimg_ = "bimg"
  dipx_ = "dipx"
  dipy_ = "dipy"
  stdv_ = "stdv"

  call from_param("smtgrd", smtgrd      , .true.  )
  call from_param("smttyp", smttyp      , "triflt")
  call from_param("grdmsk", vgrid%grdmsk, .false. )
  call from_param("gscale", gscale      , .true.  )
  call from_param("gxparm", gxparm      , .false. )

  call auxinout(gdov_)
  call auxinout(gdoe_)
  call auxinout(gdod_)
  if (vgrid%grdmsk) then
    call auxin(vmsk_)
    call auxin(emsk_)
    call auxin(dmsk_)
  end if

  call hinaxis(gdov_, vgrid%ax, 1)
  call hinaxis(gdov_, vgrid%ay, 2)
  call hinaxis(gdov_, vgrid%az, 3)

! apply gradient mask
! ------------------- 
  if (vgrid%grdmsk) then
    write(0,*) "mask precond"
     stat = bwi_grdmsk_oper(gdov_, vmsk_, vgrid)
     stat = bwi_grdmsk_oper(gdoe_, emsk_, vgrid)
     stat = bwi_grdmsk_oper(gdod_, dmsk_, vgrid)
  end if

! apply gradient smoothing
! ------------------------
  if (smtgrd) then
     select case (smttyp)
       case ("dipflt")
        write(0,*) "dip filter"
            call from_param("rectx_dip", rectx_dip, 1       )
            call from_param("recty_dip", recty_dip, 1       )
            call from_param("rectz_dip", rectz_dip, 1       )
            call from_param("nsmt_dip" , nsmt_dip , 1       )
            call from_param("dip_min"  , dip_min  , 0.      )
            call from_param("dip_max"  , dip_max  , 0.      )
            call from_param("ndip"     , adip%n   , 1       )
            call from_param("dipitr"   , dipitr   , 100     )
            call from_param("dipeps"   , dipeps   , 0.01    )
            call from_param("na"       , na       , 5       )
            call from_param("scheme"   , scheme   , "UW2x3" )
            call from_param("wntdip"   , wntdip   , .false. )
            call from_param("repeat"   , repeat   , 1       )

            call from_param("rectx_grd_v", rectx_grd_v, 1)
            call from_param("recty_grd_v", recty_grd_v, 1)
            call from_param("rectz_grd_v", rectz_grd_v, 1)
            call from_param("nsmt_grd_v" , nsmt_grd_v , 1)
            call from_param("rectx_grd_e", rectx_grd_e, 1)
            call from_param("recty_grd_e", recty_grd_e, 1)
            call from_param("rectz_grd_e", rectz_grd_e, 1)
            call from_param("nsmt_grd_e" , nsmt_grd_e , 1)
            call from_param("rectx_grd_d", rectx_grd_d, 1)
            call from_param("recty_grd_d", recty_grd_d, 1)
            call from_param("rectz_grd_d", rectz_grd_d, 1)
            call from_param("nsmt_grd_d" , nsmt_grd_d , 1)
            if (adip%n==1) then
               adip%o = dip_min
               adip%d = 1.
            else
               adip%o = dip_min
               adip%d = (dip_max-dip_min)/(adip%n-1)
            end if
            adip%n = adip%n + 1

            call auxinout(bimg_)
            call hinaxis(bimg_, igrid%ahx, 1)
            call hinaxis(bimg_, igrid%ahy, 2)
            call hinaxis(bimg_, igrid%ax , 3)
            call hinaxis(bimg_, igrid%ay , 4)
            call hinaxis(bimg_, igrid%az , 5)

            vgrid%index_imgvel_xmin = 1.5 + (igrid%ax%o-vgrid%ax%o)/vgrid%ax%d
            vgrid%index_imgvel_ymin = 1.5 + (igrid%ay%o-vgrid%ay%o)/vgrid%ay%d
            vgrid%index_imgvel_zmin = 1.5 + (igrid%az%o-vgrid%az%o)/vgrid%az%d
            vgrid%index_imgvel_xmax = 1.5 + (igrid%ax%o+(igrid%ax%n-1)*igrid%ax%d-vgrid%ax%o)/vgrid%ax%d
            vgrid%index_imgvel_ymax = 1.5 + (igrid%ay%o+(igrid%ay%n-1)*igrid%ay%d-vgrid%ay%o)/vgrid%ay%d
            vgrid%index_imgvel_zmax = 1.5 + (igrid%az%o+(igrid%az%n-1)*igrid%az%d-vgrid%az%o)/vgrid%az%d

!           tabulate dip filters in x
!           -------------------------
            if (vgrid%ax%n>1) then
               allocate(dfcx(adip%n), dfcx0(adip%n))
               allocate(dipx(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
               allocate(selx(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
               do idip=1, adip%n
                  call allocatehelix(dfcx(idip), 3*na)
               end do
               stat = bwi_diphlx_init(adip, dipeps, vgrid%ax%d, vgrid%az%d, vgrid%ax%n, na, dipitr, scheme)
               stat = bwi_diphlx_oper(dfcx, dfcx0)
               stat = bwi_diphlx_clos()
            end if

!           tabulate dip filters in y
!           -------------------------
            if (vgrid%ay%n>1) then
               allocate(dfcy(adip%n), dfcy0(adip%n))
               allocate(dipy(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
               allocate(sely(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
               do idip=1, adip%n
                  call allocatehelix(dfcy(idip), 3*na)
               end do
               stat = bwi_diphlx_init(adip, dipeps, vgrid%ay%d, vgrid%az%d, vgrid%ay%n, na, dipitr, scheme)
               stat = bwi_diphlx_oper(dfcy, dfcy0)
               stat = bwi_diphlx_clos()
            end if

!           estimate dips
!           -------------
            stat = bwi_update_dips(bimg_, dipx, dipy, selx, sely, igrid, vgrid, adip, rectx_dip, recty_dip, rectz_dip, nsmt_dip)

!           output dips
!           -----------
            if (wntdip) then
               if (vgrid%ax%n>1) then
                  call auxinout(dipx_)
                  call houaxis(dipx_, vgrid%ax, 1)
                  call houaxis(dipx_, vgrid%ay, 2)
                  call houaxis(dipx_, vgrid%az, 3)
                  call sseek(dipx_, 0, 0)
                  call sep_write(dipx, dipx_)
               end if
               if (vgrid%ay%n>1) then
                  call auxinout(dipy_)
                  call houaxis(dipy_, vgrid%ax, 1)
                  call houaxis(dipy_, vgrid%ay, 2)
                  call houaxis(dipy_, vgrid%az, 3)
                  call sseek(dipy_, 0, 0)
                  call sep_write(dipy, dipy_)
               end if
            end if

!           apply smoothing 
!           -------------------
            stat = bwi_triflt_oper(gdov_, vgrid, rectx_grd_v, recty_grd_v, rectz_grd_v, nsmt_grd_v)
            stat = bwi_triflt_oper(gdoe_, vgrid, rectx_grd_e, recty_grd_e, rectz_grd_e, nsmt_grd_e)
            stat = bwi_triflt_oper(gdod_, vgrid, rectx_grd_d, recty_grd_d, rectz_grd_d, nsmt_grd_d)
!           apply dip filtering
!           -------------------
            do iflt = 1, repeat
              stat = bwi_dipflt_oper(gdoe_, dfcx, dfcy, dfcx0, dfcy0, selx, sely, vgrid)
              stat = bwi_dipflt_oper(gdod_, dfcx, dfcy, dfcx0, dfcy0, selx, sely, vgrid)
              stat = bwi_dipflt_oper(gdov_, dfcx, dfcy, dfcx0, dfcy0, selx, sely, vgrid)
            end do

       case ("triflt")
            call from_param("rectx_grd_v", rectx_grd_v, 1)
            call from_param("recty_grd_v", recty_grd_v, 1)
            call from_param("rectz_grd_v", rectz_grd_v, 1)
            call from_param("nsmt_grd_v" , nsmt_grd_v , 1)
            stat = bwi_triflt_oper(gdov_, vgrid, rectx_grd_v, recty_grd_v, rectz_grd_v, nsmt_grd_v)
            call from_param("rectx_grd_e", rectx_grd_e, 1)
            call from_param("recty_grd_e", recty_grd_e, 1)
            call from_param("rectz_grd_e", rectz_grd_e, 1)
            call from_param("nsmt_grd_e" , nsmt_grd_e , 1)
            stat = bwi_triflt_oper(gdoe_, vgrid, rectx_grd_e, recty_grd_e, rectz_grd_e, nsmt_grd_e)
            call from_param("rectx_grd_d", rectx_grd_d, 1)
            call from_param("recty_grd_d", recty_grd_d, 1)
            call from_param("rectz_grd_d", rectz_grd_d, 1)
            call from_param("nsmt_grd_d" , nsmt_grd_d , 1)
            stat = bwi_triflt_oper(gdod_, vgrid, rectx_grd_d, recty_grd_d, rectz_grd_d, nsmt_grd_d)

       case ("spline")
            call from_param("xk_min",xk_min,0.); call from_param("xk_max",xk_max,0.); call from_param("nxk",axk%n,1)
            call from_param("yk_min",yk_min,0.); call from_param("yk_max",yk_max,0.); call from_param("nyk",ayk%n,1)
            call from_param("zk_min",zk_min,0.); call from_param("zk_max",zk_max,0.); call from_param("nzk",azk%n,1)
            xk_min = vgrid%ax%o + (int(1.5+(xk_min-vgrid%ax%o)/vgrid%ax%d)-1)*vgrid%ax%d
            xk_max = vgrid%ax%o + (int(1.5+(xk_max-vgrid%ax%o)/vgrid%ax%d)-1)*vgrid%ax%d
            yk_min = vgrid%ay%o + (int(1.5+(yk_min-vgrid%ay%o)/vgrid%ay%d)-1)*vgrid%ay%d
            yk_max = vgrid%ay%o + (int(1.5+(yk_max-vgrid%ay%o)/vgrid%ay%d)-1)*vgrid%ay%d
            zk_min = vgrid%az%o + (int(1.5+(zk_min-vgrid%az%o)/vgrid%az%d)-1)*vgrid%az%d
            zk_max = vgrid%az%o + (int(1.5+(zk_max-vgrid%az%o)/vgrid%az%d)-1)*vgrid%az%d
            axk%o = xk_min; axk%d = 1.
            ayk%o = yk_min; ayk%d = 1.
            azk%o = zk_min; azk%d = 1.
            if (axk%n>1) then
               axk%o = xk_min; axk%d = (xk_max-xk_min)/(axk%n-1)
            end if
            if (ayk%n>1) then
               ayk%o = yk_min; ayk%d = (yk_max-yk_min)/(ayk%n-1)
            end if
            if (azk%n>1) then
               azk%o = zk_min; azk%d = (zk_max-zk_min)/(azk%n-1)
            end if
            stat = bwi_spline_oper(gdov_, vgrid, axk, ayk, azk)
            stat = bwi_spline_oper(gdoe_, vgrid, axk, ayk, azk)
            stat = bwi_spline_oper(gdod_, vgrid, axk, ayk, azk)

     end select
  end if

! scale and normalize the gradient
! --------------------------------
  if (gscale) then
    write(0,*) "scaling precond"
     stat = bwi_gscale_oper(gdov_, gdoe_, gdod_, vgrid)
  end if

! scale the gradient according to standard deviation matrix
! ---------------------------------------------------------
  if (gxparm) then
    write(0,*) "x-param precond"
    stat = bwi_gxparm_oper(gdov_, gdoe_, gdod_, stdv_, vgrid)
  end if


  call sep_end_prog()

  call exit()

end program
