module ns_filter 
  ! modified from killwave_ns
  ! Yunyue Li
  ! 2011/14/04
  ! Create the non-stationary filters for 
  ! Smoothing the gradient

  use killwave_params
  use npolydiv_mod
  use nhelicon_mod
  use wilson_mod
  use nbound_mod

  implicit none

  logical			:: lscale
  real, allocatable	:: scale(:)
  real			:: ds, os
  integer			:: ns

  contains 
  !######################################
  subroutine  ns_filter_init(dips)
    integer	                              :: imod, nang, idip, nlags, lfilter, iang, ilags
    real                                      :: dip, dang, oang
    real, allocatable                         :: tdip(:)

    allocate (ptch(nx*nz),lfilter(nang))
    allocate (tdip(nx*nz))

    call from_param("nang", nang, 20)


    !                if (lscale)then !scale according to angle
    !                        call from_param("os",os,1.); call from_param("ds",ds,0.); call from_param("ns",ns,nang)
    !                        allocate (scale(ns))
    !                        do i=1,ns
    !                                scale(i)=os+(i-1)*ds
    !                        enddo
    !                endif

    ptch=0.
    tdip = reshape(dips, (/nx*nz/))
    where(tdip> 1.570)
      tdip=1.5708
    end where
    where(tdip<-1.570)
      tdip=-1.5708
    end where

    ! define the angle   
    if (maxval(tdip) == minval(tdip)) then
      nang = 1
      dang = 1
      oang = minval(tdip)
    else 
      oang = minval(tdip)
      dang = (maxval(tdip)-minval(tdip))/(nang-1)
    end if

    ! index the model using the dip
    do imod = 1, nx*nz
      dip = tdip(i)
      idip = 1.5 + (dip-oang)/dang
      ptch(i) =  idip
    end do
    where (ptch>float(nang))
      ptch=nang
    end where
    where (ptch<1.)
      ptch=1
    end where

    ! allocate the ns_filter
    call from_param("nlags", nlags, 9)
    lfilter = nlags*3

    call nallocate(ns_filt,lfilter,ptch)

    do iang=1,nang
      do ilags=1,nlags
        ns_filt%hlx(iang)%lag(        ilags) = ilags			! early lags (first row)
        ns_filt%hlx(iang)%lag(nlags  +ilags) = nz + ilags - nlags 	! late lags  (second row)
        ns_filt%hlx(iang)%lag(nlags*2+ilags) = nz + ilags 		! late late lags (second row)
      end do
    end do
  end subroutine
  !######################################
  subroutine ns_filter_build(eps_in)
    real, dimension(2)      :: u,v
    real                    :: eps_in, dip
    integer                 :: iang

    call nallocate(lapfilt, lfilter, ptch)             ! alloc. laplacian0 filter for each patch
    lapfilt0 = 1. + eps_in                             ! zero lag coeff.

    do iang=1,nang
      lapfilt%hlx(iang)%lag = (/ 1, nz-1, nz, nz+1/)               ! lag(1)= 1; lag(2)=n1  # one side only
      dip=oang+(iang-1)*dang
      write(0,*)'working on angle',iang,dip,degrad(1,dip)
      u(1)= cos(dip); u(2)=-sin(dip); 

      m=(u(1)-u(2))/2.; 
      p=(u(1)+u(2))/2.;
      lapfilt%hlx(iang)%flt = 0.;
      lapfilt%hlx(iang)%flt = (/2*m*p,-p*p,-2*m*p,-m*m/) 

      call wilson_init( 10 * n1 )
      call wilson_factor( 100, lapfilt0, lapfilt%hlx(iang), n_filt0, ns_filt%hlx(iang))
      call wilson_close()
!      ns_filt%hlx(iang)%flt=ns_filt%hlx(iang)%flt!/n_filt0
    end do	


  end subroutine

  subroutine ns_filter_smooth(nmod,snmod)
    integer                   :: stat
      call npolydiv_init(nx*nz,ns_filt)
      stat = filt_npolydiv(.false.,.false.,nmod,snmod)
      call npolydiv_close()
  end subroutine
  !######################################
  function filt_npolydiv( adj, add, xx, yy) result(stat)
    integer                   :: stat,stat1,stat2 
    logical,intent(in)        :: adj,add 
    real,dimension(:)         :: xx,yy 
    real, dimension(size(xx)) :: tmp
    tmp=0.

    if ( adj) then
      stat= npolydiv_lop(.false.,.false., yy,tmp)     ! tmp = grad y
      stat= npolydiv_lop(.true. ,.false., xx,tmp)     ! x = x + grad' tmp
    else
      stat= npolydiv_lop(.false.,.false., xx,tmp)     ! tmp = grad x
      stat= npolydiv_lop(.true. ,.false., yy,tmp)     ! y = y + grad' tmp
    endif
    stat=stat*stat2
  end function 
  !######################################
  function filt_nhconest( adj, add, xx, yy) result(stat)
    integer                   :: stat,stat1,stat2 
    logical,intent(in)        :: adj,add 
    real,dimension(:)         :: xx,yy 
    real, dimension(size(xx)) :: tmp
    tmp=0.
    if ( adj) then
      stat= nhelicon_lop(.false.,.false., yy,tmp)     ! tmp = grad y
      stat= nhelicon_lop(.true. ,.false., xx,tmp)     ! x = x + grad' tmp
    else
      stat= nhelicon_lop(.false.,.false., xx,tmp)     ! tmp = grad x
      stat= nhelicon_lop(.true. ,.false., yy,tmp)     ! y = y + grad' tmp
    endif
    stat=stat*stat2
  end function 
end module
