module steer_mod
  use filt_fact_mod
  use autocorr_mod
  use kolmogroff_mod

implicit none
  
  integer, private				:: nang, nfilt
  real, allocatable				:: A(:,:), ATA(:,:), FILT(:,:)

contains

  subroutine steer_init(nang_in,nfilt_in)
    integer						:: nang_in, nfilt_in

	nang  = nang_in;
	nfilt = nfilt_in
	allocate(A(2,2), ATA(3,3), FILT(3*nfilt,2))
  end subroutine

  subroutine steer_buildf(dip)
    real						:: dip, m, p
	real, dimension(2)			:: u, v
	real, dimension(4)			:: tfilt
	real						:: tfilt0, eps
	type(filter)				:: hfilt
	real						:: hfilt0

    A=0.; ATA =0.; FILT=0.

    u(1)= cos(dip); v(1)=-sin(dip)
    u(2)=-sin(dip); v(2)= cos(dip) 

    m=(u(1)-u(2))/2.; 
    p=(u(1)+u(2))/2.

    A(1,1)=-m; A(1,2)=p
    A(2,1)=-p; A(2,2)=m

    ATA(1,1)=-m**2 ; ATA(1,2)=2*m*p; ATA(1,3)=-p**2
    ATA(2,1)=-2*m*p; ATA(2,2)=1    ; ATA(2,3)=-2*m*p
    ATA(3,1)=-p**2 ; ATA(3,2)=2*m*p; ATA(3,3)=-m**2

    ATA(2,2)=2*(m**2+p**2)
    eps=0.0012 
    tfilt0  =ATA(2,2)
    tfilt(1)=ATA(3,2)
    tfilt(2)=ATA(1,3)
    tfilt(3)=ATA(2,3)
    tfilt(4)=ATA(3,3)

    if(verb>=2) then
            write(0,'(     4F15.5) ')A(:,:) 
            write(0,'(        A60) ')'========================================================='
            write(0,'(3(15X,F15.5))')ATA(:,:)
            write(0,'(        A60) ')'========================================================='
    endif

    call filt_fact_init (n1,n2,n3,nfilt,eps,tfilt0,tfilt,hfilt)
    if(verb1 >=1)then
            write(0,*)  hfilt%lag; write(0,'(F10.5)') (hfilt%flt)
    endif
    FILT(:,1)=hfilt%flt
    FILT(:,2)=hfilt%lag 
  end subroutine

  subroutine steer_clos()
	deallocate(A, ATA, FILT)
  end subroutine

end module
