module ani_bornsc_mod
!! This module computes the born screen operator for Vv, epsilon, and del
!! formulat for epsilon and delta are the same
!! the differences are in the coefficents


  use  base_fftwv3_mod
  implicit none

  real, dimension(:), pointer, private :: kx2, ky2
  real                       , private :: dx, dy, dz
  real                       , private :: cc(4)
  integer                    , private :: nx, ny, nvrf, bscnop 
  character(len=6)           , private :: bsctyp, bsccoe

contains
  
  integer function ani_bornsc_init(kx2_in, ky2_in, dx_in, dy_in, dz_in, nx_in, ny_in, &
                                   nvrf_in, bscnop_in, bsctyp_in, bsccoe_in) result(stat)
    real, dimension(:), target :: kx2_in, ky2_in
    real                       :: dx_in, dy_in, dz_in
    integer                    :: nx_in, ny_in, nvrf_in, bscnop_in
    character(len=6)           :: bsctyp_in, bsccoe_in

    kx2    => kx2_in
    ky2    => ky2_in
    dx     =  dx_in
    dy     =  dy_in
    dz     =  dz_in
    nx     =  nx_in
    ny     =  ny_in
    nvrf   =  nvrf_in
    bscnop =  bscnop_in
    bsctyp =  bsctyp_in
    bsccoe =  bsccoe_in

    call bwi_phsscr_coef(cc, bsccoe, bscnop)

!	cc = 1.	
    stat = 0
  end function

  !!===============================================================================================

  !!===============================================================================================

  integer function ani_bornsc_oper(gwfd, bwfd, wtmp, bvel, beps, bdel, vref, alpha_z, beta_z, vmap, ws, opsign, plan_2dxF, plan_2dxI, par, dalpha_z, dbeta_z) result(stat)
    complex, dimension(:,:)				 :: gwfd
    complex, dimension(:,:)				 :: bwfd
    complex, dimension(:,:)				 :: wtmp
    real   , dimension(:,:)				 :: bvel
    real   , dimension(:,:)				 :: beps
    real   , dimension(:,:)				 :: bdel
    real   , dimension(:,:)				 :: alpha_z
    real   , dimension(:,:)				 :: beta_z 
    real   , dimension(:,:), optional	 :: dalpha_z                    ! d \alpha d (\epsilon or \delta)
    real   , dimension(:,:), optional	 :: dbeta_z                     ! d \beta  d (\epsilon or \delta)
    real   , dimension(:)  				 :: vref
    integer, dimension(:,:)				 :: vmap
    real                  				 :: ws
    integer                				 :: opsign
    integer*8              				 :: plan_2dxF, plan_2dxI
    character(len=3)       				 :: par

	if (bsctyp=="fourie") then
		select case(par)
			case ("vel")
				call phsscr_tayexp_vel(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, ws, opsign, plan_2dxF, plan_2dxI)
			case ("eps")
				call phsscr_tayexp_ani(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, dalpha_z, dbeta_z, ws, opsign, plan_2dxF, plan_2dxI)
			case ("del")
				call phsscr_tayexp_ani(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, dalpha_z, dbeta_z, ws, opsign, plan_2dxF, plan_2dxI)
			case default
				write(0,*)"par has to be either <vel>, <eps> or <del>"
				STOP
		end select
	else if(bsctyp=="spaced") then
		select case(par)
			case ("vel")
				call phsscr_space_vel(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, ws, opsign, plan_2dxF, plan_2dxI)
			case ("eps")
				call phsscr_space_ani(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, dalpha_z, dbeta_z, ws, opsign, plan_2dxF, plan_2dxI)
			case ("del")
				call phsscr_space_ani(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, dalpha_z, dbeta_z, ws, opsign, plan_2dxF, plan_2dxI)
		    case default
				write(0,*)"par has to be either <vel>, <eps> or <del>"
				STOP
		end select
	end if

    stat = 0
  end function

  integer function ani_bornsc_clos() result(stat)
    stat = 1
    kx2 => null()
    ky2 => null()
    dx = 0; dy = 0; dz = 0
    nx = 0; ny = 0
    bscnop = 0; bsctyp = " "; bsccoe=" "
    stat = 0
  end function


  !!===============================================================================================

  !!===============================================================================================

! input background wfld, output scattered wfld by velocity perturbation 
  subroutine phsscr_tayexp_vel(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, ws, opsign, plan_2dxF, plan_2dxI)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real   , dimension(:,:) :: beps
    real   , dimension(:,:) :: bdel
    real   , dimension(:,:) :: alpha_z
    real   , dimension(:,:) :: beta_z 
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    integer                 :: iop, ikx, iky, ix, iy
    complex                 :: isdz
    real                    :: kk2, kowov
    double precision        :: vel, wov, wov2, wov4

!   optimizing in velocity
!   i * sign * dKz/dv * dz
!   dKz/dv = -w/v/v / sqrt(1-v*v/w/w*|k|^2)
!   ---------------------------------------

!   set up the sign
!   ---------------
    isdz = cmplx(0., 1.)*dz*opsign

!   compute the expansion
!   1/sqrt(1-s) = 1 + c1 * s^2 + c2* s^4 + c3 * s^6 + c4 * s^8 + ...
!   where s = v/w*|k|
!   -----------------------------------------------------------------
    gwfd = bwfd 
    do iop=1, bscnop
       wtmp = bwfd 
       call sfftw_dft_2dx_oper(plan_2dxF, wtmp, nx, ny)
       do iky=1, ny
       do ikx=1, nx
          kk2 = kx2(ikx)+ky2(iky)
          wtmp(ikx,iky) = (kk2**iop)*wtmp(ikx,iky)
       end do
       end do
       call sfftw_dft_2dx_oper(plan_2dxI, wtmp, nx, ny)
       do iy=1, ny
       do ix=1, nx
          vel  = bvel(ix,iy)
          wov  = ws/vel
          wov2 = wov*wov
	  if(iop==1) then
             gwfd(ix,iy) = bwfd(ix,iy) + wtmp(ix,iy)*dble((1./wov2)**iop)*alpha_z(ix,iy)
	  else if (iop==2) then
             gwfd(ix,iy) = gwfd(ix,iy) + 3*wtmp(ix,iy)*dble((1./wov2)**iop)*alpha_z(ix,iy)*beta_z(ix,iy)
          end if 
       end do
       end do
    end do
!   multiple the weighting function
!   -------------------------------
    do iy=1, ny
    do ix=1, nx
       vel = bvel(ix,iy)
       wov = ws/vel
       gwfd(ix,iy) = gwfd(ix,iy)*isdz*(-wov)/vel
    end do
    end do

    return
  end subroutine


  !!===============================================================================================

  !!===============================================================================================


! input background wfld, output scattered wfld by velocity perturbation 
  subroutine phsscr_space_vel(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, ws, opsign, plan_2dxF, plan_2dxI)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real   , dimension(:,:) :: beps
    real   , dimension(:,:) :: bdel
    real   , dimension(:,:) :: alpha_z
    real   , dimension(:,:) :: beta_z 
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    integer                 :: iop, ikx, iky, ix, iy
    complex                 :: isdz
    real                    :: kk2, aa, bb
    double precision        :: vel, wov, wov2, wov4

!   optimizing in velocity
!   i * sign * dKz/dv * dz
!   dKz/dv = -w/v/v / sqrt(1-v*v/w/w*|k|^2)
!   ---------------------------------------

!   set up the sign
!   ---------------
    isdz = cmplx(0., 1.)*dz*opsign

!   -----------------------------------------------------------------
    gwfd = 0.; wtmp = 0. 

    call laplac2(bwfd,gwfd)
    call laplac2(gwfd,wtmp)

!   multiple the weighting function
!   -------------------------------
    do iy=1, ny
    do ix=1, nx
       vel  = bvel(ix,iy); aa   = alpha_z(ix,iy); bb   = beta_z(ix,iy)
       wov  = ws/vel;      wov2 = wov  * wov;     wov4 = wov2 * wov2
       gwfd(ix,iy) = ( gwfd(ix,iy)*aa/wov2 & 
                     + 3*wtmp(ix,iy)*aa*bb/wov4 ) *isdz*(-wov)/vel
    end do
    end do

    return
  end subroutine


  !!===============================================================================================

  !!===============================================================================================

! input background wfld, output scattered wfld by eps perturbation 
  subroutine phsscr_tayexp_ani(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, dalpha_z, dbeta_z, ws, opsign, plan_2dxF, plan_2dxI)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real   , dimension(:,:) :: beps
    real   , dimension(:,:) :: bdel
    real   , dimension(:,:) :: alpha_z
    real   , dimension(:,:) :: beta_z 
    real   , dimension(:,:) :: dalpha_z
    real   , dimension(:,:) :: dbeta_z 
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    integer                 :: iop, ikx, iky, ix, iy
    complex                 :: isdz
    real                    :: kk2, kowov
    double precision        :: vel, wov, wov2, wov4

!   optimizing in velocity
!   i * sign * dKz/dv * dz
!   dKz/dv = -w/v/v / sqrt(1-v*v/w/w*|k|^2)
!   ---------------------------------------

!   set up the sign
!   --------------
    isdz = cmplx(0., 1.)*dz*opsign

!   compute the expansion
!   1/sqrt(1-s) = 1 + c1 * s^2 + c2* s^4 + c3 * s^6 + c4 * s^8 + ...
!   where s = v/w*|k|
!   -----------------------------------------------------------------
    gwfd = 0. 
    do iop=1, bscnop
       wtmp = bwfd 
       call sfftw_dft_2dx_oper(plan_2dxF, wtmp, nx, ny)
       do iky=1, ny
       do ikx=1, nx
          kk2   = kx2(ikx)+ky2(iky)
		  wtmp(ikx,iky) = cc(iop) * (kk2**iop)*wtmp(ikx,iky)
       end do
       end do
       call sfftw_dft_2dx_oper(plan_2dxI, wtmp, nx, ny)
       do iy=1, ny
       do ix=1, nx
          vel  = bvel(ix,iy)
          wov  = ws/vel
          wov2 = wov*wov
		  if(iop==1) then
!             gwfd(ix,iy) = gwfd(ix,iy) + wtmp(ix,iy)*dble((1./wov2)**iop)*dalpha_z(ix,iy)
             gwfd(ix,iy) = wtmp(ix,iy)*dble((1./wov2)**iop)*dalpha_z(ix,iy)
		  else if (iop==2) then
             gwfd(ix,iy) = gwfd(ix,iy) + wtmp(ix,iy)*dble((1./wov2)**iop)* & 
				   ( dalpha_z(ix,iy)*beta_z(ix,iy) + alpha_z(ix,iy)*dbeta_z(ix,iy) )
		  end if 
       end do
       end do
    end do
!   multiple the weighting function
!   -------------------------------
    do iy=1, ny
    do ix=1, nx
       vel = bvel(ix,iy)
       wov = ws/vel
       gwfd(ix,iy) = gwfd(ix,iy)*isdz*(-wov)
    end do
    end do


    return
  end subroutine

  !!===============================================================================================

  !!===============================================================================================

! input background wfld, output scattered wfld by velocity perturbation 
  subroutine phsscr_space_ani(gwfd, bwfd, wtmp, bvel, beps, bdel, alpha_z, beta_z, dalpha_z, dbeta_z, ws, opsign, plan_2dxF, plan_2dxI)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real   , dimension(:,:) :: beps
    real   , dimension(:,:) :: bdel
    real   , dimension(:,:) :: alpha_z
    real   , dimension(:,:) :: beta_z 
    real   , dimension(:,:) :: dalpha_z
    real   , dimension(:,:) :: dbeta_z 
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    integer                 :: iop, ikx, iky, ix, iy
    complex                 :: isdz
    real                    :: kk2, aa, bb, da, db
    double precision        :: vel, wov, wov2, wov4

!   optimizing in velocity
!   i * sign * dKz/dv * dz
!   dKz/dv = -w/v/v / sqrt(1-v*v/w/w*|k|^2)
!   ---------------------------------------

!   set up the sign
!   ---------------
    isdz = cmplx(0., 1.)*dz*opsign

!   -----------------------------------------------------------------
    gwfd = 0.; wtmp = 0. 

	call laplac2(bwfd,gwfd)
	call laplac2(gwfd,wtmp)

!   multiple the weighting function
!   -------------------------------
    do iy=1, ny
    do ix=1, nx
       vel  = bvel(ix,iy); aa   = alpha_z(ix,iy); bb   = beta_z(ix,iy)
       wov  = ws/vel;      wov2 = wov  * wov;     wov4 = wov2 * wov2
       da   = dalpha_z(ix,iy);  db   = dbeta_z(ix,iy)
       gwfd(ix,iy) = (bwfd(ix,iy) + gwfd(ix,iy)*da/wov2 + wtmp(ix,iy)*(aa*db+bb*da)/wov4)*isdz*(-wov)
    end do
    end do

    return
  end subroutine

  !!===============================================================================================

  !!===============================================================================================

  subroutine phsscr_fourie(gwfd, bwfd, wtmp, bvel, vref, vmap, ws, opsign, plan_2dxF, plan_2dxI)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real   , dimension(:)   :: vref
    integer, dimension(:,:) :: vmap
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    complex                 :: isdz
    real                    :: kk2, kz2, kz2max, vdif1, vdif2
    integer                 :: ix, iy, ikx, iky, ivrf, jvrf

    wtmp = 0.
    vmap = 0
!   compute the velocity map
!   ------------------------
    do iy=1, ny
    do ix=1, nx
       jvrf = 1
       do ivrf=1, nvrf-1
          vdif1 = abs(bvel(ix,iy)-vref(ivrf  ))
          vdif2 = abs(bvel(ix,iy)-vref(ivrf+1))
          if (vdif1<vdif2) then
             jvrf = ivrf
          else
             jvrf = ivrf+1
          end if
       end do
       vmap(ix,iy) = jvrf
    end do
    end do

!   set the maximum angle
!   ---------------------
    kz2max = cos(80*3.14159265359/180.)**2

    isdz = cmplx(0., 1.)*dz*opsign

    do ivrf=1, nvrf
       wtmp = bwfd
       call sfftw_dft_2dx_oper(plan_2dxF, wtmp, nx, ny) 
       do iky=1, ny
       do ikx=1, nx
          kk2 = kx2(ikx)+ky2(iky)
          kz2 = 1 - kk2/ws/ws*vref(ivrf)*vref(ivrf)
          if (kz2>=kz2max) then
             wtmp(ikx,iky) = wtmp(ikx,iky)/sqrt(kz2) 
          else 
             wtmp(ikx,iky) = 0.
          end if
       end do
       end do
       call sfftw_dft_2dx_oper(plan_2dxI, wtmp, nx, ny)
       where (vmap == ivrf)
             gwfd = wtmp * (-ws/vref(ivrf)/vref(ivrf))
       end where 
    end do 
    gwfd = gwfd * isdz !* (-ws/bvel/bvel)
    return
  end subroutine

  subroutine bwi_phsscr_coef(cc, bsccoe, bscnop)
    real, dimension(4) :: cc
    character(len=6)   :: bsccoe
    integer            :: bscnop
    select case (bsccoe)
      case ("taylor")
           cc(1) = 1./2.
           cc(2) = 3./8.
           cc(3) = 5./16.
           cc(4) = 35./128.
           if (bscnop>4) then
              write(0,*) "ERROR: bscnop <= 4, Taylor coef"
              STOP
           end if
      case ("optimz")
           select case (bscnop)
             case (2) !! up to 45 degree
                  cc(1) = 0.437271673; cc(2) = 0.743598856
             case (3) !! up to 65 degree
                  cc(1) = 0.646864667; cc(2) = -0.650508335; cc(3) = 2.020601118
             case default
                  write(0,*) "ERROR: 2 =< bscnop <= 3, Optimized coef" 
                  STOP
           end select 
      case default 
           write(0,*)"bsccoe has to be either <taylor> or <optimz>"
           STOP
    end select 
    return
  end subroutine

  !!===============================================================================================

  !!===============================================================================================
  !! 2nd Order Laplacian

  subroutine laplac2(uo,ud)

    integer   				 :: ix,iy
    complex, dimension(:,:)  :: uo,ud

	ud = 0.;

	if (ny>3 .and. nx>3 ) then
		do iy = 2, ny-1
			do ix = 2, nx-1	
				ud(ix,iy) =  (-2.*uo(ix,iy) + uo(ix-1,iy) + uo(ix+1,iy)) / (dx*dx) &
						  +	 (-2.*uo(ix,iy) + uo(ix,iy+1) + uo(ix,iy-1)) / (dy*dy)
			end do
		end do
	else
		do iy = 1, ny 	
			do ix = 2, nx-1	
				ud(ix,iy) =  (-2.*uo(ix,iy) + uo(ix-1,iy) + uo(ix+1,iy) )/(dx*dx)
			end do
		end do	
	end if	
			
  end subroutine 

end module
