module base_nshlxc_mod
! Non-stationary helix convolution
! Nonstationary convolution, inverse to deconvolution.
! Requires the filter be causal with an implicit "1." at the onset.

  use  base_hlxtyp_mod
  implicit none

  type(helix), private :: aa
contains

  subroutine base_nshlxcon_init (aa_in)
    type( nfilter)    :: aa_in 
    aa = aa_in
  end subroutine

  function base_nshlxcon_oper ( adj, add, xx, yy) result(stat)
    integer                         :: stat 
    logical,intent(in)              :: adj,add 
    real,dimension(:)               :: xx,yy 
    integer                         :: iy, ix, ia, ip
    integer, dimension( :), pointer :: lag
    real,    dimension( :), pointer :: flt

    if ( adj) then ! zero lag
      xx = xx + yy
    else
      yy = yy + xx
    end if 

    do iy = 1, size( yy)
      if ( associated( aa%mis)) then
        if ( aa%mis( iy)) cycle
      endif
      ip = aa%pch( iy)
      lag => aa%hlx( ip)%lag
      flt => aa%hlx( ip)%flt
      do ia = 1, size( lag)
        ix = iy - lag( ia)
        if ( ix < 1) cycle
        if ( adj) then
          xx(ix) = xx(ix) + yy(iy) * flt( ia)
        else
          yy(iy) = yy(iy) + xx(ix) * flt( ia)
        end if
      end do
    end do

    stat=0
  end function 


  integer function base_nshlxcon_clos() result(stat)
    stat = 0
  end function

end module
