module base_wilson_mod
  use  base_hlxcon_mod
  use  base_poldiv_mod
  implicit none

  integer                           , private :: n
  real   , dimension(:), allocatable, private :: auto, bb, cc, b, c

contains

  integer function base_wilson_init(nmax) result (stat) !!.. nmax = maxval(ss%lag) + 1
    integer :: nmax
    n = nmax
    allocate(auto(2*n-1), bb(2*n-1), cc(2*n-1), b(n), c(n))
    stat = 0
  end function

  integer function base_wilson_oper(niter, s0, ss, a0, aa, verb) result(stat)
    integer     :: niter
    real        :: s0
    type(helix) :: ss
    real        :: a0
    type(helix) :: aa
    logical     :: verb
    optional    :: verb
    real        :: eps
    integer     :: i 
    auto = 0.; auto(n) = s0
    b(1) = 1.; c = 0.; bb = 0.; cc = 0.
    auto(n+ss%lag) = ss%flt
    auto(n-ss%lag) = ss%flt

    stat = base_hlxcon_init(aa, 1.)
    stat = base_poldiv_init(aa, 1., 2*n-1)

    do i=1, niter
       stat = base_poldiv_oper(.false., .false., auto, bb)
       stat = base_poldiv_oper(.true. , .false., cc  , bb)
       b(2:n) = 0.5*(cc(n+1:2*n-1)+cc(n-1:1:-1))/cc(n)
       eps = maxval(abs(b(2:n))) 
       if (present(verb)) then
          if (verb) write(0,*) i, eps
       end if 
       stat = base_hlxcon_oper(.false., .false., b, c)
       aa%flt = c(1+aa%lag) 
       if (eps<epsilon(a0)) exit
    end do
    a0 = sqrt(cc(n))
    stat = base_poldiv_clos()
    stat = base_hlxcon_clos()
    stat = 0
  end function

  integer function base_wilson_clos() result(stat)
    deallocate(auto, bb, cc, b, c)
    stat = 0
  end function

end module
