!!$
!!$=head1 NAME
!!$
!!$ bwi-wem3d.x - 2/3D one-way wave-equation shot-profile Born modeling & migration
!!$   Written by Yaxun Tang
!!$   tang@sep.stanford.edu 
!!$   Version 1.0
!!$   June, 2011
!!$
!!$=head2 SYNOPSIS FOR BORN MODELING
!!$
!!$ Born modeling: 
!!$   bwi-wem3d.x mode=imgfwd parameters 
!!$
!!$=head3 PARAMETERS FOR BORN MODELING
!!$
!!$ csou=CSOU.H(input), source function
!!$      sep77 hypercube where n1=gx n2=gy n3=frequency n4=sx n5=sy
!!$      for point sources, set n1=1 o1=0 d1=1 d2=1 o2=0 d2=1 n4=1 o4=0 d4=1 n5=1 o5=0 d5=1
!!$      for 2-D, set n2=1 o2=0 d2=1 n5=1 o5=0 d5=1
!!$
!!$ crec=CREC.H(output), modeled Born data
!!$      sep77 hypercube where n1=gx n2=gy n3=frequency n4=sx n5=sy
!!$
!!$ bvel=BVEL.H(input), velocity model
!!$      sep77 hypercube where n1=x n2=y n3=z
!!$      for 2-D, set n2=1 o2=0 d2=1
!!$      the velocity model should be big enough to accommodate all sources and receivers
!!$
!!$ bimg=BIMG.H(input), reflectivity model(image)
!!$      sep77 hypercube where n1=subsurface-offset x n2=subsurface-offset y n3=x n4=y n5=z
!!$      for 2-D, set n2=1 o2=0 d2=1 n4=1 o4=1 d4=1
!!$
!!$ bwfs=BWFS.H(output, optional), source wavefield, useful when wntwfs=y
!!$      sep77 hypercube where n1=x n2=y n3=frequency n4=z
!!$
!!$ bwfr=BWFR.H(output, optional), receiver wavefield, useful when wntwfr=y
!!$      sep77 hypercube where n1=x n2=y n3=frequency n4=z
!!$
!!$ add=y/[n], whether to add the output to the current file
!!$            if add=y, CREC.H should already exist
!!$
!!$ radian=[y]/n, whether the input frequency is in radian or not
!!$ 
!!$ offset=y/[n], y for marine geometry where the receivers are in offset coordinates
!!$
!!$ comsou=y/[n], y for composite or areal source function
!!$               n for point source function
!!$
!!$ smtsou=y/[n], whether or not to spatially smooth the point source function, useful when comsou=n
!!$
!!$ stkimg=[y]/n, if the image input for modeling hasn't been stacked over source axis, set stkimg=y, otherwise n
!!$
!!$ srcbnd=y/[n], boundary condition for source function, which changes dipole sources into monopole
!!$
!!$ vsfc=[1500], surface velocity for source boundary condition, useful when srcbnd=y
!!$
!!$ nxpad_beg=[0], zero padding (# of samples) at the beginning of x
!!$
!!$ nxpad_end=[0], zero padding (# of samples) at the end of x 
!!$
!!$ nypad_beg=[0], zero padding (# of samples) at the beginning of y
!!$
!!$ nypad_end=[0], zero padding (# of samples) at the end of y
!!$
!!$ ntpx=[1], taper length (# of samples) in x
!!$
!!$ ntpy=[1], taper length (# of samples) in y
!!$
!!$ oper=phsft/ssfou/impfd/foufd/phspi/[ssfpi], extrapolator type
!!$                                             phsft: phase shift
!!$                                             ssfou: split-step Fourier
!!$                                             impfd: implicit finite difference
!!$                                             foufd: Fourier finite difference
!!$                                             phspi: phase-shift plus interpolation
!!$                                             ssfpi: split-step Fourier plus interpolation
!!$
!!$ nvrf=[1], number of reference velocities, useful when oper=phspi or ssfpi
!!$
!!$ pdip=[45], propagation dip for finite difference, useful when oper=impfd or foufd
!!$            for oper=impfd, pdip can be chosen from 15/45/65/79/80/87/89/90,
!!$                            wide angle propagation is achieved by Zaitian Ma's 
!!$                            higher-order splitting method
!!$            for oper=foufd, pdip can be chosen from 45/65/75 
!!$
!!$ trick=[0.14], 1/6 trick for finite difference, useful when oper=impfd or foufd
!!$
!!$ bndpr=[0], boundary condition (real part) for tridiagonal system, useful when oper=impfd or foufd
!!$
!!$ bndpi=[0], boundary condition (imaginary part) for tridiagonal system, useful when oper=impfd or foufd
!!$
!!$ bullet=y/[n], use bullet-proof finite-difference shceme or not, useful when oper=impfd or foufd
!!$
!!$ errflt=[y]/n, apply Li's compensation filter or not, useful when oper=impfd or foufd
!!$
!!$ wntwfs=y/[n], whether or not to output the source wavefield
!!$
!!$ wntwfr=y/[n], whether or not to output the receiver wavefield
!!$
!!$ nws=[1], number of frequencies to be held in memory
!!$          set nws >= OMP_NUM_THREAD for better performance
!!$
!!$ report=[y]/n, whether or not to report progress
!!$
!!$ memchk=[y]/n, memory check or not, set memchk=n to actually run the program
!!$
!!$ imgfwd_type=[bckward]/adjoint, extrapolation type for modeling/migration
!!$
!!$
!!$ 
!!$=head2 SYNOPSIS FOR BORN MIGRATION
!!$
!!$ Born migration:
!!$   bwi-wem3d.x mode=imgadj parameters
!!$
!!$=head3 PARAMETERS
!!$ 
!!$ csou=CSOU.H(input), source function
!!$      sep77 hypercube where n1=gx n2=gy n3=frequency n4=sx n5=sy
!!$      for point sources, set n1=1 o1=0 d1=1 d2=1 o2=0 d2=1 n4=1 o4=0 d4=1 n5=1 o5=0 d5=1
!!$      for 2-D, set n2=1 o2=0 d2=1 n5=1 o5=0 d5=1
!!$
!!$ crec=CREC.H(input), data to be migrated
!!$      sep77 hypercube where n1=gx n2=gy n3=frequency n4=sx n5=sy
!!$      for 2-D, set n2=1 o2=0 d2=1
!!$
!!$ bvel=BVEL.H(input), velocity model
!!$      sep77 hypercube where n1=x n2=y n3=z
!!$      for 2-D, set n2=1 o2=0 d2=1
!!$      the velocity model should be big enough to accommodate all sources and receivers
!!$
!!$ bimg=BIMG.H(output), reflectivity model(image)
!!$      sep77 hypercube where n1=subsurface-offset x n2=subsurface-offset y n3=x n4=y n5=z
!!$
!!$ bwfs=BWFS.H(output, optional), source wavefield, useful when wntwfs=y
!!$      sep77 hypercube where n1=x n2=y n3=frequency n4=z
!!$
!!$ bwfr=BWFR.H(output, optional), receiver wavefield, useful when wntwfr=y
!!$      sep77 hypercube where n1=x n2=y n3=frequency n4=z
!!$
!!$ add=y/[n], whether or not to add the ouput to the current file
!!$            if add=y, BIMG.H should already exist
!!$
!!$ radian=[y]/n, whether the input frequency is in radian or not
!!$
!!$ offset=y/[n], y for marine geometry where the receivers are in offset coordinates
!!$
!!$ comsou=y/[n], y for composite or areal source function
!!$               n for point source function
!!$
!!$ smtsou=y/[n], whether or not to spatially smooth the point source function, useful when comsou=n
!!$
!!$ stkimg=[y]/n, whether or not to stack the image over source axis
!!$               if stkimg=y, two more axes will be added to the output image, n6=sx n7=sy
!!$
!!$ srcbnd=y/[n], boundary condition for source function, which changes dipole sources into monopole
!!$
!!$ vsfc=[1500], surface velocity for source boundary condition, useful when srcbnd=y
!!$
!!$ nxpad_beg=[0], zero padding (# of samples) at the beginning of x
!!$
!!$ nxpad_end=[0], zero padding (# of samples) at the end of x
!!$
!!$ nypad_beg=[0], zero padding (# of samples) at the beginning of y
!!$
!!$ nypad_end=[0], zero padding (# of samples) at the end of y
!!$
!!$ ntpx=[1], taper length (# of samples) in x
!!$
!!$ ntpy=[1], taper length (# of samples) in y
!!$
!!$ oper=phsft/ssfou/impfd/foufd/phspi/[ssfpi], extrapolator type
!!$                                             phsft: phase shift
!!$                                             ssfou: split-step Fourier
!!$                                             impfd: implicit finite difference
!!$                                             foufd: Fourier finite difference
!!$                                             phspi: phase-shift plus interpolation
!!$                                             ssfpi: split-step Fourier plus interpolation
!!$
!!$ nvrf=[1], number of reference velocities, useful when oper=phspi or ssfpi
!!$
!!$ pdip=[45], propagation dip for finite difference, useful when oper=impfd or foufd
!!$            for oper=impfd, pdip can be chosen from 15/45/65/79/80/87/89/90,
!!$                            wide angle propagation is achieved by Zaitian Ma's 
!!$                            higher-order splitting method
!!$            for oper=foufd, pdip can be chosen from 45/65/75 
!!$
!!$ trick=[0.14], 1/6 trick for finite difference, useful when oper=impfd or foufd
!!$
!!$ bndpr=[0], boundary condition (real part) for tridiagonal system, useful when oper=impfd or foufd
!!$
!!$ bndpi=[0], boundary condition (imaginary part) for tridiagonal system, useful when oper=impfd or foufd
!!$
!!$ bullet=y/[n], use bullet-proof finite-difference shceme or not, useful when oper=impfd or foufd
!!$
!!$ errflt=[y]/n, apply Li's compensation filter or not, useful when oper=impfd or foufd
!!$
!!$ wntwfs=y/[n], whether or not to output the source wavefield
!!$
!!$ wntwfr=y/[n], whether or not to output the receiver wavefield
!!$
!!$ nws=[1], number of frequencies to be held in memory
!!$          set nws >= OMP_NUM_THREAD for better performance
!!$
!!$ report=[y]/n, whether or not to report progress
!!$
!!$ memchk=[y]/n, memory check or not, set memchk=n to actually run the program
!!$
!!$ imgfwd_type=[bckward]/adjoint, extrapolation type for modeling/migration
!!$
!!$ nhx=[1], number of subsurface offset in x
!!$
!!$ ohx=[0], origin of subsurface offset in x
!!$
!!$ dhx=[dgx], sampling rate of subsurface offset in x
!!$
!!$ nhy=[1], number of subsurface offset in y
!!$
!!$ ohy=[0], origin of subsurface offset in y
!!$
!!$ dhy=[dgy], sampling rate of subsurface offset in y
!!$
!!$ image_xmin image_xmax, minimum and maximum x values for image output
!!$ 
!!$ image_ymin image_ymax, minimum and maximum y values for image output
!!$
!!$ image_zmin image_zmax, minimum and maximum z values for image output
!!$
!!$=head1 COMMENTS
!!$
!!$ Report bugs to Yaxun Tang, tang@sep.stanford.edu 
!!$
!!$=cut

program main
  use bwi_kernel_old
  implicit none

  type(dat_grid) :: dgrid
  type(pro_grid) :: pgrid
  type(vel_grid) :: vgrid
  type(img_grid) :: igrid
  type(hes_grid) :: hgrid
  type(cef_grid) :: cgrid
  type(bwi_ftag) :: ftag_ 
  logical        :: adj, add 
  integer        :: stat

  call sep_init(SOURCE)
  call getch_add_string("head=/dev/null")
  call getch_add_string("noheader=y")
  call set_no_putch()
  call sep_begin_prog()

  call from_param("add", add, .false.)

  stat = bwi_kernel_init(dgrid, pgrid, vgrid, igrid, hgrid, cgrid, ftag_)

  if (pgrid%mode=="imgfwd" .or. pgrid%mode=="imgadj") then
     if (pgrid%mode=="imgfwd") adj = .false.
     if (pgrid%mode=="imgadj") adj = .true.

     stat = bwi_kerimg_oper(adj, add, ftag_%csou_, ftag_%crec_, ftag_%bvel_, ftag_%beps_, ftag_%bdel_, ftag_%bimg_)

  else
     write(0,*)"ERROR: WRONG MODE"
     STOP
  end if

  write(0,*)"***********************"
  write(0,*)"      Finished!!!      "
  write(0,*)"***********************"

  call sep_end_prog()

  call exit()

end program
