module bwi_bornsc_mod2
  use  base_fftwv3_mod
  implicit none

  real, dimension(:), pointer, private :: kx2, ky2
  real                       , private :: dx, dy, dz
  real                       , private :: cc(4)
  integer                    , private :: nx, ny, nopbsc, typbsc

contains
  
  integer function bwi_bornsc_init(kx2_in, ky2_in, dx_in, dy_in, dz_in, nx_in, ny_in, &
                                   nopbsc_in, typbsc_in) result(stat)
    real, dimension(:), target :: kx2_in, ky2_in
    real                       :: dx_in, dy_in, dz_in
    integer                    :: nx_in, ny_in, nopbsc_in, typbsc_in

    kx2 => kx2_in
    ky2 => ky2_in
    dx  =  dx_in
    dy  =  dy_in
    dz  =  dz_in
    nx  =  nx_in
    ny  =  ny_in

    nopbsc = nopbsc_in
    typbsc = typbsc_in

    call bwi_phsscr_coef(cc)

!    select case(typbsc) 
!      case (1)
!           write(0,*)"Born Scattering Operator: FOURIER DOMAIN"
!      case (2)
!           write(0,*)"Born Scattering Operator: SPACE DOMAIN"
!      case (3) 
!           write(0,*)"Born Scattering Operator: FINITE DIFFERENCE"
!    end select
!
!    write(0,*)"nopbsc = ", nopbsc 

    stat = 0
  end function

  integer function bwi_bornsc_oper(gwfd, bwfd, wtmp, bvel, ws, opsign, plan_2dxF, plan_2dxI) result(stat)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    select case(typbsc)
       case (1)
            call phsscr_fouri(gwfd, bwfd, wtmp, bvel, ws, opsign, plan_2dxF, plan_2dxI)
       case (2)
            call phsscr_space(gwfd, bwfd, wtmp, bvel, ws, opsign, plan_2dxF, plan_2dxI)
       case (3)
!            call phsscr_fidif(gwfd, bwfd, bvel, ws, opsign)

call phsscr_pspi(gwfd, bwfd, wtmp, bvel, ws, opsign, plan_2dxF, plan_2dxI)
    end select

    stat = 0
  end function

  integer function bwi_bornsc_clos() result(stat)
    stat = 1
    kx2 => null()
    ky2 => null()
    dx = 0; dy = 0; dz = 0
    nx = 0; ny = 0
    nopbsc = 0; typbsc = 0
    stat = 0
  end function


! input background wfld, output scattered wfld 
  subroutine phsscr_space(gwfd, bwfd, wtmp, bvel, ws, opsign, plan_2dxF, plan_2dxI)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    integer                 :: iop, ikx, iky
    complex                 :: isdz
    real                    :: kk2

!   optimizing in velocity
!   i * sign * dKz/dv * dz
!   dKz/dv = -w/v/v / sqrt(1-v*v/w/w*|k|^2)

    isdz = cmplx(0., 1.)*dz*opsign

    gwfd = bwfd

    do iop=1, nopbsc
       wtmp = bwfd 
       call sfftw_dft_2dx_oper(plan_2dxF, wtmp, nx, ny)
       do iky=1, ny
       do ikx=1, nx
          kk2 = kx2(ikx)+ky2(iky)
          wtmp(ikx,iky) = cc(iop)*(kk2**iop)*wtmp(ikx,iky)
       end do
       end do
       call sfftw_dft_2dx_oper(plan_2dxI, wtmp, nx, ny)
       gwfd = gwfd + wtmp*((bvel/ws*bvel/ws)**iop)
    end do

    gwfd = gwfd*isdz*(-ws/bvel/bvel)

    return
  end subroutine

  subroutine phsscr_fouri(gwfd, bwfd, wtmp, bvel, ws, opsign, plan_2dxF, plan_2dxI)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    integer                 :: iop, ikx, iky
    complex                 :: isdz
    real                    :: kk2, vavr, kz2, etabsc

!   i*sign*dz
    isdz = cmplx(0., 1.)*dz*opsign
    vavr = sum(bvel)/nx/ny

    wtmp = bwfd

    call sfftw_dft_2dx_oper(plan_2dxF, wtmp, nx, ny)

    etabsc = cos(80*3.14159265359/180.)**2

    do iky=1, ny
    do ikx=1, nx
       kk2 = kx2(ikx)+ky2(iky)
       kz2 = 1 - kk2/ws/ws*vavr*vavr

       if (kz2 >= etabsc) then
          wtmp(ikx,iky) = wtmp(ikx,iky)/sqrt(kz2)
       else
          wtmp(ikx,iky) = 0.
       end if

    end do
    end do

    call sfftw_dft_2dx_oper(plan_2dxI, wtmp, nx, ny)

    gwfd = wtmp * isdz * (-ws/vavr/vavr)

    return
  end subroutine

  subroutine phsscr_pspi(gwfd, bwfd, wtmp, bvel, ws, opsign, plan_2dxF, plan_2dxI)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    complex, dimension(:,:) :: wtmp
    real   , dimension(:,:) :: bvel
    real                    :: ws
    integer                 :: opsign
    integer*8               :: plan_2dxF, plan_2dxI

    complex                 :: isdz
    real                    :: vmin, vmax, dvrf, kk2, kz2, etabsc, vdif1, vdif2
    integer                 :: ix, iy, ikx, iky, ivrf, jvrf, nvrf

    integer, dimension(:,:), allocatable :: vmap
    real   , dimension(:)  , allocatable :: vref
    
    nvrf = 2

    vmin = minval(bvel)
    vmax = maxval(bvel)
    dvrf = (vmax-vmin)/(nvrf-1)

    if (.not.allocated(vmap)) allocate(vmap(nx,ny))
    if (.not.allocated(vref)) allocate(vref(nvrf))

    if (nvrf==1) then
       vref(1) = minval(bvel)
    else
       do ivrf=1, nvrf
          vref(ivrf) = vmin + (ivrf-1)*dvrf
       end do
    end if

    do iy=1, ny
    do ix=1, nx
       jvrf = 1
       do ivrf=1, nvrf-1
          vdif1 = abs(bvel(ix,iy)-vref(ivrf  ))
          vdif2 = abs(bvel(ix,iy)-vref(ivrf+1))
          if (vdif1<vdif2) then
             jvrf = ivrf
          else
             jvrf = ivrf+1
          end if
       end do
       vmap(ix,iy) = jvrf
    end do
    end do

    etabsc = cos(80*3.14159265359/180.)**2

    do ivrf=1, nvrf
       wtmp = bwfd
       call sfftw_dft_2dx_oper(plan_2dxF, wtmp, nx, ny) 
       do iky=1, ny
       do ikx=1, nx
          kk2 = kx2(ikx)+ky2(iky)
          kz2 = 1 - kk2/ws/ws*vref(ivrf)*vref(ivrf)
          if (kz2>=etabsc) then
             wtmp(ikx,iky) = wtmp(ikx,iky)/sqrt(kz2) 
          else 
             wtmp(ikx,iky) = 0.
          end if
       end do
       end do
       call sfftw_dft_2dx_oper(plan_2dxI, wtmp, nx, ny)

       where (vmap == ivrf)
             gwfd = wtmp
       end where 
    end do 

    isdz = cmplx(0., 1.)*dz*opsign
    gwfd = gwfd * isdz * (-ws/bvel/bvel)
    return
  end subroutine


  subroutine phsscr_fidif(gwfd, bwfd, bvel, ws, opsign)
    complex, dimension(:,:) :: gwfd
    complex, dimension(:,:) :: bwfd
    real   , dimension(:,:) :: bvel
    real                    :: ws
    integer                 :: opsign

    real                    :: dx2, w2, s2
    complex                 :: isdz, lapxo
    integer                 :: ix

!    isdz = cmplx(0., 1.)*dz*opsign
!    dx2  = dx*dx
!    w2   = ws*ws
!    do ix=1, nx
!       s2 = 1./bvel(ix,1)/bvel(ix,1)
!       if (ix == 1) then
!          lapxo = (bwfd(ix+1,1)-bwfd(ix,1))/dx
!       else if (ix == nx) then
!          lapxo = (bwfd(ix,1)-bwfd(ix-1,1))/dx
!       else
!          lapxo = (bwfd(ix-1,1)-2.*bwfd(ix,1)+bwfd(ix+1,1))/dx2
!       end if
!       gwfd(ix,1) = iwdz*(bwfd(ix,1)+0.5/w2/s2*lapxo)
!    end do
    return
  end subroutine

  subroutine bwi_phsscr_coef(cc)
    real, dimension(4) :: cc

    cc(1) = 1./2.
    cc(2) = 3./8.
    cc(3) = 5./16.
    cc(4) = 35./128.
    return
  end subroutine




end module
