module bwi_combos_mod
  use  bwi_struct_mod
  implicit none
contains

  integer function bwi_combos_oper(imag, ahx, ahy, hcut, dsotyp, objeps) result(stat)
    real, dimension(:,:,:,:) :: imag
    type(axis)               :: ahx, ahy
    real                     :: hcut
    integer                  :: dsotyp
    real                     :: objeps

    real                     :: hxmax, hymax, hmax
    real                     :: hx, hy, hh, wt, damp
    integer                  :: ihx, ihy, jhx0, jhy0

    jhx0 = 1.5 + (0-ahx%o)/ahx%d
    jhy0 = 1.5 + (0-ahy%o)/ahy%d 

    select case (dsotyp)

      case (1) !    |h|
           hxmax = max(abs(ahx%o+(ahx%n-1)*ahx%d), abs(ahx%o))
           hymax = max(abs(ahy%o+(ahy%n-1)*ahy%d), abs(ahy%o))
           hmax  = sqrt(hxmax*hxmax+hymax*hymax)
           do ihy=1, ahy%n; hy = ahy%o+(ihy-1)*ahy%d
           do ihx=1, ahx%n; hx = ahx%o+(ihx-1)*ahx%d
              if (ihx==jhx0 .and. ihy==jhy0) then
                 imag(ihx,ihy,:,:) = -imag(ihx,ihy,:,:)*objeps
              else
                 hh = max(sqrt(hx*hx+hy*hy)-hcut, 0.)/hmax
                 wt = hh*hh
                 imag(ihx,ihy,:,:) = imag(ihx,ihy,:,:)*wt
              end if
           end do 
           end do

      case (2) ! Gaussian taper function
           hxmax = max(abs(ahx%o+(ahx%n-1)*ahx%d), abs(ahx%o))
           hymax = max(abs(ahy%o+(ahy%n-1)*ahy%d), abs(ahy%o))
           hmax  = sqrt(hxmax*hxmax+hymax*hymax)
           damp  = 6*log(10.)/((hmax-hcut)**2)
           do ihy=1, ahy%n; hy = ahy%o+(ihy-1)*ahy%d
           do ihx=1, ahx%n; hx = ahx%o+(ihx-1)*ahx%d
              if (ihx==jhx0 .and. ihy==jhy0) then
                 imag(ihx,ihy,:,:) = -imag(ihx,ihy,:,:)*objeps
              else
                 hh = sqrt(hx*hx+hy*hy)
                 if (hh<hcut) then
                    wt = 0.
                 else
                    wt = 1-exp(-damp*(hh-hcut)**2)
                 end if
                 wt = wt*wt
                 imag(ihx,ihy,:,:) = imag(ihx,ihy,:,:)*wt
              end if
           end do
           end do

      case (3) ! modified DSO, not proved yet
           hxmax = max(abs(ahx%o+(ahx%n-1)*ahx%d), abs(ahx%o))
           hymax = max(abs(ahy%o+(ahy%n-1)*ahy%d), abs(ahy%o))
           hmax  = sqrt(hxmax*hxmax+hymax*hymax)
           do ihy=1, ahy%n; hy = ahy%o+(ihy-1)*ahy%d
           do ihx=1, ahx%n; hx = ahx%o+(ihx-1)*ahx%d
              if (ihx==jhx0 .and. ihy==jhy0) then
                 imag(ihx,ihy,:,:) = -imag(ihx,ihy,:,:)*objeps
              else
                 hh = max(sqrt(hx*hx+hy*hy)-hcut, 0.)/hmax
                 wt = hh
                 imag(ihx,ihy,:,:) = imag(ihx,ihy,:,:)*wt
              end if
           end do
           end do
           do ihx=1, ahx%n/2-1
              imag(ihx        ,1,:,:) = imag(ihx+1    ,1,:,:) - imag(ihx        ,1,:,:)
              imag(ahx%n-ihx+1,1,:,:) = imag(ahx%n-ihx,1,:,:) - imag(ahx%n-ihx+1,1,:,:)
           end do
           imag(ahx%n/2        ,1,:,:) = - imag(ahx%n/2        ,1,:,:)
           imag(ahx%n-ahx%n/2+1,1,:,:) = - imag(ahx%n-ahx%n/2+1,1,:,:)


    end select
    stat = 0
  end function

end module
