module bwi_fileio_mod
  use  sep
  use  bwi_struct_mod
  use  bwi_struct_mod
  implicit none

  integer, external, private :: sreed_window, srite_window, getch

contains
!-------------------------------------------------------------------------------------
! initialize real image
!-------------------------------------------------------------------------------------
  subroutine bwi_fio_rimg_init(igrid, filetag)
    type(img_grid)                  :: igrid
    character(len=*)                :: filetag
    real, allocatable, dimension(:) :: tmp
    integer                         :: ioffs, nmodl, noffs, nshxy
    nmodl = igrid%ax%n*igrid%ay%n*igrid%az%n
    noffs = igrid%ahx%n*igrid%ahy%n
    nshxy = igrid%asx%n*igrid%asy%n
    allocate(tmp(nmodl))
    tmp = 0.
    call sseek(filetag, 0, 0)
    do ioffs=1, noffs*nshxy
       call sep_write(tmp, filetag, esize=4)
    end do
    call sseek(filetag, 0, 0)
    deallocate(tmp)
    return
  end subroutine
!-------------------------------------------------------------------------------------
! initialize complex image
!-------------------------------------------------------------------------------------
  subroutine bwi_fio_cimg_init(igrid, filetag)
    type(img_grid)                     :: igrid
    character(len=*)                   :: filetag
    complex, allocatable, dimension(:) :: tmp
    integer                            :: ioffs, nmodl, noffs, nshxy
    nmodl = igrid%ax%n*igrid%ay%n*igrid%az%n
    noffs = igrid%ahx%n*igrid%ahy%n
    nshxy = igrid%asx%n*igrid%asy%n
    allocate(tmp(nmodl))
    tmp = 0.
    call sseek(filetag, 0, 0)
    do ioffs=1, noffs*nshxy
       call sep_write(tmp, filetag, esize=8)
    end do
    call sseek(filetag, 0, 0)
    deallocate(tmp)
    return
  end subroutine
!-------------------------------------------------------------------------------------
  subroutine bwi_fio_crec_init(dgrid, filetag)
    type(dat_grid)                     :: dgrid
    character(len=*)                   :: filetag
    complex, allocatable, dimension(:) :: tmp
    integer                            :: ish
    allocate(tmp(dgrid%agx%n*dgrid%agy%n*dgrid%aw%n))
    tmp = 0.
    call sseek(filetag, 0, 0)
    do ish=1, dgrid%nshg
       call sep_write(tmp, filetag, esize=8) 
    end do
    call sseek(filetag, 0, 0)
    deallocate(tmp)
    return
  end subroutine
!-------------------------------------------------------------------------------------
  subroutine bwi_fio_dvel_init(vgrid, filetag)
    character(len=*)                :: filetag
    type(vel_grid)                  :: vgrid
    real, allocatable, dimension(:) :: tmp
    allocate(tmp(vgrid%ax%n*vgrid%ay%n*vgrid%az%n))
    tmp = 0.
    call sseek(filetag, 0, 0)
    call sep_write(tmp, filetag, esize=4)
    deallocate(tmp)
    return
  end subroutine
  
!-------------------------------------------------------------------------------------
  subroutine bwi_fio_grdt_init(vgrid, filetag)
    character(len=*)                :: filetag
    type(vel_grid)                  :: vgrid
    real, allocatable, dimension(:) :: tmp
    allocate(tmp(vgrid%ax%n*vgrid%ay%n*vgrid%az%n))
    tmp = 0.
    call sseek(filetag, 0, 0)
    call sep_write(tmp, filetag, esize=4)
    deallocate(tmp)
    return
  end subroutine

!-------------------------------------------------------------------------------------
  subroutine bwi_fio_hess_init(hgrid, filetag)
    type(hes_grid)                  :: hgrid
    character(len=*)                :: filetag
    real, allocatable, dimension(:) :: tmp
    integer                         :: ioffs, nmodl, noffs, nshxy
    nmodl = hgrid%ax_modl%n*hgrid%ay_modl%n*hgrid%az_modl%n
    noffs = hgrid%ahx_modl%n*hgrid%ahy_modl%n
    nshxy = hgrid%asx%n*hgrid%asy%n
    allocate(tmp(nmodl))
    tmp = 0.
    call sseek(filetag, 0, 0)
    do ioffs=1, noffs*nshxy
       call sep_write(tmp, filetag, esize=4)
    end do
    call sseek(filetag, 0, 0)
    deallocate(tmp)
    return
  end subroutine
!=====================================================================================
! read or write one depth block image, real image
!-------------------------------------------------------------------------------------
  subroutine bwi_fio_rimg(modl, igrid, ogrid, jz, jsx, jsy, filetag, flag)
    real, dimension(:,:,:,:) :: modl
    type(img_grid)           :: igrid
    type(out_grid)           :: ogrid
    integer                  :: jz, jsx, jsy
    character(len=*)         :: filetag
    character(len=4)         :: flag
            
    integer, dimension(7)    :: n_cube, j_wind, n_wind, f_wind
    integer                  :: n_dim_cube, err

    n_dim_cube = 7
    n_cube = (/igrid%ahx%n,igrid%ahy%n,igrid%ax%n,igrid%ay%n,igrid%az%n,igrid%asx%n,igrid%asy%n/)
    j_wind = (/1,1,1,1,1,1,1/)
    n_wind = (/ogrid%ahx%n,ogrid%ahy%n,ogrid%ax%n,ogrid%ay%n,1,1,1/)
    f_wind = (/0,0,ogrid%index_outimg_xmin-1,ogrid%index_outimg_ymin-1,jz-1,jsx-1,jsy-1/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,modl)
       if (err .lt. 0) call seperr("wem_io_imag: trouble writing image file")
    else if(flag == "reed") then
       modl = 0.
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,modl)
       if (err .lt. 0) call seperr("wem_io_imag: trouble reading image file")
    else
       call seperr("you have to specify the flag for wem_io_imag()")
    end if

    return
  end subroutine
!=====================================================================================
! read or write one depth block image, complex image
!-------------------------------------------------------------------------------------
  subroutine bwi_fio_cimg(modl, igrid, ogrid, jz, jsx, jsy, filetag, flag)
    complex, dimension(:,:,:,:) :: modl
    type(img_grid)              :: igrid
    type(out_grid)              :: ogrid
    integer                     :: jz, jsx, jsy
    character(len=*)            :: filetag
    character(len=4)            :: flag

    integer, dimension(7)       :: n_cube, j_wind, n_wind, f_wind
    integer                     :: n_dim_cube, err
    n_dim_cube = 7
    n_cube = (/igrid%ahx%n,igrid%ahy%n,igrid%ax%n,igrid%ay%n,igrid%az%n,igrid%asx%n,igrid%asy%n/)
    j_wind = (/1,1,1,1,1,1,1/)
    n_wind = (/ogrid%ahx%n,ogrid%ahy%n,ogrid%ax%n,ogrid%ay%n,1,1,1/)
    f_wind = (/0,0,ogrid%index_outimg_xmin-1,ogrid%index_outimg_ymin-1,jz-1,jsx-1,jsy-1/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,8,modl)
       if (err .lt. 0) call seperr("wem_io_imag: trouble writing image file")
    else if(flag == "reed") then
       modl = 0.
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,8,modl)
       if (err .lt. 0) call seperr("wem_io_imag: trouble reading image file")
    else
       call seperr("you have to specify the flag for wem_io_imag()")
    end if
    return
  end subroutine
!=====================================================================================
! read or write one depth block Hessian
!-------------------------------------------------------------------------------------
  subroutine bwi_fio_hess(hess, hgrid, ogrid, jz, jsx, jsy, filetag, flag)
    real, dimension(:,:,:,:) :: hess
    type(hes_grid)           :: hgrid
    type(out_grid)           :: ogrid
    integer                  :: jz, jsx, jsy
    character(len=*)         :: filetag
    character(len=4)         :: flag

    integer, dimension(7)    :: n_cube, j_wind, n_wind, f_wind
    integer                  :: n_dim_cube, err

    n_dim_cube = 7
    n_cube = (/hgrid%ahx_modl%n,hgrid%ahy_modl%n,hgrid%ax_modl%n,hgrid%ay_modl%n,hgrid%az_modl%n,hgrid%asx%n,hgrid%asy%n/)
    j_wind = (/1,1,1,1,1,1,1/)
    n_wind = (/ogrid%ahx%n,ogrid%ahy%n,ogrid%ax%n,ogrid%ay%n,1,1,1/)
    f_wind = (/0,0,ogrid%index_outimg_xmin-1,ogrid%index_outimg_ymin-1,jz-1,jsx-1,jsy-1/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,hess)
       if (err .lt. 0) call seperr("bwi_fio_hess: trouble writing image file")
    else if(flag == "reed") then
       hess = 0.
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,hess)
       if (err .lt. 0) call seperr("bwi_fio_hess: trouble reading image file")
    else
       call seperr("you have to specify the flag for wem_io_imag()")
    end if

    return
  end subroutine

!=============================================================================
! read one frequency block receiver wavefield
!-----------------------------------------------------------------------------
  subroutine bwi_fio_crec(crec, dgrid, indw, nws, ish, filetag, flag)
    complex, dimension(:,:) :: crec
    type(dat_grid)          :: dgrid 
    integer                 :: indw, nws, ish
    character(len=*)        :: filetag
    character(len=4)        :: flag
    integer, dimension(3)   :: n_cube, j_wind, n_wind, f_wind
    integer                 :: n_dim_cube, err, i2  

    n_dim_cube = 3
    n_cube = (/dgrid%ntrg, dgrid%aw%n, dgrid%nshg/)
    j_wind = (/1, 1, 1/)
    n_wind = (/dgrid%ntrg, nws, 1/)
    f_wind = (/0, indw-1, ish-1/)

    if (flag == "rite") then
	   do i2=1,n_wind(2)
          call sseek_block(filetag,f_wind(3),n_cube(1)*n_cube(2)*8,0) !seek to the right depth
          call sseek_block(filetag,f_wind(2)+i2,n_cube(1)*8,1)   !seek to the  y location
          call sseek_block(filetag,f_wind(1),8,1)
          call srite(filetag,crec(:,i2),8*n_wind(1))
       end do
    else if(flag == "reed") then
       crec = 0.
       do i2=1,n_wind(2)
          call sseek_block(filetag,f_wind(3),n_cube(1)*n_cube(2)*8,0) !seek to the right depth
          call sseek_block(filetag,f_wind(2)+i2,n_cube(1)*8,1)   !seek to the  y location
          call sseek_block(filetag,f_wind(1),8,1)
          call sreed(filetag,crec(:,i2),8*n_wind(1))
       end do
    else
       call seperr("you have to specify the flag for wem_fio_crec()")
    end if
    return
  end subroutine

!=============================================================================
! read one frequency block receiver wavefield
!-----------------------------------------------------------------------------
  subroutine bwi_fio_csou(csou, dgrid, indw, nws, ish, filetag, flag)
    complex, dimension(:,:) :: csou
    type(dat_grid)          :: dgrid
    integer                 :: indw, nws, ish
    character(len=*)        :: filetag
    character(len=4)        :: flag
    integer, dimension(3)   :: n_cube, j_wind, n_wind, f_wind
    integer                 :: n_dim_cube, err, i2

    n_dim_cube = 3
    n_cube = (/dgrid%ntrs, dgrid%aw%n, dgrid%nshs/)
    j_wind = (/1, 1, 1/)
    n_wind = (/dgrid%ntrs, nws, 1/)
    if (dgrid%comsou) then 
       f_wind = (/0, indw-1, ish-1/)
    else
       f_wind = (/0, indw-1, 0/)
    end if   
    if (flag == "rite") then
	   do i2=1,n_wind(2)
          call sseek_block(filetag,f_wind(3),n_cube(1)*n_cube(2)*8,0) !seek to the right depth
          call sseek_block(filetag,f_wind(2)+i2,n_cube(1)*8,1)   !seek to the  y location
          call sseek_block(filetag,f_wind(1),8,1)
          call srite(filetag,csou(:,i2),8*n_wind(1))
       end do
       csou = cmplx(0.,0.)
    else if(flag == "reed") then
       csou = cmplx(0.,0.)
       do i2=1,n_wind(2)
          call sseek_block(filetag,f_wind(3),n_cube(1)*n_cube(2)*8,0) !seek to the right depth
          call sseek_block(filetag,f_wind(2)+i2,n_cube(1)*8,1)   !seek to the  y location
          call sseek_block(filetag,f_wind(1),8,1)
          call sreed(filetag,csou(:,i2),8*n_wind(1))
       end do
    else
       call seperr("you have to specify the flag for wem_fio_csou()")
    end if
    return
  end subroutine
!==================================================================================
  subroutine bwi_fio_sfnc(sfnc, dgrid, iws, nws, filetag, flag)
    complex, dimension(:) :: sfnc
    type(dat_grid)        :: dgrid
    integer               :: nws
    integer               :: iws
    character(len=*)      :: filetag
    character(len=4)      :: flag
       
    integer, dimension(1) :: n_cube, j_wind, n_wind, f_wind
    integer               :: n_dim_cube, err
    n_dim_cube = 1
    n_cube = (/dgrid%aw%n/)
    j_wind = (/1/)
    n_wind = (/nws/)
    f_wind = (/iws-1/)
    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,8,sfnc)
    else if (flag == "reed") then
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,8,sfnc)
    end if
    return
  end subroutine
!==================================================================================
  subroutine bwi_fio_velo(vmod, vgrid, pgrid, indz, filetag, flag)
    real, dimension(:,:)   :: vmod
    type(vel_grid)         :: vgrid
    type(pro_grid)         :: pgrid
    integer                :: indz
    character(len=*)       :: filetag
    character(len=4)       :: flag
    integer, dimension(3)  :: n_cube, j_wind, n_wind, f_wind
    integer                :: n_dim_cube, err, i2

    n_dim_cube = 3
    n_cube = (/vgrid%ax%n, vgrid%ay%n, vgrid%az%n/)
    j_wind = (/1, 1, 1/)
    n_wind = (/pgrid%ax%n, pgrid%ay%n, 1/)
    f_wind = (/vgrid%index_provel_xmin-1, vgrid%index_provel_ymin-1, indz-1/)
    if (flag == "rite") then
	   do i2=1,n_wind(2)
          call sseek_block(filetag,f_wind(3),n_cube(1)*n_cube(2)*4,0) !seek to the right depth
          call sseek_block(filetag,f_wind(2)+i2-1,n_cube(1)*4,1)   !seek to the  y location
          call sseek_block(filetag,f_wind(1),4,1)
          call srite(filetag,vmod(:,i2),4*n_wind(1))
       end do
       vmod = 0.
    else if (flag == "reed") then
       vmod = 0.
	   do i2=1,n_wind(2)
          call sseek_block(filetag,f_wind(3),n_cube(1)*n_cube(2)*4,0) !seek to the right depth
          call sseek_block(filetag,f_wind(2)+i2-1,n_cube(1)*4,1)   !seek to the  y location
          call sseek_block(filetag,f_wind(1),4,1)
          call sreed(filetag,vmod(:,i2),4*n_wind(1))
       end do
    end if
    return
  end subroutine
!==================================================================================
  subroutine bwi_fio_coef(coef, cgrid, filetag)
    real, dimension(:,:,:) :: coef 
    type(cef_grid)         :: cgrid
    character(len=*)       :: filetag
      call sreed(filetag,coef,cgrid%step%n*cgrid%cfd%n*cgrid%cfe%n*4) 
    return
  end subroutine

!==================================================================================
! source header
  subroutine bwi_fio_sdpt(dpth, dgrid, filetag)
    real, dimension(:,:)   :: dpth 
    type(dat_grid)         :: dgrid
    character(len=*)       :: filetag

	call sreed(filetag,dpth,dgrid%asx%n*dgrid%asy%n*4)
    return
  end subroutine

!==================================================================================
! receiver header
  subroutine bwi_fio_rdpt(dpth, dgrid, filetag)
    real, dimension(:,:)   :: dpth 
    type(dat_grid)         :: dgrid
    character(len=*)       :: filetag

	call sreed(filetag,dpth,dgrid%agx%n*dgrid%agy%n*4)
    return
  end subroutine

!===================================================================================
  subroutine bwi_fio_wfld(wfld, dgrid, pgrid, indw, nws, ish, filetag, flag)
    complex, dimension(:,:,:,:) :: wfld
    type(dat_grid)              :: dgrid
    type(pro_grid)              :: pgrid
    integer                     :: indw, nws, ish
    character(len=*)            :: filetag
    character(len=4)            :: flag
    integer, dimension(5)       :: n_cube, j_wind, n_wind, f_wind
    integer                     :: n_dim_cube, err

    n_dim_cube = 5
    n_cube = (/pgrid%ax%n, pgrid%ay%n, dgrid%aw%n, pgrid%nzstep, dgrid%nshg/)
    j_wind = (/1, 1, 1, 1, 1/)
    n_wind = (/pgrid%ax%n, pgrid%ay%n, nws, pgrid%nzstep, 1/)
    f_wind = (/0, 0, indw-1, 0, ish-1/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,8,wfld(:,:,1:nws,:))
       if (err.lt.0) call seperr("wem_fio_crec: trouble writing receiver wavefield")
    else if(flag == "reed") then
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,8,wfld(:,:,1:nws,:))
       if (err.lt.0) call seperr("wem_fio_crec: trouble reading receiver wavefield")
    else
       call seperr("you have to specify the flag for wem_fio_crec()")
    end if
    return
  end subroutine
!====================================================================================
  subroutine bwi_fio_imag_updt(imag, igrid, jz, filetag, flag)
    real, dimension(:,:,:,:) :: imag
    type(img_grid)           :: igrid
    integer                  :: jz
    character(len=*)         :: filetag
    character(len=4)         :: flag

    integer, dimension(5)    :: n_cube, j_wind, n_wind, f_wind
    integer                  :: n_dim_cube, err

    n_dim_cube = 5
    n_cube = (/igrid%ahx%n,igrid%ahy%n,igrid%ax%n,igrid%ay%n,igrid%az%n/)
    j_wind = (/1,1,1,1,1/)
    n_wind = (/igrid%ahx%n,igrid%ahy%n,igrid%ax%n,igrid%ay%n,1/)
    f_wind = (/0,0,0,0,jz-1/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,imag)
       if (err .lt. 0) call seperr("wem_io_imag: trouble writing image file")
    else if(flag == "reed") then
       imag = 0.
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,imag)
       if (err .lt. 0) call seperr("wem_io_imag: trouble reading image file")
    else
       call seperr("you have to specify the flag for wem_io_imag()")
    end if
    return
  end subroutine
!====================================================================================
  subroutine bwi_fio_istk_updt(istk, igrid, filetag, flag)
    real, dimension(:,:,:) :: istk
    type(img_grid)         :: igrid
    character(len=*)       :: filetag
    character(len=4)       :: flag

    integer, dimension(5)  :: n_cube, j_wind, n_wind, f_wind
    integer                :: n_dim_cube, err
    integer                :: jhx0, jhy0

    jhx0 = 1.5 + (0.-igrid%ahx%o)/igrid%ahx%d 
    jhy0 = 1.5 + (0.-igrid%ahy%o)/igrid%ahy%d

    n_dim_cube = 5
    n_cube  = (/igrid%ahx%n,igrid%ahy%n,igrid%ax%n,igrid%ay%n,igrid%az%n/)
    j_wind = (/1,1,1,1,1/)
    n_wind = (/1,1,igrid%ax%n,igrid%ay%n,igrid%az%n/)
    f_wind = (/jhx0-1,jhy0-1,0,0,0/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,istk)
       if (err .lt. 0) call seperr("bwi_fio_istk_updt: trouble writing image file")
    else if (flag == "reed") then
       istk = 0.
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,istk)
       if (err .lt. 0) call seperr("bwi_fio_istk_updt: trouble reading image file")
    else 
       call seperr("you have to specify the flag for bwi_fio_updt()")
    end if
    return
  end subroutine
!====================================================================================
  subroutine bwi_fio_imsk_updt(imsk, igrid, jz, filetag, flag)
    real, dimension(:,:)     :: imsk
    type(img_grid)           :: igrid
    integer                  :: jz
    character(len=*)         :: filetag
    character(len=4)         :: flag

    integer, dimension(3)    :: n_cube, j_wind, n_wind, f_wind
    integer                  :: n_dim_cube, err

    n_dim_cube = 3
    n_cube = (/igrid%ax%n,igrid%ay%n,igrid%az%n/)
    j_wind = (/1,1,1/)
    n_wind = (/igrid%ax%n,igrid%ay%n,1/)
    f_wind = (/0,0,jz-1/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,imsk)
       if (err .lt. 0) call seperr("wem_io_imag: trouble writing image file")
    else if(flag == "reed") then
       imsk = 0.
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,imsk)
       if (err .lt. 0) call seperr("wem_io_imag: trouble reading image file")
    else
       call seperr("you have to specify the flag for wem_io_imag()")
    end if
    return
  end subroutine
!====================================================================================
  subroutine bwi_fio_fmap_updt(fmap, igrid, jz, filetag, flag)
    real, dimension(:,:)     :: fmap
    type(img_grid)           :: igrid
    integer                  :: jz
    character(len=*)         :: filetag
    character(len=4)         :: flag

    integer, dimension(3)    :: n_cube, j_wind, n_wind, f_wind
    integer                  :: n_dim_cube, err

    n_dim_cube = 3
    n_cube = (/igrid%ax%n,igrid%ay%n,igrid%az%n/)
    j_wind = (/1,1,1/)
    n_wind = (/igrid%ax%n,igrid%ay%n,1/)
    f_wind = (/0,0,jz-1/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,fmap)
       if (err .lt. 0) call seperr("wem_io_imag: trouble writing fmap file")
    else if(flag == "reed") then
       fmap = 0.
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,fmap)
       if (err .lt. 0) call seperr("wem_io_imag: trouble reading fmap file")
    else
       call seperr("you have to specify the flag for bwi_fio_fmap_updt()")
    end if
    return
  end subroutine

!====================================================================================
  subroutine bwi_fio_vmod_updt(vmod, vgrid, filetag, flag)
    real, dimension(:,:,:) :: vmod
    type(vel_grid)         :: vgrid
    character(len=*)       :: filetag
    character(len=4)       :: flag

    integer, dimension(3) :: n_cube, j_wind, n_wind, f_wind
    integer               :: n_dim_cube, err

    n_dim_cube = 3
    n_cube = (/vgrid%ax%n,vgrid%ay%n,vgrid%az%n/)
    j_wind = (/1,1,1/)
    n_wind = (/vgrid%ax%n,vgrid%ay%n,vgrid%az%n/)
    f_wind = (/0,0,0/)

    if (flag == "rite") then
       err = srite_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,vmod)
       if (err .lt. 0) call seperr("wem_io_imag: trouble writing image file")
    else if(flag == "reed") then
       vmod = 0.
       err = sreed_window(filetag,n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,vmod)
       if (err .lt. 0) call seperr("wem_io_imag: trouble reading image file")
    else
       call seperr("you have to specify the flag for wem_io_imag()")
    end if
    return
  end subroutine

!====================================================================================
  subroutine bwi_fio_stdv(stdv, vgrid, filetag, flag)
    real, dimension(:,:,:,:)   :: stdv 
    type(vel_grid)             :: vgrid
    character(len=*)           :: filetag
    character(len=4)           :: flag

    if (flag == "rite") then
		call seperr("No rite option for standard deviation")	
    else if(flag == "reed") then
		stdv = 0.
		call sreed(filetag,stdv,6*vgrid%ax%n*vgrid%ay%n*vgrid%az%n*4)
    else
       call seperr("you have to specify the flag for wem_io_imag()")
    end if
    return
  end subroutine

end module
