module bwi_search_mod
  use base_struct_mod
  use bwi_kernel_mod
  use bwi_restom_mod
  implicit none

contains

  integer function bwi_sdstep_oper(srhv_, srhe_, gdov_, gdoe_, vgrid) result(stat)
    character(len=*)                    :: srhv_, srhe_
	character(len=*)					:: gdov_, gdoe_
    type(vel_grid)                      :: vgrid
    real, dimension(:,:,:), allocatable :: srhv, srhe

    allocate(srhv(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(srhe(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    call bwi_fio_vmod_updt(srhv, vgrid, gdov_, "reed")
    call bwi_fio_vmod_updt(srhe, vgrid, gdoe_, "reed")
    srhv = -srhv; srhe = -srhe;
    call bwi_fio_vmod_updt(srhv, vgrid, srhv_, "rite")
    call bwi_fio_vmod_updt(srhe, vgrid, srhe_, "rite")
    deallocate(srhv,srhe)
    stat = 0.
  end function

  integer function bwi_cgstep_oper(srhv_,srhe_,gdov_,gdoe_,gdpv_,gdpe_,vgrid,restart) result(stat)
    character(len=*)                    :: srhv_, srhe_
    character(len=*)                    :: gdov_, gdoe_
    character(len=*)                    :: gdpv_, gdpe_
    type(vel_grid)                      :: vgrid
    logical                             :: restart

    double precision                    :: beta
    real, dimension(:,:,:), allocatable :: gdov, gdoe
    real, dimension(:,:,:), allocatable :: gdpv, gdpe
    real, dimension(:,:,:), allocatable :: srhv, srhe

    allocate(gdov(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(gdoe(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(gdpv(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(gdpe(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(srhv(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(srhe(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))

!   gradient for the k'th iteration
    call bwi_fio_vmod_updt(gdov, vgrid, gdov_, "reed") ! k
    call bwi_fio_vmod_updt(gdoe, vgrid, gdoe_, "reed") ! k
!   gradient for the k+1'th iteration
    call bwi_fio_vmod_updt(gdpv, vgrid, gdpv_, "reed") ! k+1
    call bwi_fio_vmod_updt(gdpe, vgrid, gdpe_, "reed") ! k+1
!   restarting
    if (restart) then
       beta = 0.
    else
!      compute beta, using the Polak-Ribiere method
       beta = (sum(dprod(gdpv,gdpv-gdov))+sum(dprod(gdpe,gdpe-gdoe)))/(sum(dprod(gdov,gdov))+sum(dprod(gdoe,gdoe))+epsilon(beta))
!      safe guarding
       beta = max(beta, 0.)
    end if
!    write(0,*)"beta = ", beta
!   read the search direction for the k'th iteration (current)
    call bwi_fio_vmod_updt(srhv, vgrid, srhv_, "reed")
    call bwi_fio_vmod_updt(srhe, vgrid, srhe_, "reed")
!   update the search direction
    srhv = - gdov + beta*srhv
    srhe = - gdoe + beta*srhe
!   write out new search direction
    call bwi_fio_vmod_updt(srhv, vgrid, srhv_, "rite")
    call bwi_fio_vmod_updt(srhe, vgrid, srhe_, "rite")
!   shuffle the gradient
    call bwi_fio_vmod_updt(gdpv, vgrid, gdov_, "rite")
    call bwi_fio_vmod_updt(gdpe, vgrid, gdoe_, "rite")
    deallocate(gdov,gdoe)
    deallocate(gdpv,gdpe)
    deallocate(srhv,srhe)
    stat = 0
  end function
 
  integer function bwi_linsec_oper(csou_, crec_, bvel_, beta_, bdel_, bimg_, hess_, srhv_, srhe_, imsk_, fmap_, vgrid, igrid, hgrid, nsrc, alph, fobj, &
                                   hcut, dsotyp, dsorms, objtyp, objeps, grdfix) result(stat)
    character(len=*)                    :: csou_, crec_, bvel_, beta_, bdel_, bimg_, hess_, srhv_, srhe_, imsk_, fmap_
    type(vel_grid)                      :: vgrid
    type(img_grid)                      :: igrid
    type(hes_grid)                      :: hgrid
    integer                             :: nsrc
    real                                :: alph
    double precision                    :: fobj
    real                                :: hcut
    integer                             :: dsotyp
    logical                             :: dsorms
    character(len=6)                    :: objtyp
    real                                :: objeps
    logical                             :: grdfix

    double precision                    :: f(nsrc), fopt, norm
    real                                :: sfac
    integer                             :: iopt, isrc

    real, dimension(:,:,:), allocatable :: gdov, gdoe
    real, dimension(:,:,:), allocatable :: gdpv, gdpe
    real, dimension(:,:,:), allocatable :: srhv, srhe

    allocate(gdov(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(gdoe(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(gdpv(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(gdpe(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(srhv(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))
    allocate(srhe(vgrid%ax%n,vgrid%ay%n,vgrid%az%n))

    sfac = 0.5*(nsrc+1)

!   read the background model to be updated
!   ---------------------------------------
    call bwi_fio_vmod_updt(gdov, vgrid, bvel_, "reed")
    call bwi_fio_vmod_updt(gdoe, vgrid, beta_, "reed")
!   read the search direction
!   -------------------------
    call bwi_fio_vmod_updt(srhv, vgrid, srhv_, "reed")
    call bwi_fio_vmod_updt(srhe, vgrid, srhe_, "reed")
!   normalize
!   ---------
!    norm = maxval(abs(srch))
!    srch = srch/norm
!   perturb the model
!   -----------------
    do isrc=1, nsrc
!      trial velocity model
!      -------------------- 
       gdpv = gdov + alph*isrc*srhv
       gdpe = gdoe + alph*isrc*srhe
       call bwi_fio_vmod_updt(gdpv, vgrid, bvel_, "rite")
       call bwi_fio_vmod_updt(gdpe, vgrid, beta_, "rite")
!      compute the image using the trial velocity
!      ------------------------------------------
       stat = bwi_kerimg_oper(.true., .false., csou_, crec_, bvel_, beta_, bdel_, bimg_)
!      compute the objective function
!      ------------------------------
       stat = bwi_restom_oper(bimg_, hess_, imsk_, fmap_, igrid, hgrid, f(isrc), hcut, dsotyp, dsorms, objtyp, objeps, grdfix)
    end do
!   find the minimum one
!   --------------------
    fopt = f(1)
    iopt = 1
    do isrc=1, nsrc
       if (f(isrc)<=fopt) then
          iopt = isrc
          fopt = f(isrc)
       end if
    end do
!   compare with the current objective function value
!   -------------------------------------------------
    if (f(iopt)<=fobj) then
       gdpv = gdov + alph*iopt*srhv
       gdpe = gdoe + alph*iopt*srhe
       alph = alph*iopt/sfac
       fobj = f(iopt)
    else
       gdpv = gdov
       gdpe = gdoe
       alph = alph * 0.5
    end if
    call bwi_fio_vmod_updt(gdpv, vgrid, bvel_, "rite")
    call bwi_fio_vmod_updt(gdpe, vgrid, beta_, "rite")
    deallocate(gdov,gdoe)
    deallocate(gdpv,gdpe)
    deallocate(srhv,srhe)
    stat = 0
  end function


end module
