!!---------------------------------------------------------------------------
!! Author: Yaxun Tang, tang@sep.stanford.edu
!!---------------------------------------------------------------------------
module bwi_ssrepi_mod
  implicit none

contains
  !----------------------------------------------------------------------------------------------------------------------
  !.. Multiple reference velocity extrapolation plus interpolation
  !----------------------------------------------------------------------------------------------------------------------
  integer function ssr_epi(adj, Fxop, wfld_slice, vmod_slice, vref_slice, kx2, ky2, ws, dz, nx, ny, nvrf, opsign, &
                           iwrk2d, cwrk3d, plan_2dxF, plan_2dxI) result(stat)
    logical                   :: adj
    complex, dimension(:,:)   :: wfld_slice
    real   , dimension(:,:)   :: vmod_slice
    real   , dimension(:)     :: vref_slice
    real   , dimension(:)     :: kx2, ky2
    real                      :: ws
    real                      :: dz
    integer                   :: nx, ny, nvrf
    integer                   :: opsign
    integer, dimension(:,:)   :: iwrk2d
    complex, dimension(:,:,:) :: cwrk3d
    integer*8                 :: plan_2dxF
    integer*8                 :: plan_2dxI

    interface
      integer function Fxop(adj, wfld_slice, vmod_slice, kx2, ky2, vref, ws, dz, nx, ny, opsign, plan_2dxF, plan_2dxI) result(stat)
        logical                 :: adj
        complex, dimension(:,:) :: wfld_slice
        real   , dimension(:,:) :: vmod_slice
        real   , dimension(:)   :: kx2, ky2
        real                    :: vref
        real                    :: ws
        real                    :: dz
        integer                 :: nx, ny
        integer                 :: opsign
        integer*8               :: plan_2dxF
        integer*8               :: plan_2dxI
      end function
    end interface

    integer                     :: ix, iy, itr, ivrf, jvrf
    real                        :: vmin, vmax, dvel, vref, vdif1, vdif2

    if (nvrf == 1) then
       vref = vref_slice(1)
       stat = FXop(adj, wfld_slice, vmod_slice, kx2, ky2, vref, ws, dz, nx, ny, opsign, plan_2dxF, plan_2dxI)

    else

       iwrk2d = 0
!      calculate the velocity map for wavefield interpolation
!      ------------------------------------------------------
       do iy=1, ny
       do ix=1, nx
          jvrf = 1
          do ivrf=1, nvrf-1
             vdif1 = abs(vmod_slice(ix,iy)-vref_slice(ivrf)  )
             vdif2 = abs(vmod_slice(ix,iy)-vref_slice(ivrf+1))
             if (vdif1<vdif2) then
                jvrf = ivrf
                exit
             else
                jvrf = ivrf+1
             end if
          end do
          iwrk2d(ix,iy) = jvrf
       end do
       end do

       if (adj) then

!         *********************
!         ADJOINT INTERPOLATION
!         *********************

!         copy current interpolated wavefield into a backup array
!         -------------------------------------------------------
          cwrk3d(:,:,1) = wfld_slice
!         zero out buffer
!         ---------------
          cwrk3d(:,:,2) = 0.
!         loop over reference velocities
!         ------------------------------
          do ivrf=1, nvrf
             vref = vref_slice(ivrf)
!            adjoint of nearest-neighbour interpolation
!            ------------------------------------------ 
             wfld_slice = 0.
             where (iwrk2d == ivrf)
                   wfld_slice = cwrk3d(:,:,1)
             end where
!            adjoint propagation
!            -------------------
             stat = FXop(adj, wfld_slice, vmod_slice, kx2, ky2, vref, ws, dz, nx, ny, opsign, plan_2dxF, plan_2dxI)
!            adjoint of copying wavefields
!            -----------------------------
             cwrk3d(:,:,2) = cwrk3d(:,:,2) + wfld_slice
          end do
!         copy the wavefield back
!         ----------------------- 
          wfld_slice = cwrk3d(:,:,2)

       else

!         *********************
!         FORWARD INTERPOLATION
!         *********************

!         copy current wavefield into a backup array
!         ------------------------------------------
          cwrk3d(:,:,1) = wfld_slice
!         zero out buffers
!         ----------------
          cwrk3d(:,:,2) = 0.
          wfld_slice    = 0.
!         loop over reference velocities
!         ------------------------------
          do ivrf=1, nvrf
!            propagate wavefields according to different reference velocities
!            ----------------------------------------------------------------
             vref = vref_slice(ivrf)
             wfld_slice = cwrk3d(:,:,1)
             stat = FXop(adj, wfld_slice, vmod_slice, kx2, ky2, vref, ws, dz, nx, ny, opsign, plan_2dxF, plan_2dxI)
!            nearest-neighbor interpolation
!            ------------------------------ 
             where (iwrk2d == ivrf)
                   cwrk3d(:,:,2) = wfld_slice
             end where
          end do
!         copy the interpolated wavefield back to current wavefield
!         ---------------------------------------------------------
          wfld_slice = cwrk3d(:,:,2)

       end if 
   end if
   stat = 0
  end function


end module
