!!.. read nzs depth step velocity or slowness
module bwi_velref_mod
  implicit none
  integer, private :: nvrf
contains

!=====================================================================
!
  integer function bwi_velref_init(nvrf_in) result(stat)
    integer :: nvrf_in
    integer :: nx_in
    integer :: ny_in

    nvrf  = nvrf_in

    stat  = 0
  end function
!======================================================================
  integer function bwi_velref_clos() result(stat)
    nvrf  = 0
    stat  = 0
  end function
!======================================================================
!.. select the reference velocity for the current depth step
!.. Lloyd's method will be used
!----------------------------------------------------------------------
  integer function bwi_velref_oper(vmod_slice, vref_slice) result(stat)
    real, dimension(:,:) :: vmod_slice
    real, dimension(:)   :: vref_slice

    integer              :: iref, nvrf_sec, ix, iy, ic
    real                 :: vmin_pri, vmax_pri
    real                 :: vmin_sec, vmax_sec, dvel

    stat = 1
    if(nvrf > size(vref_slice)) call seperr("ERROR: nref larger than the size of vref_slice(:)")
    vref_slice=0.0

    if (nvrf==1) then
       vref_slice(1) = sum(vmod_slice)/size(vmod_slice)

    else if (nvrf==2) then
       vref_slice(1) = minval(vmod_slice)
       vref_slice(2) = maxval(vmod_slice)

    else if (nvrf==3) then
       vref_slice(1) = minval(vmod_slice)
       vref_slice(2) = sum(vmod_slice)/size(vmod_slice)
       vref_slice(3) = maxval(vmod_slice)

    else
       vmin_pri = minval(vmod_slice)
       vmax_pri = maxval(vmod_slice)

       vmin_sec = minval(vmod_slice, MASK=vmod_slice-vmin_pri>+50.)
       vmax_sec = maxval(vmod_slice, MASK=vmod_slice-vmax_pri<-50.)

       if (abs(vmin_sec)<vmin_pri .or. abs(vmin_sec)>vmax_pri) vmin_sec = vmin_pri
       if (abs(vmax_sec)<vmin_pri .or. abs(vmax_sec)>vmax_pri) vmax_sec = vmax_pri

       vref_slice(1)    = vmin_pri
       vref_slice(nvrf) = vmax_pri
       nvrf_sec         = nvrf - 2 

       dvel=(vmax_sec-vmin_sec)/(nvrf_sec-1)
       do iref=1, nvrf_sec
          vref_slice(iref+1)=vmin_sec+(iref-1)*dvel
       end do
    end if
    stat = 0 
  end function

end module
