program main
  use base_struct_mod
  use base_hlxtyp_mod
  use hesinv_dipflt_mod
  use hesinv_dipcal_mod
  use base_ioutil_mod
  use bwi_dipcal_mod
  use bwi_diphlx_mod
  use sep

  implicit none

  type(hes_grid)                                    :: hgrid
  type(axis)                                        :: axchk, aychk, azchk, adip, aeps, aiter
  real, dimension(:)  , allocatable                 :: refl
  real, dimension(:)  , allocatable                 :: imag 
  real, dimension(:)  , allocatable                 :: mask
  real, dimension(:)  , allocatable                 :: dipr
  real, dimension(:)  , allocatable                 :: hwgt
  real, dimension(:)  , allocatable                 :: vareps
  real, dimension(:,:), allocatable                 :: mmov, rmov
  real, dimension(:)  , allocatable                 :: tmp1, tmp2, tmp3
  real, dimension(:)  , allocatable                 :: mod1, mod2, dat1, dat2

  character(len=256)                              :: refl_, imag_, hess_, mask_, hwgt_, dipx_, dipy_, dipr_, diag_, mmov_, rmov_
  real                                            :: const_dipx, const_dipy 
  integer                                         :: ndim, niter, index_dipx, index_dipy
  integer                                         :: nxpad_beg, nxpad_end
  integer                                         :: nypad_beg, nypad_end
  integer                                         :: nzpad_beg, nzpad_end
  integer                                         :: stat
  logical                                         :: expand, dptest, estdip

  type(helix)    , dimension(:)     , allocatable :: dfcx , dfcy
  real           , dimension(:)     , allocatable :: dfcx0, dfcy0
  integer        , dimension(:,:,:) , allocatable :: selx, sely

  integer                                         :: dipitr, na, idip, iter, i, repeat
  integer                                         :: rectx_dip, recty_dip, rectz_dip, nsmt_dip
  real                                            :: dipeps, regeps, sigma
  real                                            :: dip_min, dip_max, beta_max
  real            , dimension(:,:,:), allocatable :: dipx, dipy
  character(len=5)                                :: scheme
  logical                                         :: wntdip, smooth
  character(len=6)                                :: method


! Hybrid norm solver
  real                                            :: qntm,qntd
  integer                                         :: qt_period
  character(len=12)                               :: str_datnorm, str_modnorm

! L-BFGS solver
  character*60                                    :: task, csave
  logical                                         :: lsave(4), lbfgsb
  integer                                         :: n, m, iprint, isave(44)
  integer         , dimension(:), allocatable     :: nbd, iwa
  double precision                                :: f, factr, pgtol, dsave(29)
  double precision, dimension(:), allocatable     :: x, l, u, g, wa, gd, gm
  double precision                                :: fd, fm


  call sep_init(SOURCE)
  call getch_add_string("head=/dev/null")
  call getch_add_string("noheader=y")
  call set_no_putch()
  call sep_begin_prog()

  refl_ = "refl"
  imag_ = "imag"
  dipr_ = "dipr"
  mask_ = "mask"
  dipx_ = "dipx"
  dipy_ = "dipy"
  hwgt_ = "hwgt"

  call auxinout(imag_)
  call auxinout(refl_)
  call auxinout(mask_)
  call auxinout(dipr_)
!  call auxinout(hwgt_)

  call hinaxis(imag_, hgrid%ax_modl, 1) 
  call hinaxis(imag_, hgrid%ay_modl, 2)
  call hinaxis(imag_, hgrid%az_modl, 3)

  if (axchk%n/=hgrid%ax_modl%n) expand = .true.
  if (aychk%n/=hgrid%ay_modl%n) expand = .true.
  if (azchk%n/=hgrid%az_modl%n) expand = .true.

  call houaxis(refl_, hgrid%ax_modl, 1)
  call houaxis(refl_, hgrid%ay_modl, 2)
  call houaxis(refl_, hgrid%az_modl, 3)

  ndim = hgrid%ax_modl%n*hgrid%ay_modl%n*hgrid%az_modl%n

  write(0,*)"NDIM = ", ndim

  allocate(imag(ndim))
  allocate(refl(ndim))
  allocate(mask(ndim))
  allocate(hwgt(ndim))
  allocate(dipr(ndim))

  call sep_read(imag, imag_)
  call sep_read(mask, mask_)
!  call sep_read(hwgt, hwgt_)
  call sep_read(dipr, dipr_)
!    dip filters
  call from_param("rectx_dip", rectx_dip, 1       )
  call from_param("recty_dip", recty_dip, 1       )
  call from_param("rectz_dip", rectz_dip, 1       )
  call from_param("nsmt_dip" , nsmt_dip , 1       )
  call from_param("dip_min"  , dip_min  , 0.      )
  call from_param("dip_max"  , dip_max  , 0.      )
  call from_param("ndip"     , adip%n   , 1       )
  call from_param("dipitr"   , dipitr   , 100     )
  call from_param("dipeps"   , dipeps   , 0.001   )
  call from_param("na"       , na       , 5       )
  call from_param("scheme"   , scheme   , "UW2x3" )
  call from_param("wntdip"   , wntdip   , .false. )
  call from_param("smooth"   , smooth   , .true.  )
  call from_param("repeat"   , repeat   , 1       )
  call from_param("estdip"   , estdip   , .true.  )

  if (adip%n==1) then
     adip%o = dip_min
     adip%d = 1.
  else
     adip%o = dip_min
     adip%d = (dip_max-dip_min)/(adip%n-1)
  end if
  adip%n = adip%n + 1

! tabulate dip filters in x
! -------------------------
  if (hgrid%ax_modl%n>1) then
     allocate(dfcx(adip%n), dfcx0(adip%n))
     allocate(dipx(hgrid%ax_modl%n,hgrid%ay_modl%n,hgrid%az_modl%n))
     allocate(selx(hgrid%ax_modl%n,hgrid%ay_modl%n,hgrid%az_modl%n))
     do idip=1, adip%n
        call allocatehelix(dfcx(idip), 3*na)
     end do
     stat = bwi_diphlx_init(adip, dipeps, hgrid%ax_modl%d, hgrid%az_modl%d, hgrid%ax_modl%n, na, dipitr, scheme)
     stat = bwi_diphlx_oper(dfcx, dfcx0)
     stat = bwi_diphlx_clos()
  end if
! tabulate dip filters in y
! -------------------------
  if (hgrid%ay_modl%n>1) then
     allocate(dfcy(adip%n), dfcy0(adip%n))
     allocate(dipy(hgrid%ax_modl%n,hgrid%ay_modl%n,hgrid%az_modl%n))
     allocate(sely(hgrid%ax_modl%n,hgrid%ay_modl%n,hgrid%az_modl%n))
     do idip=1, adip%n
        call allocatehelix(dfcy(idip), 3*na)
     end do
     stat = bwi_diphlx_init(adip, dipeps, hgrid%ay_modl%d, hgrid%az_modl%d, hgrid%ay_modl%n, na, dipitr, scheme)
     stat = bwi_diphlx_oper(dfcy, dfcy0)
     stat = bwi_diphlx_clos()
  end if

  if (hgrid%ax_modl%n>1) then
     call auxinout(dipx_)
     call houaxis(dipx_, hgrid%ax_modl, 1)
     call houaxis(dipx_, hgrid%ay_modl, 2)
     call houaxis(dipx_, hgrid%az_modl, 3)
  end if
  if (hgrid%ay_modl%n>1) then
     call auxinout(dipy_)
     call houaxis(dipy_, hgrid%ax_modl, 1)
     call houaxis(dipy_, hgrid%ay_modl, 2)
     call houaxis(dipy_, hgrid%az_modl, 3)
  end if

! estimate the dips
  imag = imag*mask 
  if (estdip) then
     stat = hesinv_dipcal_init(hgrid%ax_modl,hgrid%ay_modl,hgrid%az_modl,adip,rectx_dip,recty_dip,rectz_dip,nsmt_dip)
     stat = hesinv_dipcal_oper(dipr,dipx,dipy,selx,sely)
     stat = hesinv_dipcal_clos()
     call sep_write(dipx, dipx_)
     call sep_write(dipy, dipy_)
  else
     call from_param("const_dipx", const_dipx, 0.)
     call from_param("const_dipy", const_dipy, 0.)
     index_dipx = 1.5+(const_dipx-adip%o)/adip%d
     index_dipy = 1.5+(const_dipy-adip%o)/adip%d
     selx = index_dipx
     sely = index_dipy
        write(0,*) const_dipx,index_dipx
        write(0,*) index_dipy
  end if

  do i=1, repeat
     write(0,*)"repeat= ", i
     stat = hesinv_dipflt_init(dfcx,dfcy,dfcx0,dfcy0,selx,sely,hgrid%ax_modl,hgrid%ay_modl,hgrid%az_modl,smooth)
     stat = hesinv_dipflt_oper(.true., .false., refl, imag)
     stat = hesinv_dipflt_clos()
     imag = refl
!     if (estdip) then
!        stat = hesinv_dipcal_init(hgrid%ax_modl,hgrid%ay_modl,hgrid%az_modl,adip,rectx_dip,recty_dip,rectz_dip,nsmt_dip)
!        stat = hesinv_dipcal_oper(imag,dipx,dipy,selx,sely)
!        stat = hesinv_dipcal_clos()
!        call sseek(dipx_, 0, 0)
!        call sseek(dipy_, 0, 0)
!        call sep_write(dipx, dipx_)
!        call sep_write(dipy, dipy_)
!     end if
  end do


  call sep_write(refl, refl_)

  deallocate(imag, refl, mask, hwgt)
end program
