module hesinv_dipcal_mod
  use bwi_dipcal_mod
  implicit none

  type(axis)                           , private :: ax, ay, az, adip
  integer                              , private :: rectx, recty, rectz, nsmt 
contains

  integer function hesinv_dipcal_init(ax_in, ay_in, az_in, adip_in, rectx_in, recty_in, rectz_in, nsmt_in) result(stat)
    type(axis)                        :: ax_in, ay_in, az_in, adip_in
    integer                           :: rectx_in, recty_in, rectz_in, nsmt_in
    stat = 1
    ax = ax_in
    ay = ay_in
    az = az_in
    adip = adip_in
    rectx = rectx_in
    recty = recty_in
    rectz = rectz_in
    nsmt  = nsmt_in
    stat = 0
  end function

  integer function hesinv_dipcal_clos() result(stat)
    stat = 1
    stat = 0
  end function

  integer function hesinv_dipcal_oper(imag, dipx, dipy, selx, sely) result(stat)
    real   , dimension(:)     :: imag
    real   , dimension(:,:,:) :: dipx, dipy
    integer, dimension(:,:,:) :: selx, sely
    stat = 1
    call hesinv_dipcal(imag, dipx, dipy, selx, sely)
    stat = 0
  end function

  subroutine hesinv_dipcal(imag, dipx, dipy, selx, sely)
    real   , dimension(ax%n,ay%n,az%n) :: imag, dipx, dipy
    integer, dimension(ax%n,ay%n,az%n) :: selx, sely

    integer :: ix, iy, iz, iselx, isely, stat
    integer :: nrx, nry, nrz, nst

    nrx = max(1, rectx)
    nry = max(1, recty)
    nrz = max(1, rectz)
    nrx = min(nrx, ax%n)
    nry = min(nry, ay%n)
    nrz = min(nrz, az%n)
    nst = max(1, nsmt)

!   estimate the dip field in x
!   ---------------------------
    if (ax%n>1) then
       stat = bwi_dipcal_init(ax,az,rectx,rectz,nsmt)
       do iy=1, ay%n
          stat = bwi_dipcal_oper(imag(:,iy,:), dipx(:,iy,:))
       end do
       stat = bwi_dipcal_clos()
       stat = base_tris3d_oper(dipx, nrx, nry, nrz, nst)
       selx = adip%n
       do iz=1, az%n
       do iy=1, ay%n
       do ix=1, ax%n
          iselx = 1.5 + (dipx(ix,iy,iz)-adip%o)/adip%d
          if (iselx<1 .or. iselx>adip%n-1) cycle
          selx(ix,iy,iz) = iselx
       end do
       end do
       end do
    end if

!   estimate the dip field in x
!   ---------------------------
    if (ay%n>1) then
       stat = bwi_dipcal_init(ay,az,recty,rectz,nsmt)
       do ix=1, ax%n
          stat = bwi_dipcal_oper(imag(ix,:,:), dipy(ix,:,:))
       end do
       stat = bwi_dipcal_clos()
       stat = base_tris3d_oper(dipy, nrx, nry, nrz, nst)
       sely = adip%n
       do iz=1, az%n
       do iy=1, ay%n
       do ix=1, ax%n
          isely = 1.5 + (dipy(ix,iy,iz)-adip%o)/adip%d
          if (isely<1 .or. isely>adip%n-1) cycle
          sely(ix,iy,iz) = isely
       end do
       end do
       end do
    end if


    return
  end subroutine

end module
