program main
  use sep
  use base_struct_mod
  use base_ioutil_mod
  use base_hlxtyp_mod
  use bwi_poldiv_mod
  use bwi_diphlx_mod
  use bwi_dipcal_mod

  implicit none

  type(helix), dimension(:), allocatable :: fltx, flty
  real       , dimension(:), allocatable :: fltx0, flty0

  type(axis)                             :: ax, ay, az
  type(axis)                             :: adip
 
  real                                   :: dip_min, dip_max, eps, PI
  integer                                :: nsmt, niter, rectx, recty, rectz, na 
  integer                                :: idip, jdip, iselx, isely, stat, ix, iy, iz

  real   , dimension(:,:,:), allocatable :: f, g, t, r, dipx, dipy
  integer, dimension(:,:,:), allocatable :: selx, sely
  logical                                :: dipin
  character(len=5)                       :: scheme

  PI = 3.14159265359

  call sep_init(SOURCE)
  call getch_add_string("head=/dev/null")
  call getch_add_string("noheader=y")
  call set_no_putch()
  call sep_begin_prog()

  call from_param("rectx"  , rectx  , 1      )
  call from_param("recty"  , recty  , 1      )
  call from_param("rectz"  , rectz  , 1      )
  call from_param("nsmt"   , nsmt   , 1      )
  call from_param("dip_min", dip_min, 0.     )
  call from_param("dip_max", dip_max, 0.     )
  call from_param("ndip"   , adip%n , 1      )
  call from_param("niter"  , niter  , 20     )
  call from_param("na"     , na     , 3      )
  call from_param("eps"    , eps    , 0.01   )
  call from_param("dipin"  , dipin  , .false.)
  call from_param("scheme" , scheme , "UW2x3")

  call auxinout("f")
  call auxinout("g")
  call auxinout("dipx")
  call auxinout("dipy")
  call auxinout("ref")

  call hinaxis("f", ax, 1)
  call hinaxis("f", ay, 2)
  call hinaxis("f", az, 3)

  call houaxis("g", ax, 1)
  call houaxis("g", ay, 2)
  call houaxis("g", az, 3)

  if (adip%n==1) then
     adip%o = dip_min
     adip%d = 1.
  else
     adip%o = dip_min
     adip%d = (dip_max-dip_min)/(adip%n-1)
  end if

! add a laplacian
  adip%n = adip%n + 1

  allocate(f(ax%n,ay%n,az%n), g(ax%n,ay%n,az%n), t(ax%n,ay%n,az%n))
  allocate(r(ax%n,ay%n,az%n), dipx(ax%n,ay%n,az%n), dipy(ax%n,ay%n,az%n))
  allocate(selx(ax%n,ay%n,az%n), sely(ax%n,ay%n,az%n))
  allocate(fltx(adip%n), fltx0(adip%n))
  allocate(flty(adip%n), flty0(adip%n))

  call sep_read(f, "f"  )
  call sep_read(r, "ref")

! allocate the dip factors
! ------------------------
  do idip=1, adip%n
     call allocatehelix(fltx(idip), 3*na)
     call allocatehelix(flty(idip), 3*na)
  end do

  if (ax%n>1) then
!    tabulate the dip factors in x
!    -----------------------------
     stat = bwi_diphlx_init(adip, eps, ax%d, az%d, ax%n, na, niter, scheme) 
     stat = bwi_diphlx_oper(fltx, fltx0)
     stat = bwi_diphlx_clos()
  end if

  if (ay%n>1) then 
!    tabulate in y
!    -------------
     stat = bwi_diphlx_init(adip, eps, ay%d, az%d, ay%n, na, niter, scheme)
     stat = bwi_diphlx_oper(flty, flty0)
     stat = bwi_diphlx_clos()
  end if
 
  if (dipin) then
     call sep_read(dipx, "dipx")
     dipx = atan(dipx)*180/PI
  else
     call houaxis("dipx", ax, 1)
     call houaxis("dipx", ay, 2)
     call houaxis("dipx", az, 3)

     call houaxis("dipy", ax, 1)
     call houaxis("dipy", ay, 2)
     call houaxis("dipy", az, 3)

write(0,*)"ax = ", ax
write(0,*)"ay = ", ay
write(0,*)"az = ", az

!    estiamte the dips in x
!    ----------------------
     if (ax%n>1) then
        stat = bwi_dipcal_init(ax, az, rectx, rectz, nsmt)
        do iy=1, ay%n
           stat = bwi_dipcal_oper(r(:,iy,:), dipx(:,iy,:))
        end do
        stat = bwi_dipcal_clos()
     end if
!    estimate the dips in y
!    ----------------------
     if (ay%n>1) then
        stat = bwi_dipcal_init(ay, az, recty, rectz, nsmt)
        do ix=1, ax%n
           stat = bwi_dipcal_oper(r(ix,:,:), dipy(ix,:,:))
        end do 
        stat = bwi_dipcal_clos()
     end if

write(0,*)"dipx:", maxval(abs(dipx)), minval(abs(dipx))
write(0,*)"dipy:", maxval(abs(dipy)), minval(abs(dipy))

     if (ax%n>1 .and. ay%n>1 .and. az%n>1) then
        stat = base_tris3d_oper(dipx,rectx,recty,rectz,nsmt)
        stat = base_tris3d_oper(dipy,rectx,recty,rectz,nsmt)
     end if

     call sep_write(dipx, "dipx")
     call sep_write(dipy, "dipy")
  end if

!v(1:ax%n/2,:,:) = -80
!v(ax%n/2:,:,:)  = 80


! build the selector matrix
! -------------------------
  do iz=1, az%n
  do iy=1, ay%n
  do ix=1, ax%n
     iselx = 1.5 + (dipx(ix,iy,iz)-adip%o)/adip%d
     isely = 1.5 + (dipy(ix,iy,iz)-adip%o)/adip%d
     selx(ix,iy,iz) = min(max(iselx,1),adip%n)
     sely(ix,iy,iz) = min(max(isely,1),adip%n) 
  end do
  end do
  end do 

write(0,*)"adip", adip
!call random_number(f)
!f = f*2 - 1

!selx = 15
!sely = 10

!f = 0.
!f(ax%n/2,ay%n/2,az%n/2) = 1.

! A = Ax Ay

! apply dip smoothing in inline direction Ax^{-T}
! -----------------------------------------------
  if (ax%n>1) then
     stat = bwi_poldiv_init(fltx, fltx0, ax%n, az%n)
     do iy=1, ay%n
        stat = bwi_poldiv_oper(.true.,.false.,t(:,iy,:),f(:,iy,:),selx(:,iy,:))
     end do
     stat = bwi_poldiv_clos()
  end if

! apply dip smoothing in crossline direction Ay^{-T}
! --------------------------------------------------
  if (ay%n>1) then
     stat = bwi_poldiv_init(flty, flty0, ay%n, az%n)
     do ix=1, ax%n
        stat = bwi_poldiv_oper(.true. ,.false.,g(ix,:,:),t(ix,:,:),sely(ix,:,:))
        stat = bwi_poldiv_oper(.false.,.false.,g(ix,:,:),t(ix,:,:),sely(ix,:,:))
     end do
     stat = bwi_poldiv_clos()
  end if

  if (ax%n>1) then
     stat = bwi_poldiv_init(fltx, fltx0, ax%n, az%n)
     do iy=1, ay%n
        stat = bwi_poldiv_oper(.false.,.false.,t(:,iy,:),g(:,iy,:),selx(:,iy,:))
     end do
     stat = bwi_poldiv_clos()
  end if

!  selc = adip%n
!  f = g
!  stat = bwi_poldiv_oper(.false.,.false.,f,t)
!  stat = bwi_poldiv_oper(.true. ,.false.,g,t) 

!stat = bwi_hlxcon_init(dipfac, dipfac0, selc, ax%n, ay%n, az%n)
!stat = bwi_hlxcon_oper(.true.,.false.,t,f)

!stat = bwi_poldiv_init(dipfac, dipfac0, selc, ax%n, ay%n, az%n)
!stat = bwi_poldiv_oper(.true.,.false.,g,t)

  call sep_write(g, "g")
  call exit()
end program
